#' Retrieve Annotation Details from CCS Sections
#' 
#' Retrieve the annotation information from the Collision Cross Section 
#' section of data sources, per page. As this requires
#' retrieving the entire JSON file (up to 1000 entries), this can take 
#' some time (hence large default timeout). Based on getPcAnno.Metabolism.
#' Thanks to Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcAnno.CCS(source, heading, page=1, file_name="", timeout=100)
#' 
#' @param source Data source name of the annotations data, e.g. \code{"CCSbase"}
#' @param heading Annotation category desired, e.g. \code{"Collision Cross Section"}
#' @param page Page number to retrieve. Use \link{getPcAnno.TotalPages} to 
#' determine page numbers available
#' @param file_name If empty, \code{"heading_source_pageX.csv"} is created, 
#' overwriting any existing file of the same name in the directory. 
#' @param timeout The timeout, in seconds. Should be generous as these are large files
#' @return file_name of a saved multi-column CSV containing Source Name, Source ID, 
#' Source Chemical Name, Reference text, Description text and 
#' pipe-separated CIDs of the metabolites (\code{tp_cids}).
#' 
#' @details 
#' This function may work for other annotation headings, but has not been 
#' tested so far.
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' PubChem Data Sources (with annotations):
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/#type=Annotations}
#' 
#' @seealso \code{\link{getPcAnno.TotalPages}}
#' 
#' @examples
#' # This retrieves only the first page
#' getPcAnno.CCS(source = 'CCSbase',heading = 'Collision Cross Section')
#' # This retrieves only the first page
#' getPcAnno.CCS(source = 'NORMAN Suspect List Exchange', 
#' heading = 'Collision Cross Section',file_name='SLE_CCS_test_p1.csv',page=1)
#' # This will retrieve all (in single files)
#' n_pages <- getPcAnno.TotalPages('CCSbase','Collision Cross Section')
#' for (i in 1:n_pages) {
#'   getPcAnno.CCS(source = 'CCSbase',heading = 'Collision Cross Section', page=i)
#' }
#' # merge together
#' CCSbase_data <- read.csv("CCS_CCSbase_page1.csv")
#' CCSbase_data2 <- read.csv("CCS_CCSbase_page2.csv")
#' CCSbase_data_all <- merge(CCSbase_data,CCSbase_data2,all.x=TRUE,all.y=TRUE)
#' 
#' for (i in 3:n_pages) {
#'     file_name <- paste0("CCS_CCSbase_page",i,".csv")
#'     CCSbase_data2 <- read.csv(file_name)
#'     CCSbase_data_all <- merge(CCSbase_data_all,CCSbase_data2,all.x=TRUE,all.y=TRUE)
#'     }
#' 
#' @export
getPcAnno.CCS <- function(source, heading, page=1, file_name="", timeout=100) {
  # set file_name if not set...
  #file_name <- "CCSbase_CCS_test_p1.csv"
  if (nchar(file_name) < 4) {
    file_name <- paste0(sub("/","_",heading), "_", source, "_page", page,".csv")
  }
  # set URL
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source='
  #source <- 'NORMAN Suspect List Exchange'
  #source <- 'CCSbase'
  midURL <- '&heading='
  #heading <- 'Collision Cross Section'
  #endURL <- '&heading_type=Compound&page=1&response_type=display'
  endURL <- '&heading_type=Compound&page='
  #page <- 1
  url <- paste0(baseURL, source, midURL, heading, endURL, page)
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    #data <- getURL(URLencode(url),timeout=timeout),
    {
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  r <- fromJSON(data)
  
  if(!is.null(r$Fault)) {
    return(NA)
  }
  
  n_annos <- length(r$Annotations$Annotation) #1000 per page
  source_name <- vector(mode="character",length=n_annos)
  source_ID <- vector(mode="character",length=n_annos)
  #source_chemical_name <- vector(mode="character",length=n_annos)
  #desc_text <- vector(mode="character",length=n_annos)
  #ref_text <- vector(mode="character",length=n_annos)
  data_ref <- vector(mode="character",length=n_annos)
  data_CCS <- vector(mode="character",length=n_annos)
  linked_cid <- vector(mode="character",length=n_annos)
  #tp_cids <- vector(mode="character",length=n_annos)
  n <- 1
  #i <- 3
  
  for (i in 1:n_annos) {
  #for (i in 1:3) {
    l1 <- r$Annotations$Annotation[[i]]
    #l2 <- l1$Data[[1]]
    
    n_refs <- length(l1$Data)
    # for each reference ... 
    for (j in 1:n_refs) {
      l2 <- l1$Data[[j]]
      n_vals <- length(l2$Value$StringWithMarkup)
      
      for (k in 1:n_vals) {
        #print source name
        source_name[n] <- l1$SourceName # Source Name
        source_ID[n] <- l1$SourceID #ID from source
        #source_chemical_name[n] <- l1$Name # name of chemical from source
        data_ref[n] <- l2$Reference
        data_CCS[n] <- sub(",",";",l2$Value$StringWithMarkup[[k]]$String,fixed=T)
        # if (length(l1$LinkedRecords$CID)>1) {
        #   linked_cid[n] <- l1$LinkedRecords$CID
        # } else {
        #   linked_cid[n] <- NA
        # }
        if (source == "CCSbase") {
          linked_cid[n] <- l1$LinkedRecords[2]
        } else {
          linked_cid[n] <- l1$LinkedRecords
        }
        #linked_cid[n] <- l1$LinkedRecords
        n <- n+1
      }
    }
  }
  anno_data <- cbind(source_name, source_ID, data_ref, data_CCS, linked_cid)
  #anno_data <- cbind(source_name, source_ID, source_chemical_name, data_ref, data_CCS, linked_cid)
  write.csv(anno_data, file_name,row.names = F,quote = T)
  return(file_name)
}

# this is a wrapper for the above over all pages
getPcAnno.allCCS <- function(source, heading, base_file_name="", timeout=100) {
  #set file name if not given
  if (nchar(base_file_name) < 4) {
    base_file_name <- paste0(sub("/","_",heading), "_", source)
  }
  #determine the number of pages
  n_pages <- getPcAnno.TotalPages(source, heading, timeout)
  # for each page, retrieve data
  for (i in 1:n_pages) {
    file_name <- paste0(base_file_name,"_page",i,".csv")
    getPcAnno.CCS(source = source,heading = heading, file_name = file_name, page=i)
  }
  # merge together
  CCS_data <- read.csv(paste0(base_file_name,"_page1.csv"),stringsAsFactors = F)
  CCS_data2 <- read.csv(paste0(base_file_name,"_page2.csv"),stringsAsFactors = F)
  CCS_data_all <- merge(CCS_data,CCS_data2,all=TRUE)

  if (n_pages > 3) {
    for (i in 3:n_pages) {
      file_name <- paste0(base_file_name,"_page",i,".csv")
      CCS_data2 <- read.csv(file_name)
      CCS_data_all <- merge(CCS_data_all,CCS_data2,all=TRUE)
    }
  }
  
  write.csv(CCS_data_all, paste0(base_file_name,"_all.csv"),row.names = F,quote = T)
  return(file_name)
  
}

