## Get Concept IDs and preferred names from PubChem 


library(httr)
library(rjson)



#' Retrieve Concept ID, Preferred Name, CIDs from PubChem
#' 
#' Retrieves the Concept ID, Preferred Name and any 
#' associated CIDs from PubChem using PUG REST.
#' Default behaviour is to retrieve by name. 
#' 
#' Thanks to Paul Thiessen from PubChem team. 
#' 
#' @usage getPCID.concepts(concept_name, timeout=30)
#' 
#' @param concept_name string of the name to search
#' @param timeout The timeout, in seconds. For records with many entries, 
#' this may need to be increased - e.g. if only one \code{NA} is returned. 
#' @return A list containing the concept ID, preferred name and any associated CIDs
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem search: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' PubChem PUG REST:
#' \url{https://pubchemdocs.ncbi.nlm.nih.gov/pug-rest}
#' 
#' @examples
#' getPCID.concepts("dimethylbenzol")
#' getPCID.concepts("ethanoate")
#' getPCID.concepts("Polychlorinated biphenyls")
#' #a decoy search entry (returns NAs)
#' getPCID.concepts("blah")
#' # an example not yet available
#' getPCID.concepts("Alkanes, C10-13, chloro")
#' 
#' @export
getPCID.concepts <- function(concept_name,timeout=30)
{
  baseURL <- "https://pubchem.ncbi.nlm.nih.gov/rest/pug/concepts/name/JSON?name="
  url <- paste0(baseURL, concept_name)
  #https://pubchem.ncbi.nlm.nih.gov/rest/pug/concepts/name/JSON?name=dimethylbenzol
  
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    #url_data <- getURL(URLencode(url),timeout=timeout),
    {
      res <- GET(URLencode(url))
      url_data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  # This happens if the PCID is not found:
  r <- fromJSON(url_data)
  
  if(!is.null(r$Fault)) {
    ConceptsAndCIDs <- list()
    ConceptsAndCIDs[['ConceptName']] <- NA
    ConceptsAndCIDs[['ConceptID']] <- NA
    ConceptsAndCIDs[['PreferredCID']] <- NA
    ConceptsAndCIDs[['Description']] <- NA
    return(ConceptsAndCIDs)
  } else {
    ConceptName <- r$ConceptsAndCIDs$Concept[[1]]$ConceptName
    ConceptID <- r$ConceptsAndCIDs$Concept[[1]]$ConceptID
    PreferredCID <- r$ConceptsAndCIDs$Concept[[1]]$PreferredCID
    Description <- r$ConceptsAndCIDs$Concept[[1]]$Description
    #Description <- paste(r$ConceptsAndCIDs$Concept[[1]]$Description,"|")
    
    if (is.null(ConceptName)) {
      ConceptName <- NA
    }
    if (is.null(ConceptID)) {
      ConceptID <- NA
    }
    if (is.null(PreferredCID)) {
      PreferredCID <- NA
    }
    if (is.null(Description)) {
      Description <- NA
    }
    
    ConceptsAndCIDs <- list()
    ConceptsAndCIDs[['ConceptName']] <- ConceptName
    ConceptsAndCIDs[['ConceptID']] <- ConceptID
    ConceptsAndCIDs[['PreferredCID']] <- PreferredCID
    ConceptsAndCIDs[['Description']] <- Description
    return(ConceptsAndCIDs)
    
  }
}
