# Functions to extract data from PubChem Classification Trees
# E. Schymanski, 30/10/2020 - 12/11/2020
# Plus Evan Bolton and Paul Thiessen
# Functions ported from hid_tree_JSON.R

##### Contents ######
# getPcHidTree - returns info about contents of one entire classification tree
# getPcHidTreeList - returns basic information about all classification trees
# getPcHidTreeInfo - retrieves basic info about one classification tree (for 
#                    getPcHidTreeList)
# getPcListCache - gets a list of IDs from cache key (see getPcHidCacheID)
# getPcHidCacheID - returns a cache ID to get a ID list given a hid and hnid
# getKWInfo - get basic info about tree entries matching keyword regex
# getKWInfoCIDs - get the CIDs corresponding to keyword
# createKwCIDfile - merge all KW CID files into one big file


#### Future Info #####
## Tree id types in the near future.
## Current functions are focusing on CID / Compound
# id_types <- c("Substance",
#               "Compound",
#               "BioAssay",
#               "Patent",
#               "GI",
#               "PubMed",
#               "DOI",
#               "Gene",
#               "Protein",
#               "Taxonomy",
#               "Pathway",
#               "Disease"
# )

#### The functions #####

## this function returns a CSV file containing information about the contents
## of one entire tree, where query = hid number
## depth=2 can be used to specify how deep; this is definitely needed
## for large trees like hid=1 and 2. 
getPcHidTree <- function(query,depth=2)
{
  baseURL <- "https://pubchem.ncbi.nlm.nih.gov/classification/cgi/classifications.fcgi?"
  if (!is.na(depth)) {
    url <- paste0(baseURL, "format=json","&depth=",depth,"&hid=", query)
  } else {
    url <- paste0(baseURL, "format=json&hid=", query)
  }
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    {#		data <- getURL(URLencode(url),timeout=8),
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  # get the data into useful format
  r <- fromJSON(data)
  
  if(!is.null(r$Fault))
    return(NA)
  
  dat <- r$Hierarchies$Hierarchy[[1]]
  hid <- dat$HID
  source_name <- dat$SourceName
  sourceID <- dat$SourceID
  hnid <- dat$Information$HNID
  # to extract as numbers (but this won't help as they are listed as node_XXX)
  # childIDs <- as.numeric(sub("node_","",dat$Information$ChildID))
  childIDs <- dat$Information$ChildID
  # this is assuming CIDs are always first and SIDs second
  #otherwise this would have to test the "Type"
  #types <- dat$Information$Counts
  i_cid <- NA
  for (i in 1:length(dat$Information$Counts)) {
    type <- dat$Information$Counts[[i]]$Type
    if (type=="CID" || type=="Compound") {
      i_cid <- i
    }
  }
  if (!is.na(i_cid)) {
    n_cids <- dat$Information$Counts[[i_cid]]$Count
  }
  # look further below for code that still needs updating ... 
  
  # n_cids <- dat$Information$Counts[[1]]$Count
  # if (length(dat$Information$Counts)>1) {
  #   # then maybe we have SIDs too
  #   n_sids <- dat$Information$Counts[[2]]$Count
  # } else {
  #   n_sids <- 0
  # }
  
  
  n_nodes <- length(dat$Node)
  
  #nodeNames <- which(unlist(lapply(dat$Node, function(i) !is.null(i$Information$Name))))
  nodeNames <- unlist(lapply(dat$Node, function(i) i$Information$Name))
  # for KEGG we seem to have Description not Names
  nodeDesc <- unlist(lapply(dat$Node, function(i) i$Information$Description))
  nodeHNID <- unlist(lapply(dat$Node, function(i) i$Information$HNID))
  # now test if nodeNames is NULL, if so, replace with Desc
  if (is.null(nodeNames) && !is.null(nodeDesc)) {
    nodeNames <- nodeDesc
  } else if (is.null(nodeNames) && is.null(nodeDesc)) {
    warn_msg <- paste0("Neither Name nor Description available for Nodes for ",
                       "hid ",query)
    warning(warn_msg)
    nodeNames <- vector(mode="character",length=length(nodeHNID))
  }
  # not all entries have CIDs
  #node_info <- dat$Node
  i_node_nCIDs <- vector(mode="numeric",length=n_nodes)
  node_nCIDs <- vector(mode="numeric",length=n_nodes)
  for (i in 1:n_nodes) {
    node_data <- dat$Node[[i]]
    i_node_cid <- NA
    # test if there are any counts ... if so look for CIDs ...
    if (!is.null(node_data$Information$Counts)) {
      for (j in 1:length(node_data$Information$Counts)) {
        type <- node_data$Information$Counts[[j]]$Type
        if (type=="CID" || type=="Compound") {
          i_node_cid <- j
        }
      }
    }
    # if there's CIDs, save the count
    if (!is.na(i_node_cid)) {
      node_nCID <- node_data$Information$Counts[[i_node_cid]]$Count
      node_nCIDs[i] <- node_nCID
    } #otherwise remains at 0
    
    i_node_nCIDs[i] <- i_node_cid
    #node_nCIDs[i] <- node_nCID
  }
  # i_node_nCIDs <- which(unlist(lapply(dat$Node, function(i) !is.null(i$Information$Counts[[1]]$Count))))
  # node_nCIDs <- unlist(lapply(dat$Node, function(i) i$Information$Counts[[1]]$Count))
  
  nodeIDs <- unlist(lapply(dat$Node, function(i) i$NodeID))
  parentIDs <- unlist(lapply(dat$Node, function(i) i$ParentID))
  #sapply(NodeText, function(x)dat$Node[[x]]$Information$Name)
  #dat$Node[[]]
  
  #  hid_info <- cbind(nodeNames,nodeHNID,node_nCIDs,nodeIDs,parentIDs)
  # hid_info <- as.data.frame(cbind(nodeNames,nodeHNID,nodeIDs,parentIDs))
  hid_info <- as.data.frame(cbind(nodeNames,nodeHNID,nodeIDs,parentIDs,node_nCIDs))
  
  # now add in global data
  hid_info$hid <- hid
  hid_info$SourceName <- source_name
  hid_info$SourceID <- sourceID
  hid_info$HNID <- hnid
  hid_info$nCIDs <- n_cids
  # # now add nCIDs
  # hid_info$node_nCIDs <- 0
  # 
  # for (i in 1:length(i_node_nCIDs)) {
  #   index <- i_node_nCIDs[i]
  #   hid_info$node_nCIDs[index] <- node_nCIDs[i]
  # }
  
  #do some reordering
#  hid_info <- hid_info[,c(5:8,1:4,9)]
  hid_info <- hid_info[,c(6:10,1:5)]
  
  if (!is.na(depth)) {
    export_name <- paste0("classification_tree_hid",query,"_depth",depth,"_export.csv")
  } else {
    export_name <- paste0("classification_tree_hid",query,"_export.csv")
  }
  #export_name <- paste0("classification_tree_hid",query,"_export.csv")
  write.csv(hid_info,export_name,row.names = F)
  
  if(is.null(hid_info)){
    return(NA)
  } else{
    return(export_name)
  }
  
}


## This function returns a csv file (name returned) containing
## basic information about all classification trees in the specified range
## The csv output contains this basic info, NA is the hid does not exist
## depth is needed especially for large trees (hid=1, 2)
getPcHidTreeList <- function(hid_start=1,hid_stop=110,depth=2) {
  # this function checks the URLs and returns a file with listing
  # NOTE: HIDs 1 and 2 are too slow and are done manually
  # HID1 takes ages ... HID 2 returns bad gateway(?!)
  # NB this is fixed with the "depth=2" parameter!
  queries <- hid_start:hid_stop
  # set up vectors
  #n_hids <- (hid_stop-hid_start)+1
  n_hids <- length(queries)
  hids <- vector("numeric",length=n_hids)
  source_names <- vector("character",length=n_hids)
  sourceIDs <- vector("character",length=n_hids)
  tree_names <- vector("character",length=n_hids)
  hnids <- vector("numeric",length=n_hids)
  hasCIDs <- vector("logical",length=n_hids)
  
  # run loop
  for (i in 1:n_hids) {
    query <- queries[i]
    tree_info <- getPcHidTreeInfo(query,depth=depth)
    
    # if (query==1) {
    #   tree_info <- list()
    #   tree_info[['HID']] <- 1
    #   tree_info[['SourceName']] <- "MeSH"
    #   tree_info[['SourceID']] <- "DescTree"
    #   tree_info[['TreeName']] <- "MeSH Tree"
    #   tree_info[['HNID']] <- 1
    #   tree_info[['hasCIDs']] <- TRUE
    #   
    # } else if (query==2) {
    #   tree_info <- list()
    #   tree_info[['HID']] <- 2
    #   tree_info[['SourceName']] <- "ChEBI"
    #   tree_info[['SourceID']] <- NA
    #   tree_info[['TreeName']] <- NA
    #   tree_info[['HNID']] <- NA
    #   tree_info[['hasCIDs']] <- TRUE
    #   
    # } else {
    #   tree_info <- getPcHidTreeInfo(query,depth=depth)
    # }
    
    if (!is.na(tree_info[[1]])) {
      hids[i] <- tree_info$HID
      source_names[i] <- tree_info$SourceName
      sourceIDs[i] <- tree_info$SourceID
      tree_names[i] <- tree_info$TreeName
      hnids[i] <- tree_info$HNID
      if(length(tree_info$hasCIDs)>0) {
        hasCIDs[i] <- tree_info$hasCIDs
      } else {
        hasCIDs[i] <- FALSE
      }
      
    } else {
      hids[i] <- query
      source_names[i] <- NA
      sourceIDs[i] <- NA
      tree_names[i] <- NA
      hnids[i] <- NA
      hasCIDs[i] <- NA
    }
  }
  # now merge together
  tree_summary <- cbind(hids,source_names,sourceIDs,tree_names,hnids,hasCIDs)
  colnames(tree_summary) <- c("HID","SourceName","SourceID","TreeName",
                              "HNID","hasCIDs")
  hid_file_name <- paste0("HID_Tree_Summary_",hid_start,"_to_",hid_stop,".csv")
  write.csv(tree_summary,hid_file_name,row.names = F)
  return(hid_file_name)
  
}


## This function returns basic information about a single classification 
## tree, returning the output as a list of this basic info
## Feeds into getPcHidTreeList
## depth is needed especially for large trees (hid=1, 2)
# tree_info[['HID']] <- hid
# tree_info[['SourceName']] <- source_name
# tree_info[['SourceID']] <- sourceID
# tree_info[['TreeName']] <- tree_name
# tree_info[['HNID']] <- hnid
# tree_info[['hasCIDs']] <- hasCIDs
getPcHidTreeInfo <- function(query, depth=2)
{
  baseURL <- "https://pubchem.ncbi.nlm.nih.gov/classification/cgi/classifications.fcgi?"
  if (!is.na(depth)) {
    url <- paste0(baseURL, "format=json","&depth=",depth,"&hid=", query)
  } else {
    url <- paste0(baseURL, "format=json&hid=", query)
  }
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    {#		data <- getURL(URLencode(url),timeout=8),
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  exception_check <- grep("Exception during execution: NCBI",data)
  
  if (length(exception_check)>0 && exception_check==1) {
    warn_string <- paste0("No data file for HID ",query)
    warning(warn_string)
    return(NA)
  }
  
  # Otherwise proceed
  r <- fromJSON(data)
  
  if(!is.null(r$Fault))
    return(NA)
  
  dat <- r$Hierarchies$Hierarchy[[1]]
  hid <- dat$HID
  if (!(as.character(hid)==as.character(query))){
    warn_string <- paste0("Query hid ",query," and retrieved hid ",hid," don't match")
    warning(warn_string)
  }
  source_name <- dat$SourceName
  sourceID <- dat$SourceID
  tree_name <- dat$Information$Name
  hnid <- dat$Information$HNID
#  hasCIDs <- which(dat$Information$Counts[[1]]$Type == "CID")>0
#  hasCIDs <- which(dat$Information$Counts[[1]]$Type == "Compound")>0
  hasCIDs <- grep("Compound",unlist(dat$Information$Counts))>0
  
  tree_info <- list()
  tree_info[['HID']] <- hid
  tree_info[['SourceName']] <- source_name
  tree_info[['SourceID']] <- sourceID
  tree_info[['TreeName']] <- tree_name
  tree_info[['HNID']] <- hnid
  tree_info[['hasCIDs']] <- hasCIDs
  
  return(tree_info)
  
}



## This function uses a cache ID from getPcHidCacheID to get a CID list
getPcListCache <- function(cache_key) {
  baseURL <- "https://pubchem.ncbi.nlm.nih.gov/list_gateway/list_gateway.cgi?"
  #  if (!is.na(depth)) {
  url <- paste0(baseURL, "format=json","&action=cache_get","&cache_key=", cache_key)
  #  } else {
  #    url <- paste0(baseURL, "format=json&hid=", query)
  #  }
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    {#		data <- getURL(URLencode(url),timeout=8),
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  # Otherwise proceed
  r <- fromJSON(data)
  
  if(!is.null(r$Fault))
    return(NA)
  
  # all this information comes with the call
  id_type <- r$ID_Container$identifier_type
  number_ids <- r$ID_Container$number_of_ids
  # but only return this for now
  ids <- r$ID_Container$ids$uint32
  return(ids)
  
}




# # hid_url <- "https://pubchem.ncbi.nlm.nih.gov/classification/cgi/classifications.fcgi?format=json&hid=101&hnid=4041191"
# # 
# # hid <- "101"
# # hnid <- "4041191"
# # cache_uid <- "cid"
# getPcHidCacheID(101,4041191)
# 
# getPcListCache(getPcHidCacheID(101,4041191))
# getPcListCache(getPcHidCacheID(4,2291199))

## This function returns a cache ID to get a CID list given a hid and hnid
getPcHidCacheID <- function(hid,hnid,cache_uid="cid") {
  baseURL <- "https://pubchem.ncbi.nlm.nih.gov/classification/cgi/classifications.fcgi?"
  #  if (!is.na(depth)) {
  url <- paste0(baseURL, "format=json","&hid=",hid,"&hnid=", hnid,
                "&cache_uid_type=",cache_uid)
  #  } else {
  #    url <- paste0(baseURL, "format=json&hid=", query)
  #  }
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    {#		data <- getURL(URLencode(url),timeout=8),
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  exception_check <- grep("Exception during execution: ",data)
  
  if (length(exception_check)>0 && exception_check==1) {
    warn_string <- paste0("No data file for HID ",hid," and HNID ",hnid)
    warning(warn_string)
    return(NA)
  }
  
  # Otherwise proceed
  r <- fromJSON(data)
  
  if(!is.null(r$Fault))
    return(NA)
  
  cache_key <- r$Hierarchies$CacheKey
  return(cache_key)
  
}


## This function takes the tree info file, and the kw summary file
## and retrieves the keyword regex, to create info files for each 
## keyword ... saving into a subfolder ...  
# need tree info file, and keyword summary file
## returns the folder name containing the files. 
getKWInfo <- function(tree_info_file, kw_summary_file) {
  tree_info <- read.csv(tree_info_file,stringsAsFactors = F)
  kw_summary <- read.csv(kw_summary_file,stringsAsFactors = F)
  kw_info_dir <- paste0(getwd(),"/kw_info/")
  if (!dir.exists(kw_info_dir)) {
    dir.create(kw_info_dir)
  }
  # now, get the patterns
  #kw_patterns <- kw_summary$Keyword_RegEx
  
  for (j in 1:length(kw_summary$Keyword_RegEx)) {
    kw_pattern <- strsplit(kw_summary$Keyword_RegEx[j],"|",fixed=T)[[1]]
    
    for (i in 1:length(kw_pattern)) {
      temp_info <- tree_info[grep(kw_pattern[i],tree_info$nodeNames),]
      if (i == 1) {
        # this relies on the first entry always returning a pattern match
        kw_info <- temp_info
        #  } else if (!is.na(temp_info[1])) {
      } else if (dim(temp_info)[1]>0) {
        #kw_list <- c(temp_list,drug_list)
        kw_info <- merge(kw_info,temp_info,all=T)
      }
    }
    #length(kw_info$nodeNames)
    # add new column to set whether to include the node. 
    # Overwrite to FALSE when manual checking
    kw_info$includeNode <- TRUE
    # merging takes care of uniqueness
    kw_info_filename <- paste0(kw_info_dir,kw_summary$Keyword[j],"_keyword_summary.csv")
    write.csv(kw_info,kw_info_filename,row.names = F)
  }
  # return the file name
  print_msg <- paste0("Files printed into keyword info dir")
  print(print_msg)
  #  return(kw_info_filename)
  return(kw_info_dir)
  
}


## This function takes a keyword info file and retrieves the CIDs
## saving keyword CID mapping into a subfolder. 
# input: kw_info_list
# output: kw cid file
# if addCIDs=TRUE this overwrites the source file with the same info plus CIDs
# NOTE this function runs quite long as it runs many queries
# add a print to give updates at n=10?
getKWInfoCIDs <- function(kw_info_filename,addCIDs=FALSE) {
  kw <- strsplit(basename(kw_info_filename),"_")[[1]][1]
  kw_info <- read.csv(kw_info_filename,stringsAsFactors = F)
  # now subset according to includeNode=TRUE
  kw_info <- kw_info[which(kw_info$includeNode==TRUE),]
  kw_cid_dir <- paste0(getwd(),"/kw_cid_files/")
  
  if (!dir.exists(kw_cid_dir)) {
    dir.create(kw_cid_dir)
  }
  
  kw_info$nodeCIDs <- ""
  for (i in 1:length(kw_info$hid)) {
    hid <- kw_info$hid[i]
    hnid <- kw_info$Node[i]
    node_hnid <- kw_info$nodeHNID[i]
    cache_key <- getPcHidCacheID(hid,node_hnid)
    if (!is.na(cache_key)) {
      row_cids <- getPcListCache(cache_key)
      kw_info$nodeCIDs[i] <- paste(row_cids,collapse="|")
    } else {
      row_cids <- NULL
    }
    
    if (i == 1) {
      cids <- row_cids
    } else if (length(row_cids)>0) {
      cids <- c(row_cids,cids)
    }
  }
  #length(cids)
  unique_cids <- unique(cids)
  #length(unique_cids)
  
  # we can add an option to add CIDs into the kw_info_files
  if (addCIDs) {
    write.csv(kw_info,kw_info_filename,row.names = F)
  }
  
  #write.table(sort(unique_cids),"vitamin_unique_CIDs.txt",row.names = F,col.names=F)
  # Evan would like two col file: keyword cid and then merge many keyword files together
  kw_cid_filename <- paste0(kw_cid_dir,kw,"_uniqueCIDs_keyword.txt")
  # write.table(cbind(kw,sort(unique_cids)),"vitamin_unique_CIDs_keyword.txt",
  #             row.names = F,col.names=F,quote = F)
  #cbind is introducting numerical formatting issues ...
  # write.table(cbind(sort(unique_cids),kw),kw_cid_filename,sep="\t",
  #             row.names = F,col.names=F,quote = F)
  kw_df <- as.data.frame(sort(unique_cids))
  kw_df$cid_char <- formatC(format="fg",kw_df[,1])
  kw_df$kw <- kw
  write.table(kw_df[,2:3],kw_cid_filename,sep="\t",
              row.names = F,col.names=F,quote = F)
  return(kw_cid_filename)
}


# This takes a folder of all the keyword CID files from getKWInfoCIDs
# and merges contents into one big file just for Evan .. 
createKwCIDfile <- function(kw_cid_dir,kw_dir,all_kw_cid_filename="") {
  kw_cid_filenames <- list.files(kw_cid_dir,full.names = T)
  
  for (i in 1:length(kw_cid_filenames)){
    kw_cid_info <- read.table(kw_cid_filenames[i],sep="\t")
    if (i == 1) {
      all_kw_cid_info <- kw_cid_info
    } else if (!is.null(kw_cid_info)) {
      all_kw_cid_info <- merge(all_kw_cid_info,kw_cid_info,all=T)
    }
  }
  if (nchar(all_kw_cid_filename)<3) {
    all_kw_cid_filename <- paste0(kw_dir,"all_keyword_cids.txt")
  }
  
  write.table(all_kw_cid_info,all_kw_cid_filename,sep="\t",
              row.names = F,col.names=F,quote = F)
  return(all_kw_cid_filename)
}

