# Preparing adducts and TP information for SLOD
# E. Schymanski, Uni LU, 12 October 2020

##### set up dirs etc #####
base_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/annotations/slod/"
dir.exists(base_dir)
#dir.create(base_dir)
setwd(base_dir)

##### adducts ######
# This builds off the excellent and detailed adduct table that 
# comes with enviPat by Martin Loos, formerlly Eawag, now enviBee. 
# WebApp: https://envipat.eawag.ch/
# Reference: Loos, M., Gerber, C., Corona, F., Hollender, J., Singer, H. (2015). 
# Accelerated isotope fine structure calculation using pruned transition trees, 
# Analytical Chemistry 87(11), 5738-5744.
# DOI: 10.1021/acs.analchem.5b00941

library(enviPat)
data(adducts)
adducts$DisplayName <- ""

i <- 1
for (i in 1:length(adducts$Name)) {
  if (adducts$Ion_mode[i]=="positive") {
    if (adducts$Charge[i]==1) {
      if (adducts$Name[i] != "M+") {
        adducts$DisplayName[i] <- paste0("[",adducts$Name[i],"]+")
      } else {
        adducts$DisplayName[i] <- "[M]+"
      }
    } else {
      adducts$DisplayName[i] <- paste0("[",adducts$Name[i],"]",adducts$Charge[i],"+")
    }
  } else { #otherwise negative
    if (adducts$Charge[i]==(-1)) {
      if (adducts$Name[i] != "M-") {
        adducts$DisplayName[i] <- paste0("[",adducts$Name[i],"]-")
      } else {
        adducts$DisplayName[i] <- "[M]-"
      }
    } else {
      adducts$DisplayName[i] <- paste0("[",adducts$Name[i],"]",
                                       sub("-","",adducts$Charge[i],fixed=T),"-")
    }
  }
}

write.table(adducts, "adducts_enviPat.txt",sep="\t",row.names=FALSE)
write.csv(adducts, "adducts_enviPat.csv",row.names=FALSE)

default_adducts <- adducts[c(1:6,11),]
write.table(default_adducts, "default_adducts_enviPat.txt",sep="\t",row.names=FALSE)
write.csv(default_adducts, "default_adducts_enviPat.csv",row.names=FALSE)
# would be possible to select columns too, of course ... 

##### TPs #######
# Currently building off the MetXBioDB efforts
# Also in this GitLab repo

metxbiodb_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/annotations/tps/MetXBioDB/"
dir.exists(metxbiodb_dir)

# read in the depiction Transformations (has SMILES present)
tps_depict <- read.csv(paste0(metxbiodb_dir,"MetXBioDB_Transformations_depiction.csv"),
                       stringsAsFactors = F) 
transformations <- read.csv(paste0(metxbiodb_dir,"MetXBioDB_Transformations.csv"),
                            stringsAsFactors = F)

tps_depict$Transformation <- ""
tps_depict$Predecessor_Mass <- 0
tps_depict$Successor_Mass <- 0
tps_depict$MassDiff <- 0

i <- 1
for (i in 1:length(tps_depict$Predecessor_CID)) {
  # find the transformation statement
  biotid <- tps_depict$Source_ID[i]
  i_biotid <- grep(biotid,transformations$Source_ID,fixed=T)
  if (length(i_biotid)>0){
    trans_str <- transformations$Transformation[i_biotid[1]]
    trans_str <- strsplit(trans_str,split=" / ",fixed=T)[[1]][1]
    tps_depict$Transformation[i] <- trans_str
  }
  # calculate the mass difference and print it out
  smiles1 <- tps_depict$Predecessor_SMILES[i]
  smiles2 <- tps_depict$Successor_SMILES[i]
  mass1 <- getSuspectFormulaMass(smiles1)$Monoiso_mass
  mass2 <- getSuspectFormulaMass(smiles2)$Monoiso_mass
  if (mass1>0 && mass2>0) {
    tps_depict$Predecessor_Mass[i] <- mass1
    tps_depict$Successor_Mass[i] <- mass2
    tps_depict$MassDiff[i] <- mass2-mass1
  } else {
    tps_depict$Predecessor_Mass[i] <- NA
    tps_depict$Successor_Mass[i] <- NA
    tps_depict$MassDiff[i] <- NA
  }
}

# reverse mass diff as it was illogical
# have now corrected this above
# tps_depict$MassDiff <- -tps_depict$MassDiff
#legacy column removal
#tps_depict <- tps_depict[,-10]

write.csv(tps_depict,
          paste0(metxbiodb_dir,"MetXBioDB_Transformations_depiction_extra.csv"),
          row.names = F)

# now look at how many unique transformations
#unique_mass_diffs <- unique(round(tps_depict$MassDiff,digits=4))
mass_diffs <- as.character(round(tps_depict$MassDiff,digits=4))
unique_mass_diffs <- unique(mass_diffs)
mass_diff_count <- vector("numeric",length=length(unique_mass_diffs))

i <- 1

for (i in 1:length(mass_diff_count)) {
#  mass_diff_count[i] <- length(grep(unique_mass_diffs[i],mass_diffs,fixed=TRUE))
  mass_diff_count[i] <- length(which(mass_diffs == unique_mass_diffs[i]))
}

tp_mass_diffs <- as.data.frame(cbind(unique_mass_diffs,mass_diff_count))
colnames(tp_mass_diffs) <- c("Mass_difference","Count")
write.csv(tp_mass_diffs,"TP_mass_diff_MetXBioDB.csv",row.names = F)

# need to verify the counts <= cross checked 17/10

o <- order(mass_diff_count,decreasing = T)
tp_mass_diffs <- as.data.frame(cbind(as.numeric(unique_mass_diffs[o]),
                                     as.numeric(mass_diff_count[o])))
colnames(tp_mass_diffs) <- c("Mass_difference","Count")

#write.csv(tp_mass_diffs,"TP_mass_diff_MetXBioDB.csv",row.names = F)
write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "TP_mass_diff_MetXBioDB_ge5.csv",row.names = F)


##### TPs directly from Jeff's Transformation Table ######

library(httr)
library(rjson)
library(RChemMass)

base_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/annotations/slod/"
dir.exists(base_dir)
#dir.create(base_dir)
setwd(base_dir)

metxbiodb_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/annotations/tps/MetXBioDB/"
dir.exists(metxbiodb_dir)

anno_dir <- "C:/Users/emma.schymanski/Documents/GitHub/pubchem/annotations/"
dir.exists(metxbiodb_dir)
tp_dir <- paste0(anno_dir,"tps/")
dir.exists(tp_dir)

tp_file <- paste0(tp_dir,"norman_transformations_tsv.txt")
  #list.files(tp_dir,pattern="norman_transformations_tsv")
file.exists(tp_file)

tp_cols <- c("cid","predecessor","predecessorcid","transformation","successor",
             "successorcid","evidencedoi","evidenceref","sourcecomment",
             "sourcecommentfull","datasetdoi","datasetref","enzyme","biosystem")
trans_info <- read.delim(tp_file,header=F,skip=4,col.names = tp_cols,
                         stringsAsFactors = F)

# now, remove the first col and deduplicate, as each rxn is in twice
trans_info <- unique(trans_info[,-1])

trans_info$predecessorsmiles <- ""
trans_info$successorsmiles <- ""

n_iter <- round(length(trans_info$predecessorcid)/500)
n_entries <- length(trans_info$predecessorcid)

# assign predecessor smiles
j <- 1
for (j in 1:n_iter) {
  if (j==1) {
    prec_cids <- paste(trans_info$predecessorcid[1:500],collapse=",")
  } else if (j==n_iter) {
    prec_cids <- paste(trans_info$predecessorcid[((j-1)*500):n_entries],collapse=",")
  } else {
    prec_cids <- paste(trans_info$predecessorcid[((j-1)*500):(j*500)],collapse=",")
  }
  # retrieve SMILES
  prec_smiles <- getSMILESfromCIDs(prec_cids)
  # assign to file
  #i <- 1
  for (i in 1:length(prec_smiles$CIDs)) {
    i_tp <- which(trans_info$predecessorcid==prec_smiles$CIDs[i])
    trans_info$predecessorsmiles[i_tp] <- prec_smiles$smiles[i] 
  }
}


# assign successor smiles
# for these, more errors are cropping up, so try batches of 400 not 500
# last one had an error with 400 ... so try with 300 ... 
n_div <- 300
n_iter <- round(length(trans_info$predecessorcid)/n_div)
n_entries <- length(trans_info$predecessorcid)


j <- 1
for (j in 1:n_iter) {
  if (j==1) {
    succ_cids <- paste(trans_info$successorcid[1:n_div],collapse=",")
  } else if (j==n_iter) {
    succ_cids <- paste(trans_info$successorcid[((j-1)*n_div):n_entries],collapse=",")
  } else {
    succ_cids <- paste(trans_info$successorcid[((j-1)*n_div):(j*n_div)],collapse=",")
  }
  # retrieve SMILES
  succ_smiles <- getSMILESfromCIDs(succ_cids)
  # assign to file
  #i <- 1
  for (i in 1:length(succ_smiles$CIDs)) {
    i_tp <- which(trans_info$successorcid==succ_smiles$CIDs[i])
    trans_info$successorsmiles[i_tp] <- succ_smiles$smiles[i] 
  }
}


write.csv(trans_info,"norman_transformations_tsv_wExtras.csv",row.names = F)

trans_info$rxn_smiles <- ""

i <- 1
for (i in 1:length(trans_info$predecessor)) {
  rxn_smiles <- paste0(trans_info$predecessorsmiles[i],">[",
                       strsplit(trans_info$transformation[i]," ")[[1]][1],"]>",
                       trans_info$successorsmiles[i]," ",
                       strsplit(trans_info$datasetref[i],"|",fixed=T)[[1]][1], 
                       " CIDs: ",
                       trans_info$predecessorcid[i], " & ",
                       trans_info$successorcid[i])
  trans_info$rxn_smiles[i] <- rxn_smiles
}

#now calc mass differences etc
trans_info$predecessormass<- 0
trans_info$successormass <- 0
trans_info$massdiff <- 0


for (i in 1:length(trans_info$predecessor)) {
  smiles1 <- trans_info$predecessorsmiles[i]
  smiles2 <- trans_info$successorsmiles[i]
  # actually, this would be way quicker to get mass from PubChem...
  mass1 <- getSuspectFormulaMass(smiles1)$Monoiso_mass
  mass2 <- getSuspectFormulaMass(smiles2)$Monoiso_mass
  if (mass1>0 && mass2>0) {
    trans_info$predecessormass[i] <- mass1
    trans_info$successormass[i] <- mass2
    trans_info$massdiff[i] <- mass2-mass1
  } else {
    trans_info$predecessormass[i] <- NA
    trans_info$successormass[i] <- NA
    trans_info$massdiff[i] <- NA
  }
}

write.csv(trans_info,"norman_transformations_tsv_wExtras.csv",row.names = F)

# now look at how many unique transformations
#unique_mass_diffs <- unique(round(tps_depict$MassDiff,digits=4))
mass_diffs <- as.character(round(as.numeric(trans_info$massdiff),digits=4))
unique_mass_diffs <- unique(mass_diffs)
mass_diff_count <- vector("numeric",length=length(unique_mass_diffs))

i <- 1

for (i in 1:length(mass_diff_count)) {
  #  mass_diff_count[i] <- length(grep(unique_mass_diffs[i],mass_diffs,fixed=TRUE))
  mass_diff_count[i] <- length(which(mass_diffs == unique_mass_diffs[i]))
}

tp_mass_diffs <- as.data.frame(cbind(unique_mass_diffs,mass_diff_count))
colnames(tp_mass_diffs) <- c("Mass_difference","Count")
write.csv(tp_mass_diffs,"transformations_mass_diffs.csv",row.names = F)

# now look at them in order
o <- order(mass_diff_count,decreasing = T)
tp_mass_diffs <- as.data.frame(cbind(as.numeric(unique_mass_diffs[o]),
                                     as.numeric(mass_diff_count[o])))
colnames(tp_mass_diffs) <- c("Mass_difference","Count")

#write.csv(tp_mass_diffs,"TP_mass_diff_MetXBioDB.csv",row.names = F)
write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "transformations_mass_diffs_ge5.csv",row.names = F)


# match with previous MetXBioDB file
tp_mass_diffs$Reaction <- ""
tp_mass_diffs$Formula_gained <- ""
tp_mass_diffs$Formula_lost <- ""
tp_mass_diffs$SMARTS <- ""

MetXBioDB_data <- read.csv("TP_mass_diff_MetXBioDB_ge5_wAnnotation.csv",
                           stringsAsFactors = F)

i <- 1
for (i in 1:length(tp_mass_diffs$Mass_difference)) {
  massdiff_i <- tp_mass_diffs$Mass_difference[i]
  #i_metx <- grep(massdiff_i,MetXBioDB_data$Mass_difference,fixed=T,value=T)
  i_metx <- which(MetXBioDB_data$Mass_difference==massdiff_i)
  if (length(i_metx)>0) {
    tp_mass_diffs$Reaction[i] <- MetXBioDB_data$Reaction[i_metx]
    tp_mass_diffs$Formula_gained[i] <- MetXBioDB_data$Formula_gained[i_metx]
    tp_mass_diffs$Formula_lost[i] <- MetXBioDB_data$Formula_lost[i_metx]
    tp_mass_diffs$SMARTS[i] <- MetXBioDB_data$SMARTS[i_metx]
  }
}

write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "transformations_mass_diffs_ge5_wAnnotation.csv",row.names = F)



# #prec_cids <- paste(trans_info$predecessorcid[1:10],collapse=",")
# #prec_cids <- paste(trans_info$predecessorcid[1:100],collapse=",")
# 
# #prec_cids <- paste(trans_info$predecessorcid,collapse=",")
# 
# prec_smiles <- getSMILESfromCIDs(prec_cids)
# 
# i <- 1
# for (i in 1:length(prec_smiles$CIDs)) {
#   i_tp <- which(trans_info$predecessorcid==prec_smiles$CIDs[i])
#   trans_info$predecessorsmiles[i_tp] <- prec_smiles$smiles[i] 
# }
# 

###### Functions needed to get this to work ######

# testing
url <- "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/cid/2244,2554/property/IsomericSMILES/json"


#' Retrieve IsomericSMILES from PubChem CIDs
#' 
#' Retrieves Isomeric SMILES from PubChem CIDs using PUG REST.
#' 
#' This searches PubChem for many CIDs and returns the Isomeric SMILES. 
#' 
#' @usage getSMILESfromCIDs(query, from = "cid", to="IsomericSMILES")
#' 
#' @param query ID to be converted
#' @param from Type of input ID (default \code{cid}, i.e. PubChem CID)
#' @return The Isomeric SMILES of the given CID list
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem search: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' Pubchem REST:
#' \url{https://pubchem.ncbi.nlm.nih.gov/pug_rest/PUG_REST.html}
#' 
#' @examples
#' getSMILESfromCIDs("2244,2554")
#' getSMILESfromCIDs("2244,2554,123456")
#' getSMILESfromCIDs("2244,2554,blah")
#' # the latter returns NA
#' 
#' @export
getSMILESfromCIDs <- function(query, from = "cid", to="IsomericSMILES")
{
  baseURL <- "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound"
  url <- paste(baseURL, from, query, "property", to, "json",sep="/")
  #https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/cid/2244,2554/property/IsomericSMILES/json
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    {#url_data <- getURL(URLencode(url),timeout=5),
      res <- GET(URLencode(url))
      url_data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  # This happens if the PCID is not found:
  r <- fromJSON(url_data)
  
  if(!is.null(r$Fault))
    return(NA)
  
  CIDs <- unlist(lapply(r$PropertyTable$Properties, function(i) i$CID))
  smiles <- unlist(lapply(r$PropertyTable$Properties, function(i) i$IsomericSMILES))
  
  if(is.null(smiles)){
    return(NA)
  } else{
    return(as.data.frame(cbind(CIDs,smiles),stringsAsFactors = F))
  }
}

