# Script to get CIDs for transformations and form summary file
# E. Schymanski, 16 May 2021
# Functions now ported to extractAnnotations.R
# 4 June 2021 Merging in ChEMBL_Transformations.R to make
# one master file


#### Set up ####
# Packages
library(webchem)
library(data.table)

# Scripts:
source('~/GitHub/pubchem/annotations/tps/extractAnnotations.R')

# Locations
#PC_trans_dir <- "C:/DATA/PubChem/Transformations/"
PC_trans_dir <- "~/GitHub/pubchem/annotations/tps/Transformations/"
PC_tp_dir <- "~/GitHub/pubchem/annotations/tps/"
setwd(PC_trans_dir)



#### Retrieve NORMAN-SLE Transformation Information ####

# retrieve directly using getPcTrans
SLE_tp_output <- getPcTrans(dataset="transformations",
                                 file_name="NORMAN-SLE_all_transformations.csv")

# tp_info contains filename and unique CIDs
SLE_tp_output$TP_File_Name
length(SLE_tp_output$TP_Unique_CIDs)
# check for NAs
which(is.na(SLE_tp_output$TP_Unique_CIDs))
SLE_cids <- na.exclude(SLE_tp_output$TP_Unique_CIDs)
length(SLE_cids)
# check for NULLs
SLE_NULL_i <- grep("NULL",SLE_cids)
if (length(SLE_NULL_i)>0) {
  SLE_cids <- SLE_cids[-SLE_NULL_i]
}
length(SLE_cids)
# 4771 on June 5th, 4744 on June 12th (S79 was missing), 4786 June 13th.
# 4880 Oct 19th 2021. 4928 on 1 Sept 2022. 
# 5127 on 17 Apr 2023, 5440 on 5 July 2023, 5441 on 28 July 2023
# 5602 on 13 Dec 2023, 5628 on 2 Apr 2024

# read the contents in ... 
SLE_trans <- read.csv(SLE_tp_output$TP_File_Name,stringsAsFactors = F)
# check CID count
length(unique(na.exclude(c(SLE_trans$predecessorcid,SLE_trans$successorcid))))
# remove the NULL entries
SLE_NULL_prec_i <- grep("NULL",SLE_trans$predecessorcid)
SLE_NULL_succ_i <- grep("NULL",SLE_trans$successorcid)
if (length(SLE_NULL_prec_i)>0) {
  SLE_trans <- SLE_trans[-SLE_NULL_prec_i,]
}
if (length(SLE_NULL_succ_i)>0) {
  SLE_trans <- SLE_trans[-SLE_NULL_succ_i,]
}

n_SLE_cids <- length(unique(na.exclude(c(SLE_trans$predecessorcid,SLE_trans$successorcid))))
n_SLE_cids
# 4771 on June 5th. 4880 Oct 19th. 4928 on 20220901. 5127 on 20230417, 
# 5440 on 20230705, 5441 on 20230728, 5602 on 20231213, 5628 on 20240402

# Legacy code to retrieve per CID much slower now below. 


#### Retrieve ChEMBL Transformation Information ####

# retrieve directly using getPcTrans
ChEMBL_tp_output <- getPcTrans(dataset="chemblmetabolism",
                               file_name="ChEMBL_all_transformations.csv")

# tp_info contains filename and unique CIDs
ChEMBL_tp_output$TP_File_Name

ChEMBL_trans <- read.csv(ChEMBL_tp_output$TP_File_Name,stringsAsFactors = F)
#length(ChEMBL_tp_info$TP_Unique_CIDs)
# doesn't exist for ChEMBL, likely due to mismatching headers. 
# which(is.na(ChEMBL_tp_info$TP_Unique_CIDs))
# ChEMBL_cids <- na.exclude(tp_info$TP_Unique_CIDs)

ChEMBL_cids <- unique(c(ChEMBL_trans$substratecid,ChEMBL_trans$metabolitecid))
n_ChEMBL_cids <- length(ChEMBL_cids)
n_ChEMBL_cids
# 1418 on June 5th & 13th, Oct 19th 2021, 1 Sept 2022, 17 Apr 2023, 5 & 28 July 2023, 
# 1418 on 13 Dec 2023, 2 Apr 2024


#### Merging NORMAN-SLE and ChEMBL Data #####

#setwd(PC_trans_dir)
colnames(SLE_trans)
colnames(ChEMBL_trans)

# NORMAN-SLE
SLE_cols <- c("cid","predecessor", "predecessorcid", "transformation", "successor",
              "successorcid", "evidencedoi", "evidenceref", "sourcecomment",
              "sourcecommentfull", "datasetdoi", "datasetref", "enzyme", 
              "biosystem")#, "cmpdname")
# rename because CID col is incorrectly recognised
colnames(SLE_trans) <- SLE_cols

# these may change ... harmonization in progress ...
ChEMBL_cols <- c("metid", "substrate", "metabolite", "substratechemblid",
                 "metabolitechemblid", "substratecid", "metabolitecid",
                 "metconversion", "enzyme", "enzymechemblid", "cids",
                 "protacxns", "geneids", "pmids", "dois")
# rename because first col is incorrectly recognised
colnames(ChEMBL_trans) <- ChEMBL_cols

# now rename some of the ChEMBL cols
ChEMBL_cols_rename <- c("metid", "predecessor", "successor", "substratechemblid",
                        "metabolitechemblid", "predecessorcid", "successorcid",
                        "transformation", "enzyme", "enzymechemblid", "cids",
                        "protacxns", "geneids", "evidenceref", "evidencedoi")
colnames(ChEMBL_trans) <- ChEMBL_cols_rename
ChEMBL_trans$sourcecomment <- ""
ChEMBL_trans$sourcecommentfull <- ""
ChEMBL_trans$datasetref <- "ChEMBL"

for (i in 1:length(ChEMBL_trans$sourcecomment)) {
  ChEMBL_trans$sourcecomment[i] <- paste0("ChEMBL IDs (pred,succ,enzyme): ",
                                          ChEMBL_trans$substratechemblid[i],
                                          "|",ChEMBL_trans$metabolitechemblid[i],
                                          "|",ChEMBL_trans$enzymechemblid[i])
  ChEMBL_trans$sourcecommentfull[i] <- paste0("metid:", ChEMBL_trans$metid[i],
                                              "; protacxns:",ChEMBL_trans$protacxns[i],
                                              "; geneids:",ChEMBL_trans$geneids[i])
}

# merge the two datasets

# overall merge
all_trans <- merge(SLE_trans,ChEMBL_trans,all=T)

# trim columns not needed
col_headers_rm <- c("cid","cmpdname","metid","substratechemblid",
                    "metabolitechemblid","enzymechemblid", "cids",
                    "protacxns", "geneids")
which(colnames(all_trans) %in% col_headers_rm)
all_trans <- all_trans[,-which(colnames(all_trans) %in% col_headers_rm)]
# now unique-ify, since some trans are duplicated, especially SLE
all_trans <- unique(all_trans)
length(all_trans$predecessor)
# 5844 entries, June 5th and June 13th. 5971 Oct 19th.
# 6108 on 1 Sept 2022. 6554 on 17 Apr 2023. 7244 on 5 July 2023, 
# 7245 on 28 July 2023, 7479 on 13 Dec 2023, 7504 on 2 Apr 2024
n_pred <- length(unique(all_trans$predecessorcid))
n_succ <- length(unique(all_trans$successorcid))


# write out merged file - to subdir and main dir
write.csv(all_trans,"PubChem_all_transformations.csv",row.names = F)
write.csv(all_trans,paste0(PC_tp_dir,"PubChem_all_transformations.csv"),row.names=F)

#### Add Chemical and Reaction Information ####

# get all CIDs
all_CIDs <- unique(na.omit(c(all_trans$predecessorcid,all_trans$successorcid)))
length(all_CIDs) 
#6007 - June 5th; 6022 June 13th; 6109 Oct 19th. 6155 1 Sept 2022.
# 6351 on 17 Apr 2023, 6640 on 5 July 2023, 6641 on 28 July 2023
# 6782 on 13 Dec 2023, 6806 on 2 Apr 2024

# retrieve the information using tips from Rick
selected_properties <- c("MolecularFormula","ExactMass","XlogP",
                         "CanonicalSMILES","IsomericSMILES",
                         "InChI","InChIKey","Title","IUPACName")
# retrieve info with webchem
#CID_info_interim <- webchem::pc_prop(all_CIDs[1:1000], selected_properties)
CID_info_all <- as.data.table(webchem::pc_prop(all_CIDs, selected_properties))

# add IKFB
CID_info_all$IKFB <- strtrim(CID_info_all$InChIKey,14)

# output
write.csv(CID_info_all,"Transformations_CID_Info_all.csv",row.names = F)


# now add the reaction information

# have to remove successors and predecessors that are "NA"
# these are kept in earlier file for debugging
# Update 13 June - should be no more NAs
NA_succ <- which(is.na(all_trans$successorcid))
length(NA_succ)
if (length(NA_succ > 0)) {
  all_trans <- all_trans[-NA_succ,]
}
NA_pred <- which(is.na(all_trans$predecessorcid))
if (length(NA_pred > 0)) {
  all_trans <- all_trans[-NA_pred,]
}
# for the record, the new NULLs were removed further up 

length(all_trans$predecessorcid) 
# June 5: 5822; June 13: 5844. Oct 19th 5971. 1 Sept 2022: 6108
# 17 Apr 2023: 6554. 5 July 2023: 7244. 28 July 2023: 7245.
# 13 Dec 2023: 7479. 2 Apr 2024: 7504


all_trans$CID_to_CID <- ""
all_trans$IK_to_IK <- ""
all_trans$IKFB_to_IKFB <- ""
all_trans$ReactionSMILES <- ""
all_trans$DescReactionSMILES <- ""
all_trans$predecessorEM <- ""
all_trans$successorEM <- ""
all_trans$MassDiff <- ""
all_trans$XlogPDiff <- ""

i <- 1
for (i in 1:length(all_trans$predecessorcid)) {
  pred_CID <- all_trans$predecessorcid[i]
  succ_CID <- all_trans$successorcid[i]
  i_pred_CID <- which(pred_CID==CID_info_all$CID)
  #i_pred_CID <- grep(pred_CID,CID_info_all$CID,fixed = T)
  if (length(i_pred_CID) >1) {
    print(paste("Matching predecessor CIDs indexes:", paste(i_pred_CID,collapse = ",")))
  }
  i_succ_CID <- which(succ_CID==CID_info_all$CID)
  #i_succ_CID <- grep(succ_CID,CID_info_all$CID,fixed = T)
  if (length(i_succ_CID) >1) {
    print(paste("Matching predecessor CIDs indexes:", paste(i_succ_CID,collapse = ",")))
  }
  pred_IK <- CID_info_all$InChIKey[i_pred_CID]
  succ_IK <- CID_info_all$InChIKey[i_succ_CID]
  pred_SMILES <- CID_info_all$IsomericSMILES[i_pred_CID]
  succ_SMILES <- CID_info_all$IsomericSMILES[i_succ_CID]
  pred_EM <- as.numeric(CID_info_all$ExactMass[i_pred_CID])
  succ_EM <- as.numeric(CID_info_all$ExactMass[i_succ_CID])
  pred_XlogP <- as.numeric(CID_info_all$XLogP[i_pred_CID])
  succ_XlogP <- as.numeric(CID_info_all$XLogP[i_succ_CID])
  
  CID_to_CID <- paste0(pred_CID,">>",succ_CID)
  IK_to_IK <- paste0(pred_IK,">>",succ_IK)
  IKFB_to_IKFB <- paste0(strtrim(pred_IK,14),">>",strtrim(succ_IK,14))
  ReactionSMILES <- paste0(pred_SMILES,">>",succ_SMILES)
  DescReactionSMILES <- paste0(pred_SMILES,">[",all_trans$transformation[i],
                               "]>",succ_SMILES," CID: ",pred_CID, " to CID: ",
                               succ_CID)
  MassDiff <- succ_EM - pred_EM
  XlogPDiff <- succ_XlogP-pred_XlogP
  
  all_trans$CID_to_CID[i] <- CID_to_CID
  all_trans$IK_to_IK[i] <- IK_to_IK
  all_trans$IKFB_to_IKFB[i] <- IKFB_to_IKFB
  all_trans$ReactionSMILES[i] <- ReactionSMILES
  all_trans$DescReactionSMILES[i] <- DescReactionSMILES
  all_trans$predecessorEM[i] <- pred_EM
  all_trans$successorEM[i] <- succ_EM
  all_trans$MassDiff[i] <- MassDiff
  all_trans$XlogPDiff[i] <- XlogPDiff
}

# output of full file with all info
write.csv(all_trans,"PubChem_all_transformations_wExtraInfo.csv",row.names = F)
write.csv(all_trans,paste0(PC_tp_dir,"PubChem_all_transformations_wExtraInfo.csv"),
          row.names = F)


# now some stats
# paste0("Unique Transformation Entries: ",length(all_trans$predecessor))
# paste0("Unique Reactions by CID: ",length(unique(all_trans$CID_to_CID)))
# paste0("Unique Reactions by IK: ",length(unique(all_trans$IK_to_IK)))
# paste0("Unique Reactions by IKFB: ",length(unique(all_trans$IKFB_to_IKFB)))
# # look at the XlogPs and Mass Diffs
# range(as.numeric(all_trans$XlogPDiff),na.rm = T)
# range(as.numeric(all_trans$MassDiff))

summary_text <- c(paste0("Unique Transformation Entries: ",length(all_trans$predecessor)),
                  paste0("Unique Reactions by CID: ",length(unique(all_trans$CID_to_CID))),
                  paste0("Unique Reactions by IK: ",length(unique(all_trans$IK_to_IK))),
                  paste0("Unique Reactions by IKFB: ",length(unique(all_trans$IKFB_to_IKFB))),
                  paste0("Unique NORMAN-SLE Compounds by CID: ",n_SLE_cids),
                  paste0("Unique ChEMBL Compounds by CID: ",n_ChEMBL_cids),
                  paste0("Unique Compounds (all) by CID: ",length(all_CIDs)),
                  paste0("Unique Predecessors (all) by CID: ",n_pred),
                  paste0("Unique Successors (all) by CID: ",n_succ),
                  paste0("Range of XlogP Differences: ",
                         paste(range(as.numeric(all_trans$XlogPDiff),na.rm = T),collapse = ",")),
                  paste0("Range of Mass Differences: ",
                         paste(range(as.numeric(all_trans$MassDiff)),collapse = ",")))

writeLines(summary_text)

## June 5
# Unique Transformation Entries: 5822
# Unique Reactions by CID: 4914
# Unique Reactions by IK: 4903
# Unique Reactions by IKFB: 4654
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## June 13:
# Unique Transformation Entries: 5844
# Unique Reactions by CID: 4934
# Unique Reactions by IK: 4923
# Unique Reactions by IKFB: 4667
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## October 19:
# Unique Transformation Entries: 5971
# Unique Reactions by CID: 5056
# Unique Reactions by IK: 5045
# Unique Reactions by IKFB: 4783
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## September 1st, 2022:
# Unique Transformation Entries: 6108
# Unique Reactions by CID: 5190
# Unique Reactions by IK: 5179
# Unique Reactions by IKFB: 4885
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## April 17, 2023: 
# Unique Transformation Entries: 6554
# Unique Reactions by CID: 5564
# Unique Reactions by IK: 5553
# Unique Reactions by IKFB: 5237
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## July 5, 2023: 
# Unique Transformation Entries: 7244
# Unique Reactions by CID: 5923
# Unique Reactions by IK: 5912
# Unique Reactions by IKFB: 5573
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## July 28, 2023
# Unique Transformation Entries: 7245
# Unique Reactions by CID: 5924
# Unique Reactions by IK: 5913
# Unique Reactions by IKFB: 5573
# Unique NORMAN-SLE Compounds by CID: 5441
# Unique ChEMBL Compounds by CID: 1418
# Unique Compounds (all) by CID: 6641
# Unique Predecessors (all) by CID: 2448
# Unique Successors (all) by CID: 4979
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## 13 Dec 2023
# Unique Transformation Entries: 7479
# Unique Reactions by CID: 6101
# Unique Reactions by IK: 6090
# Unique Reactions by IKFB: 5737
# Unique NORMAN-SLE Compounds by CID: 5602
# Unique ChEMBL Compounds by CID: 1418
# Unique Compounds (all) by CID: 6782
# Unique Predecessors (all) by CID: 2490
# Unique Successors (all) by CID: 5100
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

## 2 April 2024
# Unique Transformation Entries: 7504
# Unique Reactions by CID: 6122
# Unique Reactions by IK: 6111
# Unique Reactions by IKFB: 5756
# Unique NORMAN-SLE Compounds by CID: 5628
# Unique ChEMBL Compounds by CID: 1418
# Unique Compounds (all) by CID: 6806
# Unique Predecessors (all) by CID: 2497
# Unique Successors (all) by CID: 5119
# Range of XlogP Differences: -9.9,9.4
# Range of Mass Differences: -893.1581377,749.104645569

summary_file <- "Transformations_Summary_Stats.txt"
writeLines(summary_text,con=summary_file)

##### Mass Difference Analysis #####

# now look at how many unique transformations
all_unique_trans <- unique(all_trans[,c(14,21)])
# col 14 = CID>>CID and col 21 = MassDiff
mass_diffs <- as.character(round(as.numeric(all_unique_trans$MassDiff),digits=4))
unique_mass_diffs <- unique(mass_diffs)
length(unique_mass_diffs)
mass_diff_count <- vector("numeric",length=length(unique_mass_diffs))

i <- 1
for (i in 1:length(mass_diff_count)) {
  #  mass_diff_count[i] <- length(grep(unique_mass_diffs[i],mass_diffs,fixed=TRUE))
  mass_diff_count[i] <- length(which(mass_diffs == unique_mass_diffs[i]))
}

tp_mass_diffs <- as.data.frame(cbind(unique_mass_diffs,mass_diff_count))

# now look at them in order
o <- order(mass_diff_count,decreasing = T)
tp_mass_diffs <- as.data.frame(cbind(as.numeric(unique_mass_diffs[o]),
                                     as.numeric(mass_diff_count[o])))
colnames(tp_mass_diffs) <- c("MassDiff","Count")

#write.csv(tp_mass_diffs,"TP_mass_diff_MetXBioDB.csv",row.names = F)
write.csv(tp_mass_diffs,"Transformations_MassDiffs_all.csv",row.names = F)
write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "Transformations_MassDiffs_ge5.csv",row.names = F)


# match with previous SLOD file
tp_mass_diffs$Reaction <- ""
tp_mass_diffs$Formula_gained <- ""
tp_mass_diffs$Formula_lost <- ""
tp_mass_diffs$SMARTS <- ""

#mass_diff_anno_URL <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/slod/Transformations_MassDiffs_ge5_wAnnotation.csv"
#download.file(mass_diff_anno_URL,destfile = "slod_Transformations_MassDiffs_ge5_wAnnotation.csv")
slod_data <- read.csv("slod_Transformations_MassDiffs_ge5_wAnnotation.csv",
                      stringsAsFactors = F)


i <- 1
for (i in 1:length(tp_mass_diffs$MassDiff)) {
  massdiff_i <- tp_mass_diffs$MassDiff[i]
  #i_metx <- grep(massdiff_i,MetXBioDB_data$Mass_difference,fixed=T,value=T)
  i_metx <- which(slod_data$MassDiff==massdiff_i)
  if (length(i_metx)>0) {
    tp_mass_diffs$Reaction[i] <- slod_data$Reaction[i_metx]
    tp_mass_diffs$Formula_gained[i] <- slod_data$Formula_gained[i_metx]
    tp_mass_diffs$Formula_lost[i] <- slod_data$Formula_lost[i_metx]
    tp_mass_diffs$SMARTS[i] <- slod_data$SMARTS[i_metx]
  }
}

write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "Transformations_MassDiffs_ge5_wAnnotation.csv",row.names = F)
#file.remove("slod_Transformations_MassDiffs_ge5_wAnnotation.csv")

# if there are new entries in this file, take a look and update ... 
# ... and also update the SLOD file ... 

tp_md_annos <- read.csv("Transformations_MassDiffs_ge5_wAnnotation.csv", 
                        stringsAsFactors = F)
tp_md_anno_csv <- "Transformations_MassDiffs_ge5_wAnnotation.csv"
slod_dir <- sub("/tps/","/slod/",PC_tp_dir,fixed = T)
write.csv(tp_md_annos,paste0(slod_dir,tp_md_anno_csv),row.names = F)

##### Legacy Code ######

SLE_dir <- paste0(PC_trans_dir,"SLE_files/")
if(!dir.exists(SLE_dir)) {
  dir.create(SLE_dir)
}

ChEMBL_dir <- paste0(PC_trans_dir,"ChEMBL_files/")
if(!dir.exists(ChEMBL_dir)) {
  dir.create(ChEMBL_dir)
}


##### NORMAN-SLE by files ######

source <- "NORMAN Suspect List Exchange"
heading <- "Transformations"
page <- 1
i <- 1

n_pages <- getPcAnno.TotalPages(source, heading)

for (i in 1:n_pages) {
  if (i == 1) {
    cids <- getPcAnno.Transformations(source, heading, page=i)
  } else {
    cids_px <- getPcAnno.Transformations(source, heading, page=i)
    cids <- c(cids,cids_px)
  }
}

length(cids)
# 4771 CIDs
which(is.na(cids))
cids <- cids[-which(is.na(cids))]
length(cids)
#4747
length(unique(cids))
#4747 - note new entries, need to rerun!

# now for each CID, have to get the transformation tables ... 
setwd(SLE_dir)

# try with no CID selected
tp_info <- getPcCand.trans("",dataset="transformations",
                           file_name="SLE_Transformations.csv")

for (i in 1:length(cids)) {
  tp_info <- getPcCand.trans(cids[i])
}

#Note that this took 1 hr 39 min to retrieve all files ... (4706 total)
# with 25 warnings for the NA files ... (so only 25 missing..)

## Now merge all the files together ... 
trans_files <- list.files(SLE_dir, full.names = T)
length(trans_files)
#remove the NA file if present ...
trans_files <- trans_files[-grep("_NA_",trans_files)]
length(trans_files)

trans_data <- read.csv(trans_files[1],stringsAsFactors = F)
trans_data2 <- read.csv(trans_files[2],stringsAsFactors = F)
trans_data_all <- merge(trans_data,trans_data2,all.x=TRUE,all.y=TRUE)
for (i in 3:length(trans_files)) {
  file_name <- trans_files[i]
  # NA_test <- grep("_NA_",file_name,fixed=TRUE)
  # if (length(NA_test)==0) {
  trans_data2 <- read.csv(file_name)
  trans_data_all <- merge(trans_data_all,trans_data2,all.x=TRUE,all.y=TRUE)
  # }
}
all_trans_file <- paste0(PC_trans_dir,"NORMAN-SLE_all_transformations.csv")
#write.csv(trans_data_all[,c(-1,-2)],all_trans_file,row.names = F)
# Jeff fixed empty leading cols
write.csv(trans_data_all,all_trans_file,row.names = F)

#### ChEMBL by file #####

# n_pages <- getPcAnno.TotalPages("ChEMBL","Transformations")
# # answer = 2

source <- "ChEMBL"
heading <- "Transformations"
n_pages <- getPcAnno.TotalPages(source, heading)
for (i in 1:n_pages) {
  if (i == 1) {
    cids <- getPcAnno.Transformations(source, heading, page=i)
  } else {
    cids_px <- getPcAnno.Transformations(source, heading, page=i) 
    cids <- c(cids,cids_px)
  }
}
length(cids)

# May 2021 this was 4730 CIDs for NORMAN-SLE
# May 2021 this was 1418 for ChEMBL

setwd(ChEMBL_dir)

for (i in 1:length(cids)) {
  #for (i in 1046:length(cids)) {
  getPcCand.trans(cids[i],dataset = "chemblmetabolism")
}


## Now merge all the files together ... 

trans_files <- list.files(ChEMBL_dir, full.names = T)
length(trans_files)
# #remove the NA file if present ...
# trans_files <- trans_files[-grep("_NA_",trans_files)]
# length(trans_files)

trans_data <- read.csv(trans_files[1],stringsAsFactors = F)
trans_data2 <- read.csv(trans_files[2],stringsAsFactors = F)
trans_data_all <- merge(trans_data,trans_data2,all.x=TRUE,all.y=TRUE)
for (i in 3:length(trans_files)) {
  file_name <- trans_files[i]
  # NA_test <- grep("_NA_",file_name,fixed=TRUE)
  # if (length(NA_test)==0) {
  trans_data2 <- read.csv(file_name)
  trans_data_all <- merge(trans_data_all,trans_data2,all.x=TRUE,all.y=TRUE)
  # }
}
all_trans_file <- paste0(PC_trans_dir,"ChEMBL_all_transformations.csv")
# # trim two lead cols from the dataset
# write.csv(trans_data_all[,c(-1,-2)],all_trans_file,row.names = F)
# this should now be fixed, so no trimming
write.csv(trans_data_all,all_trans_file,row.names = F)


#### Legacy merging / eval #####

# keep only essential columns:
SLE_trans <- SLE_trans[,c(2:6)]
ChEMBL_trans <- ChEMBL_trans[,c(2,6,8,3,7)]
merged_colnames <- SLE_cols[2:6]
colnames(ChEMBL_trans) <- merged_colnames
length(SLE_trans$predecessorcid) # 8084, matches manual eval
length(ChEMBL_trans$predecessorcid) #1748, matches manual eval

merged_trans <- merge(SLE_trans,ChEMBL_trans,all=T)
length(merged_trans$predecessorcid) #9832 - lots of duplicates
length(unique(merged_trans)$predecessorcid) # 5393, but possible duplicates due to names




# now, uniquify the merged Transformations with new CID info
trans_info <- unique(merged_trans[,c(2,5,6)])
length(trans_info$cid_to_cid) #4900, same as manual eval

trans_info$IK_to_IK <- ""
trans_info$IKFB_to_IKFB <- ""
trans_info$ReactionSMILES <- ""
trans_info$predecessorEM <- ""
trans_info$successorEM <- ""
trans_info$MassDiff <- ""
trans_info$XlogPDiff <- ""

# have to remove successors and predecessors that are "NA"
NA_succ <- which(is.na(trans_info$successorcid))
trans_info <- trans_info[-NA_succ,]
NA_pred <- which(is.na(trans_info$predecessorcid))
trans_info <- trans_info[-NA_pred,]
length(trans_info$predecessorcid) # now 4885

i <- 1
for (i in 1:length(trans_info$predecessorcid)) {
  pred_CID <- trans_info$predecessorcid[i]
  succ_CID <- trans_info$successorcid[i]
  i_pred_CID <- which(pred_CID==CID_info_all$CID)
  #i_pred_CID <- grep(pred_CID,CID_info_all$CID,fixed = T)
  if (length(i_pred_CID) >1) {
    print(paste("Matching predecessor CIDs indexes:", paste(i_pred_CID,collapse = ",")))
  }
  i_succ_CID <- which(succ_CID==CID_info_all$CID)
  #i_succ_CID <- grep(succ_CID,CID_info_all$CID,fixed = T)
  if (length(i_succ_CID) >1) {
    print(paste("Matching predecessor CIDs indexes:", paste(i_succ_CID,collapse = ",")))
  }
  pred_IK <- CID_info_all$InChIKey[i_pred_CID]
  succ_IK <- CID_info_all$InChIKey[i_succ_CID]
  pred_SMILES <- CID_info_all$IsomericSMILES[i_pred_CID]
  succ_SMILES <- CID_info_all$IsomericSMILES[i_succ_CID]
  pred_EM <- as.numeric(CID_info_all$ExactMass[i_pred_CID])
  succ_EM <- as.numeric(CID_info_all$ExactMass[i_succ_CID])
  pred_XlogP <- as.numeric(CID_info_all$XLogP[i_pred_CID])
  succ_XlogP <- as.numeric(CID_info_all$XLogP[i_succ_CID])
  
  IK_to_IK <- paste0(pred_IK,">>",succ_IK)
  IKFB_to_IKFB <- paste0(strtrim(pred_IK,14),">>",strtrim(succ_IK,14))
  ReactionSMILES <- paste0(pred_SMILES,">>",succ_SMILES)
  MassDiff <- succ_EM - pred_EM
  XlogPDiff <- succ_XlogP-pred_XlogP
  
  trans_info$IK_to_IK[i] <- IK_to_IK
  trans_info$IKFB_to_IKFB[i] <- IKFB_to_IKFB
  trans_info$ReactionSMILES[i] <- ReactionSMILES
  trans_info$predecessorEM[i] <- pred_EM
  trans_info$successorEM[i] <- succ_EM
  trans_info$MassDiff[i] <- MassDiff
  trans_info$XlogPDiff[i] <- XlogPDiff
}

write.csv(trans_info,"Transformations_Rxn_Info_all.csv",row.names = F)

# now some stats
length(unique(trans_info$IKFB_to_IKFB)) #4624 <= this is slightly more than manual eval
# but this is confirmed taking this out into excel ... also 4624. 
# look at the XlogPs
range(as.numeric(trans_info$XlogPDiff),na.rm = T)
range(as.numeric(trans_info$MassDiff))


# now look at how many unique transformations
mass_diffs <- as.character(round(as.numeric(trans_info$MassDiff),digits=4))
unique_mass_diffs <- unique(mass_diffs)
length(unique_mass_diffs)
mass_diff_count <- vector("numeric",length=length(unique_mass_diffs))

i <- 1

for (i in 1:length(mass_diff_count)) {
  #  mass_diff_count[i] <- length(grep(unique_mass_diffs[i],mass_diffs,fixed=TRUE))
  mass_diff_count[i] <- length(which(mass_diffs == unique_mass_diffs[i]))
}

tp_mass_diffs <- as.data.frame(cbind(unique_mass_diffs,mass_diff_count))
#colnames(tp_mass_diffs) <- c("Mass_difference","Count")

# now look at them in order
o <- order(mass_diff_count,decreasing = T)
tp_mass_diffs <- as.data.frame(cbind(as.numeric(unique_mass_diffs[o]),
                                     as.numeric(mass_diff_count[o])))
colnames(tp_mass_diffs) <- c("MassDiff","Count")

#write.csv(tp_mass_diffs,"TP_mass_diff_MetXBioDB.csv",row.names = F)
write.csv(tp_mass_diffs,"Transformations_MassDiffs_all.csv",row.names = F)
write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "Transformations_MassDiffs_ge5.csv",row.names = F)


# match with previous SLOD file
tp_mass_diffs$Reaction <- ""
tp_mass_diffs$Formula_gained <- ""
tp_mass_diffs$Formula_lost <- ""
tp_mass_diffs$SMARTS <- ""

# mass_diff_anno_URL <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/slod/transformations_mass_diffs_ge5_wAnnotation.csv"
# download.file(mass_diff_anno_URL,destfile = "slod_transformations_mass_diffs_ge5_wAnnotation.csv")
# slod_data <- read.csv("slod_transformations_mass_diffs_ge5_wAnnotation.csv",
#                            stringsAsFactors = F)
# # use updated file in future
mass_diff_anno_URL <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/slod/Transformations_MassDiffs_ge5_wAnnotation.csv"
download.file(mass_diff_anno_URL,destfile = "slod_Transformations_MassDiffs_ge5_wAnnotation.csv")
slod_data <- read.csv("slod_Transformations_MassDiffs_ge5_wAnnotation.csv",
                      stringsAsFactors = F)


i <- 1
for (i in 1:length(tp_mass_diffs$MassDiff)) {
  massdiff_i <- tp_mass_diffs$MassDiff[i]
  #i_metx <- grep(massdiff_i,MetXBioDB_data$Mass_difference,fixed=T,value=T)
  i_metx <- which(slod_data$MassDiff==massdiff_i)
  if (length(i_metx)>0) {
    tp_mass_diffs$Reaction[i] <- slod_data$Reaction[i_metx]
    tp_mass_diffs$Formula_gained[i] <- slod_data$Formula_gained[i_metx]
    tp_mass_diffs$Formula_lost[i] <- slod_data$Formula_lost[i_metx]
    tp_mass_diffs$SMARTS[i] <- slod_data$SMARTS[i_metx]
  }
}

write.csv(tp_mass_diffs[which(tp_mass_diffs$Count>=5),],
          "Transformations_MassDiffs_ge5_wAnnotation.csv",row.names = F)





#### getPcAnno.Transformations ####

# now ported to extractAnnotations.R (16/5/2021)


#' Retrieve CIDs with Transformations Entries (by page)
#' 
#' Retrieve the CIDs with Transformations entries from the 
#' annotation information at the Transformation 
#' section of data sources, per page. As this requires
#' retrieving the entire JSON file (up to 1000 entries), this can take 
#' some time (hence large default timeout). Thanks to Paul Thiessen, 
#' Jeff Zhang and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcAnno.Transformations(source, heading, page=1, file_name="", timeout=100)
#' 
#' @param source Data source name of the annotations data, e.g. \code{"NORMAN Suspect List Exchange"}
#' @param heading Annotation category desired, e.g. \code{"Transformations"}
#' @param page Page number to retrieve. Use \link{getPcAnno.TotalPages} to 
#' determine page numbers available
#' @param timeout The timeout, in seconds. Should be generous as these are large files
#' @return The CIDs mentioned in the record.
#' 
#' @details 
#' This function will only extract CIDs and thus probably only works for
#' Transformations sections (and/or any other section where only CIDs are
#' needed)
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' PubChem Data Sources (with annotations):
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/#type=Annotations}
#' 
#' Transformations Example: 
#' \url{https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source=NORMAN%20Suspect%20List%20Exchange&heading_type=Compound&heading=Transformations&page=1}
#' 
#' @seealso \code{\link{getPcAnno.TotalPages}}, \code{\link{getPcAnno.Metabolism}}
#' 
#' @examples
#' # This retrieves only the CIDs from the first page
#' getPcAnno.Transformations(source = 'NORMAN Suspect List Exchange',heading = 'Transformations')
#' # This will retrieve all as a list of CIDs
#' source <- "NORMAN Suspect List Exchange"
#' heading <- "Transformations"
#' n_pages <- getPcAnno.TotalPages(source, heading)
#' for (i in 1:n_pages) {
#'   if (i == 1) {
#'     cids <- getPcAnno.Transformations(source, heading, page=i)
#'   } else {
#'     cids_px <- getPcAnno.Transformations(source, heading, page=i)
#'     cids <- c(cids,cids_px)
#'   }
#' }
#' # May 2021 this was 4730 CIDs
#' 
#' @export
getPcAnno.Transformations <- function(source, heading, page=1, timeout=100) {
  # set URL
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source='
  #source <- 'HSDB'
  midURL <- '&heading='
  #heading <- 'Metabolism/Metabolites'
  #endURL <- '&heading_type=Compound&page=1&response_type=display'
  endURL <- '&heading_type=Compound&page='
  url <- paste0(baseURL, source, midURL, heading, endURL, page)
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    #data <- getURL(URLencode(url),timeout=timeout),
    {
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  r <- fromJSON(data)
  
  if(!is.null(r$Fault)) {
    return(NA)
  }
  
  n_annos <- length(r$Annotations$Annotation) #1000 per page
  cids <- vector(mode="character",length=n_annos)

  for (i in 1:n_annos) {
    #for (i in 1:3) {
    l1 <- r$Annotations$Annotation[[i]]
    cid <- l1$LinkedRecords$CID
    if (is.null(cid)) {
      cids[i] <- NA
    } else {
      cids[i] <- l1$LinkedRecords$CID
    }
  }
  
  return(cids)
}

