# Functions for Extracting Annotations Data from PubChem
# Established to get Metabolites/Metabolism Information from HSDB (and others)
# As yet not thoroughly tested on other annotation sources or types
# Emma Schymanski, 1 May 2020 (based off earlier files from late Apr. 2020)
# In collaboration with PubChem (Evan Bolton, Paul Thiessen, Jeff Zhang)

#depends
library(RChemMass)
#setwd("C:/DATA/PubChem/")


#' Retrieve Number of Pages for Annotation JSON file
#' 
#' Retrieves the number of pages for annotations files. As this requires
#' retrieving the entire JSON file (up to 1000 entries), this can take 
#' some time (hence large default timeout). Thanks to Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcAnno.TotalPages(source, heading, timeout=100)
#' 
#' @param source Data source name of the annotations data, e.g. \code{"HSDB"}
#' @param heading Annotation category desired, e.g. \code{"Metabolism/Metabolites"}
#' @param timeout The timeout, in seconds.  
#' @return The number of pages
#' 
#' @details 
#' This is primarily a supporter function for retrieving full annotation data
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' PubChem Data Sources (with annotations):
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/#type=Annotations}
#' 
#' @seealso \code{\link{getMSInfo.files}}
#' 
#' @examples
#' n_pages <- getPcAnno.TotalPages('HSDB','Metabolism/Metabolites')
#' n_pages <- getPcAnno.TotalPages("NORMAN Suspect List Exchange","Transformations")
#' 
#' @export
getPcAnno.TotalPages <- function(source, heading, timeout=100) {
  # set URL
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source='
  midURL <- '&heading='
  endURL <- '&heading_type=Compound&page=1'
  url <- paste0(baseURL, source, midURL, heading, endURL)
  #Example:
  #https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source=NORMAN%20Suspect%20List%20Exchange&heading_type=Compound&heading=Transformations&page=1
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    #data <- getURL(URLencode(url),timeout=timeout),
    {
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  r <- fromJSON(data)
  
  if(!is.null(r$Fault)) {
    return(NA)
  }
  
  n_pages <- r$Annotations$TotalPages
  return(n_pages)
  
}


#' Retrieve Annotation Details from Metabolism/Metabolites Sections
#' 
#' Retrieve the annotation information from the Metabolism/Metabolites 
#' section of data sources, per page. As this requires
#' retrieving the entire JSON file (up to 1000 entries), this can take 
#' some time (hence large default timeout). Thanks to Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcAnno.Metabolism(source, heading, page=1, file_name="", timeout=100)
#' 
#' @param source Data source name of the annotations data, e.g. \code{"HSDB"}
#' @param heading Annotation category desired, e.g. \code{"Metabolism/Metabolites"}
#' @param page Page number to retrieve. Use \link{getPcAnno.TotalPages} to 
#' determine page numbers available
#' @param file_name If empty, \code{"heading_source_pageX.csv"} is created, 
#' overwriting any existing file of the same name in the directory. 
#' @param timeout The timeout, in seconds. Should be generous as these are large files
#' @return file_name of a saved multi-column CSV containing Source Name, Source ID, 
#' Source Chemical Name, Reference text, Description text and 
#' pipe-separated CIDs of the metabolites (\code{tp_cids}).
#' 
#' @details 
#' This function may work for other annotation headings, but has not been 
#' tested beyond Metabolism/Metabolites for HSDB.
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' PubChem Data Sources (with annotations):
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/#type=Annotations}
#' 
#' @seealso \code{\link{getPcAnno.TotalPages}}, \code{\link{getPcAnno.SourceCIDs}}
#' 
#' @examples
#' # This retrieves only the first page
#' getPcAnno.Metabolism(source = 'HSDB',heading = 'Metabolism/Metabolites')
#' # This will retrieve all (in single files)
#' n_pages <- getPcAnno.TotalPages('HSDB','Metabolism/Metabolites')
#' for (i in 1:n_pages) {
#'   getPcAnno.Metabolism(source = 'HSDB',heading = 'Metabolism/Metabolites', page=i)
#' }
#' # merge together
#' HSDB_data <- read.csv("Metabolism_Metabolites_HSDB_page1.csv")
#' HSDB_data2 <- read.csv("Metabolism_Metabolites_HSDB_page2.csv")
#' HSDB_data_all <- merge(HSDB_data,HSDB_data2,all.x=TRUE,all.y=TRUE)
#' 
#' for (i in 3:n_pages) {
#'     file_name <- paste0("Metabolism_Metabolites_HSDB_page",i,".csv")
#'     HSDB_data2 <- read.csv(file_name)
#'     HSDB_data_all <- merge(HSDB_data_all,HSDB_data2,all.x=TRUE,all.y=TRUE)
#'     }
#' 
#' @export
getPcAnno.Metabolism <- function(source, heading, page=1, file_name="", timeout=100) {
  # set file_name if not set...
  if (nchar(file_name) < 4) {
    file_name <- paste0(sub("/","_",heading), "_", source, "_page", page,".csv")
  }
  # set URL
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source='
  #source <- 'HSDB'
  midURL <- '&heading='
  #heading <- 'Metabolism/Metabolites'
  #endURL <- '&heading_type=Compound&page=1&response_type=display'
  endURL <- '&heading_type=Compound&page='
  url <- paste0(baseURL, source, midURL, heading, endURL, page)
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    #data <- getURL(URLencode(url),timeout=timeout),
    {
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  r <- fromJSON(data)
  
  if(!is.null(r$Fault)) {
    return(NA)
  }
  
  n_annos <- length(r$Annotations$Annotation) #1000 per page
  source_name <- vector(mode="character",length=n_annos)
  source_ID <- vector(mode="character",length=n_annos)
  source_chemical_name <- vector(mode="character",length=n_annos)
  desc_text <- vector(mode="character",length=n_annos)
  ref_text <- vector(mode="character",length=n_annos)
  tp_cids <- vector(mode="character",length=n_annos)
  n <- 1
  
  for (i in 1:n_annos) {
    #for (i in 1:3) {
    l1 <- r$Annotations$Annotation[[i]]
    n_refs <- length(l1$Data)
    
    for (k in 1:n_refs) {
      #print source name
      source_name[n] <- l1$SourceName #HSDB
      source_ID[n] <- l1$SourceID #ID from source
      source_chemical_name[n] <- l1$Name # name of chemical from source
      #extract next layer
      l2 <- l1$Data[[k]]
      if (length(l2$Reference)>0) {
        ref_text[n] <- l2$Reference[1] # reference
      }
      # get next layer
      l3 <- l2$Value$StringWithMarkup[[1]]
      n_cids <- length(l3$Markup)
      desc_text[n] <- l3$String # description mentioning metabolites ... 
      # loop over n_cids ... 
      cids <- ""
      if (n_cids > 0) {
        for (j in 1:n_cids) {
          cid <- sub("CID-","",l3$Markup[[j]]$Extra)
          if (j == 1) {
            cids <- cid
          } else {
            cids <- paste(cids,cid,sep="|")
          }
        }
      }
      tp_cids[n] <- cids
      n <- n+1
    }
  }
  anno_data <- cbind(source_name, source_ID, source_chemical_name, ref_text, desc_text, tp_cids)
  write.csv(anno_data, file_name ,row.names = F)
  return(file_name)
}




#' Retrieve SourceID-CID mappings for HSDB
#' 
#' Retrieve the sourceID-CID mappings for HSDB. As this requires
#' retrieving the entire JSON file (up to 1000 entries), this can take 
#' some time (hence large default timeout). Thanks to Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcAnno.SourceCIDs(source, heading, page=1, file_name="",timeout=100)
#' 
#' @param source Data source name of the annotations data, e.g. \code{"HSDB"}
#' @param heading Heading of the category desired, e.g. 
#' \code{"Hazardous%20Substances%20DataBank%20Number"}
#' @param page Page number to retrieve. Use \link{getPcAnno.TotalPages} to 
#' determine page numbers available
#' @param file_name If empty, \code{"heading_source_pageX.csv"} is created, 
#' overwriting any existing file of the same name in the directory. 
#' @param timeout The timeout, in seconds. Should be generous as these are large files
#' @return file_name of a saved multi-column CSV containing Source Name, Source ID, 
#' Source Chemical Name, the first CID (\code{cids}), the number of CIDs 
#' (\code{n_cids}), pipe-separated CIDs of the metabolites (\code{all_cids}).
#' 
#' @details 
#' Although written more generically, this function is a special case for HSDB, 
#' unless extended to other annotation sources in the future. 
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' HSDB Annotations:
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/11933#data=Annotations}
#' 
#' @seealso \code{\link{getPcAnno.TotalPages}}, \code{\link{getPcAnno.Metabolism}}
#' 
#' @examples
#' # This retrieves only the first page
#' getPcAnno.SourceCIDs('HSDB','Hazardous%20Substances%20DataBank%20Number')
#' # Get all data
#' n_source_pages <- getPcAnno.TotalPages('HSDB','Hazardous%20Substances%20DataBank%20Number')
#' getPcAnno.SourceCIDs('HSDB','Hazardous%20Substances%20DataBank%20Number',page=1)
#' HSDB_data <- read.csv("HazardousSubstancesDataBankNumber_HSDB_page1.csv")
#' for (i in 2:n_source_pages) {
#'   getPcAnno.SourceCIDs('HSDB','Hazardous%20Substances%20DataBank%20Number',page=i)
#' }
#' HSDB_data <- read.csv("HazardousSubstancesDataBankNumber_HSDB_page1.csv")
#' HSDB_data2 <- read.csv("HazardousSubstancesDataBankNumber_HSDB_page2.csv")
#' HSDB_data_all <- merge(HSDB_data,HSDB_data2,all.x=TRUE,all.y=TRUE)
#' for (i in 3:n_source_pages) {
#'   file_name <- paste0("HazardousSubstancesDataBankNumber_HSDB_page",i,".csv")
#'   HSDB_data2 <- read.csv(file_name)
#'   HSDB_data_all <- merge(HSDB_data_all,HSDB_data2,all.x=TRUE,all.y=TRUE)
#' }
#' 
#' write.csv(HSDB_data_all,"HazardousSubstancesDataBankNumber_HSDB_allpages.csv")
#' 
#' @export
getPcAnno.SourceCIDs <- function(source, heading, page=1, file_name="",timeout=100) {
  # set file_name if not set...
  if (nchar(file_name) < 4) {
    file_name <- paste0(gsub("%20","",heading,fixed=T), "_", source, "_page", page,".csv")
  }
  # set URL
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source='
  midURL <- '&heading='
  endURL <- '&heading_type=Compound&page='
  url <- paste0(baseURL, source, midURL, heading, endURL, page)
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
#    data <- getURL(URLencode(url),timeout=timeout),
    {
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  r <- fromJSON(data)
  
  if(!is.null(r$Fault)) {
    return(NA)
  }
  
  n_annos <- length(r$Annotations$Annotation) #1000 per page
  source_name <- vector(mode="character",length=n_annos)
  source_ID <- vector(mode="character",length=n_annos)
  source_chemical_name <- vector(mode="character",length=n_annos)
  cids <- vector(mode="character",length=n_annos)
  n_cids <- vector(mode="character",length=n_annos)
  all_cids <- vector(mode="character",length=n_annos)
  
  for (i in 1:n_annos) {
    #for (i in 1:3) {
    l1 <- r$Annotations$Annotation[[i]]
    source_name[i] <- l1$SourceName #HSDB
    source_ID[i] <- l1$SourceID #ID from source
    source_chemical_name[i] <- l1$Name # name of chemical from source
    cids_entry <- l1$LinkedRecords$CID
    if (length(cids_entry)>0) {
      cids[i] <- l1$LinkedRecords$CID[1]
      n_cids[i] <- length(cids_entry)
      all_cids[i] <- paste(cids_entry,collapse="|")
    }
  }
  anno_data <- cbind(source_name, source_ID, source_chemical_name, cids, n_cids, all_cids)
  #colnames(anno_data) 
  write.csv(anno_data, file_name ,row.names = F)
  return(file_name)
  
}




#' Retrieve Metabolite CIDs from Extracted Annotation Data 
#' 
#' Retrieve the metabolite CIDs from extracted annotation data and save
#' the extracted annotation information in a CSV. Thanks to Paul Thiessen, 
#' Jeff Zhang and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcAnno.TPcids(cid,anno_csv_name,tp_csv=TRUE)
#' 
#' @param cid PubChem CID to look up the metabolites, e.g. \code{2256}
#' @param anno_csv_name Name of the file containing the extracted annotation 
#' information, retrieved with \link{getPcAnno.TotalPages} 
#' @param tp_csv Default \code{TRUE} writes a CSV with the same name as
#' \code{anno_csv_name_CID.csv"} is created, 
#' overwriting any existing file of the same name in the directory. 
#' @return CIDs list and CSV file of exactly the same format as the input
#' file, only containing entries matching the input CID.
#' 
#' @details 
#' The files that are directly extracted do not contain the CIDs, these 
#' have to be merged. Function outstanding ... 
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' @seealso \code{\link{getPcAnno.Metabolism}}
#' 
#' @examples
#' #This retrieves only the first page (OK for atrazine)
#' anno_csv_name <- getPcAnno.Metabolism(source = 'HSDB',heading = 'Metabolism/Metabolites')
#' # retrieve the CIDs (saving a CSV as well)
#' getPcAnno.TPcids(2256,anno_csv_name)
#' 
#' @export
getPcAnno.TPcids <- function(cid,anno_csv_name,tp_csv=TRUE) {
  anno_data <- read.csv(anno_csv_name,stringsAsFactors = F)
  tps_index <- which(anno_data$cids==cid)
  if (length(tps_index)>0) {
    all_cids <- paste(anno_data$tp_cids[tps_index],collapse="|")
    #  unique_cids <- na.omit(suppressWarnings(as.numeric(unique(strsplit(all_cids,"|",fixed=T)[[1]]))))
    unique_cids <- suppressWarnings(as.numeric(unique(strsplit(all_cids,"|",fixed=T)[[1]])))
    if (tp_csv) {
      tp_file_name <- sub(".csv",paste0("_",cid,".csv"),anno_csv_name)
      anno_data <- anno_data[tps_index,]
      anno_data$selected_tp_cids <- anno_data$tp_cids
      anno_data$use <- FALSE
      anno_data$transformation <- ""
      anno_data$isPredecessor <- TRUE #most cases, the starting entry will be predecessor
      anno_data$comment <- ""
      write.csv(anno_data,tp_file_name,row.names = F)
    } else {
      tp_file_name <- NA
    }
  } else {
    tp_file_name <- NA
    unique_cids <- NA
  }
  results <- list()
  results[['TP_File_Name']] <- tp_file_name
  results[['TP_Unique_CIDs']] <- unique_cids
  return(results)
  # 
  # return(unique_cids)
}


#' Retrieve CIDs with Transformations Entries (by page)
#' 
#' Retrieve the CIDs with Transformations entries from the 
#' annotation information at the Transformation 
#' section of data sources, per page. As this requires
#' retrieving the entire JSON file (up to 1000 entries), this can take 
#' some time (hence large default timeout). Thanks to Paul Thiessen, 
#' Jeff Zhang and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcAnno.Transformations(source, heading, page=1, file_name="", timeout=100)
#' 
#' @param source Data source name of the annotations data, e.g. \code{"NORMAN Suspect List Exchange"}
#' @param heading Annotation category desired, e.g. \code{"Transformations"}
#' @param page Page number to retrieve. Use \link{getPcAnno.TotalPages} to 
#' determine page numbers available
#' @param timeout The timeout, in seconds. Should be generous as these are large files
#' @return The CIDs mentioned in the record.
#' 
#' @details 
#' This function will only extract CIDs and thus probably only works for
#' Transformations sections (and/or any other section where only CIDs are
#' needed)
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' PubChem Data Sources (with annotations):
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/#type=Annotations}
#' 
#' Transformations Example: 
#' \url{https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source=NORMAN%20Suspect%20List%20Exchange&heading_type=Compound&heading=Transformations&page=1}
#' 
#' @seealso \code{\link{getPcAnno.TotalPages}}, \code{\link{getPcAnno.Metabolism}}
#' 
#' @examples
#' # This retrieves only the CIDs from the first page
#' getPcAnno.Transformations(source = 'NORMAN Suspect List Exchange',heading = 'Transformations')
#' # This will retrieve all as a list of CIDs
#' source <- "NORMAN Suspect List Exchange"
#' heading <- "Transformations"
#' n_pages <- getPcAnno.TotalPages(source, heading)
#' for (i in 1:n_pages) {
#'   if (i == 1) {
#'     cids <- getPcAnno.Transformations(source, heading, page=i)
#'   } else {
#'     cids_px <- getPcAnno.Transformations(source, heading, page=i)
#'     cids <- c(cids,cids_px)
#'   }
#' }
#' # May 2021 this was 4730 CIDs
#' 
#' @export
getPcAnno.Transformations <- function(source, heading, page=1, timeout=100) {
  # set URL
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/rest/pug_view/annotations/heading/JSON/?source='
  #source <- 'HSDB'
  midURL <- '&heading='
  #heading <- 'Metabolism/Metabolites'
  #endURL <- '&heading_type=Compound&page=1&response_type=display'
  endURL <- '&heading_type=Compound&page='
  url <- paste0(baseURL, source, midURL, heading, endURL, page)
  
  errorvar <- 0
  currEnvir <- environment()
  
  tryCatch(
    #data <- getURL(URLencode(url),timeout=timeout),
    {
      res <- GET(URLencode(url))
      data <- httr::content(res, type="text", encoding="UTF-8")
    },
    error=function(e){
      currEnvir$errorvar <- 1
    })
  
  if(errorvar){
    return(NA)
  }
  
  r <- fromJSON(data)
  
  if(!is.null(r$Fault)) {
    return(NA)
  }
  
  n_annos <- length(r$Annotations$Annotation) #1000 per page
  cids <- vector(mode="character",length=n_annos)
  
  for (i in 1:n_annos) {
    #for (i in 1:3) {
    l1 <- r$Annotations$Annotation[[i]]
    cid <- l1$LinkedRecords$CID
    if (is.null(cid)) {
      cids[i] <- NA
    } else {
      cids[i] <- l1$LinkedRecords$CID
    }
  }
  
  return(cids)
}



#' Retrieve Transformations Table from PubChem Transformation Sections
#' 
#' Retrieve the transformation table (CSV) from the Transformation
#' section of individual compounds, by CID. Thanks to Jeff Zhang, Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcCand.trans(query, dataset="transformations",file_name="")
#' 
#' @param query CID to retrieve the transformation table, e.g. \code{2256}
#' @param dataset The name of the table behind the scenes, 
#' currently \code{"transformations"}. Other values will likely not work. 
#' @param file_name If empty, \code{"CID_query_dataset.csv"} is created, 
#' overwriting any existing file of the same name in the directory. 
#' @return file_name of a saved multi-column CSV containing the transformation
#' information. 
#' 
#' @details 
#' NOTE: The transformations section is under development and this function may
#' break unexpectedly at any point. Changes in format are also possible.
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' Example PubChem Data Source (with transformations):
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/23819#data=Annotations}
#' 
#' Example Transformations entry: 
#' \url{https://pubchem.ncbi.nlm.nih.gov/compound/2256#section=Transformations}
#' 
#' @examples
#' getPcCand.trans(1017)
#' getPcCand.trans(2256)
#' # an entry that doesn't have any transformation entry
#' getPcCand.trans(1234567)
#' 
#' @export
getPcCand.trans <- function(query, dataset="transformations",file_name="") {
  # set file_name if not set...
  if (nchar(file_name) < 4) {
    file_name <- paste0("CID_",query,"_",dataset,".csv")
  }
  
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/sdq/sdqagent.cgi?infmt=json&outfmt=csv&query={"download":"*","collection":"'
  #then dataset
  midURL1 <- '","where":{"ands":[{"cid":"'
  #then CID
  midURL2 <- '"}]},"order":["relevancescore,desc"],"start":1,"limit":10000000,"downloadfilename":"'
  #then file_name
  endURL <- '"}'
  url <- paste0(baseURL, dataset, midURL1, query, midURL2, file_name, endURL)
  con <- curl(url)
  content <- suppressWarnings(readLines(con,n=1))
  close(con)
  if (length(grep("Warning",content))==0) {
    download.file(url,file_name,quiet=T)
    print(paste0("CID_",query,"_",dataset,": download successful"))
    if (file.exists(file_name)) {
      tp_info <- read.csv(file_name,stringsAsFactors = F)
      unique_cids <- unique(c(tp_info$predecessorcid,tp_info$successorcid))
    } else {
      unique_cids <- NA
    }
    
  } else {
    print(paste0("CID_",query,"_",dataset,": ",content))
    unique_cids <- NA
  }
  
  results <- list()
  results[['TP_File_Name']] <- file_name
  results[['TP_Unique_CIDs']] <- unique_cids
  return(results)
  #return(file_name)
}


#' Retrieve All Data from PubChem Transformation Sections
#' 
#' Retrieve the combined transformation table (CSV) from the Transformation
#' section. Thanks to Jeff Zhang, Paul Thiessen, Siqian He 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcTrans(dataset="transformations",file_name="")
#' 
#' @param dataset The name of the SDQ table behind the scenes, 
#' currently \code{"transformations"}. Other values will likely not work. 
#' @param file_name If empty, \code{"dataset.csv"} is created, 
#' overwriting any existing file of the same name in the directory. 
#' @return file_name of a saved multi-column CSV containing the transformation
#' information. 
#' 
#' @details 
#' NOTE: The transformations section is under development and this function may
#' break unexpectedly at any point. Changes in format are also possible. 
#' It may also be possible to remove the "order" from the URL
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' Example PubChem Data Source (with transformations):
#' \url{https://pubchem.ncbi.nlm.nih.gov/source/23819#data=Annotations}
#' 
#' Example Transformations entry: 
#' \url{https://pubchem.ncbi.nlm.nih.gov/compound/2256#section=Transformations}
#' 
#' @examples
#' getPcTrans()
#' getPcTrans(dataset="transformations")
#' getPcTrans(dataset="chemblmetabolism")
#' 
#' @export
getPcTrans <- function(dataset="transformations",file_name="") {
  # set file_name if not set...
  if (nchar(file_name) < 4) {
    file_name <- paste0(dataset,".csv")
  }
  # super skinny URL (removing order) that's not yet implemented: 
  #https://pubchem.ncbi.nlm.nih.gov/sdq/sdqagent.cgi?infmt=json&outfmt=csv&query={"download":"*","collection":"transformations","start":1,"limit":10000000,"downloadfilename":"pubchem_cid_transformations"}
  
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/sdq/sdqagent.cgi?infmt=json&outfmt=csv&query={"download":"*","collection":"'
  #then dataset
  midURL1 <- '","order":["relevancescore,desc"],"start":1,"limit":10000000,"downloadfilename":"'
  #then file_name
  endURL <- '"}'
  url <- paste0(baseURL, dataset, midURL1, file_name, endURL)
  con <- curl(url)
  content <- suppressWarnings(readLines(con,n=1))
  close(con)
  if (length(grep("Warning",content))==0) {
    download.file(url,file_name,quiet=T)
    print(paste0(dataset,": download successful"))
    if (file.exists(file_name)) {
      tp_info <- read.csv(file_name,stringsAsFactors = F)
      unique_cids <- unique(c(tp_info$predecessorcid,tp_info$successorcid))
    } else {
      unique_cids <- NA
    }
    
  } else {
    print(paste0(dataset,": ",content))
    unique_cids <- NA
  }
  
  results <- list()
  results[['TP_File_Name']] <- file_name
  results[['TP_Unique_CIDs']] <- unique_cids
  return(results)
  #return(file_name)
}




#' Retrieve Download File from PubChem Queries (text-based search)
#' 
#' Download the results of text-based queries to PubChem. Thanks to Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getPcCand.query(query,file_name="")
#' 
#' @param query text-based query to search, e.g. \code{"coronavirus"}
#' @param file_name If empty, \code{"PubChem_Candidates_query.csv"} is created, 
#' overwriting any existing file of the same name in the directory. 
#' @return file_name of the multi-column CSV download file containing 
#' the search results. 
#' 
#' @details 
#' NOTE: This was designed to work off small queries and could download
#' huge files for large query terms...this has not been tested.
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' Example PubChem Text-Based Query:
#' \url{https://pubchem.ncbi.nlm.nih.gov/#query=coronavirus}
#' 
#' @examples
#' getPcCand.query("coronavirus","PC_CoronaQuery.csv")
#' 
#' @export
getPcCand.query <- function(query, file_name="") {
  # set file_name if not set...
  if (nchar(file_name) < 4) {
    file_name <- paste0("PubChem_Candidates_",query,".csv")
  }
  
  baseURL <- 'https://pubchem.ncbi.nlm.nih.gov/sdq/sdqagent.cgi?query={"download":"*","collection":"compound","where":{"ands":{"*":"'
  endURL <- '"}},"start":1,"limit":100000}'
  url <- paste0(baseURL, query, endURL)
  download.file(url,file_name)
  return(file_name)
}


#' Retrieve Information for Mass Spectral Data Processing from CIDs
#' 
#' Retrieve various pieces of information for mass spectral screening, 
#' by CID, via parent CID mappings. Thanks to Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getMSInfo.cid(cid,useParent=TRUE)
#' 
#' @param cid CID to retrieve information
#' @param useParent Default \code{TRUE} searches and returns info for
#' parent CID. If \code{FALSE}, retrieves on \code{cid}.
#' @return list containing the mass spectral screening information. 
#' 
#' @details 
#' This currently relies on functionality in RChemMass. 
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' @seealso \code{\link{getMSInfo.files}} \code{\link{getMSInfo.cids}}
#' 
#' @examples
#' getMSInfo.cid(2256)
#' # generates NAs when invalid CIDs are given
#' getMSInfo.cid("blah")
#' 
#' @export
getMSInfo.cid <- function(cid,useParent=TRUE) {
  Input_CID <- cid
  if (useParent) {
    Parent_CID <- getPCIDs.CIDtype(cid,type = "parent",timeout=60)
  } else {
    Parent_CID <- cid
  }
  
  # get properties
  extra_info <- getPCproperty.MF(Parent_CID)
  name_info <- getPCdesc.title(Parent_CID)
  # fill in the gaps
  Name <- name_info$Title
  IUPAC_Name <- extra_info$IUPACName
  SMILES <- extra_info$IsomericSMILES
  InChI <- extra_info$InChI
  InChIKey <- extra_info$InChIKey
  MolecularFormula <- extra_info$MolecularFormula
  ExactMass <- extra_info$MonoisotopicMass
  
  MSInfo <- list()
  MSInfo[['Input_CID']] <- Input_CID
  MSInfo[['Parent_CID']] <- Parent_CID
  MSInfo[['Name']] <- Name
  MSInfo[['IUPAC_Name']] <- IUPAC_Name
  MSInfo[['SMILES']] <- SMILES
  MSInfo[['InChI']] <- InChI
  MSInfo[['InChIKey']] <- InChIKey
  MSInfo[['MolecularFormula']] <- MolecularFormula
  MSInfo[['ExactMass']] <- ExactMass
  return(MSInfo)
}



#' Retrieve Information for Mass Spectral Data Processing from CIDs
#' 
#' Retrieve various pieces of information for mass spectral screening, 
#' by CID, via parent CID mappings. Thanks to Paul Thiessen 
#' and Evan Bolton from PubChem team for assistance. 
#' 
#' @usage getMSInfo.cids(cids,file_name)
#' 
#' @param cids Vector of CIDs to retrieve information
#' @param file_name File name to save the resulting information
#' @return file_name of the resulting multi-column CSV containing the 
#' mass spectral screening information. 
#' 
#' @details 
#' NOTE: This function goes through two series of web retrievals and
#' can take a while to run (several minutes for tens of CIDs)
#' Future improvement will be to retrieve by entire list not 1 by 1
#' This currently relies on functionality in RChemMass. 
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' @seealso \code{\link{getMSInfo.files}} \code{\link{getMSInfo.cid}}
#' 
#' @examples
#' test_cids <- c(2256, 124886, 70292631, 136096020, 136114911, NA)
#' # the last 3 cause various problems and have been included deliberately
#' test_file <- getMSInfo.cids(test_cids,"test_MSinfo.csv")
#' 
#' @export
getMSInfo.cids <- function(cids,file_name) {
  n_cids <- length(cids)
  Input_CID <- vector(mode="character",length=n_cids)
  Parent_CID <- vector(mode="character",length=n_cids)
  Name <- vector(mode="character",length=n_cids)
  IUPAC_Name <- vector(mode="character",length=n_cids)
  SMILES <- vector(mode="character",length=n_cids)
  InChI <- vector(mode="character",length=n_cids)
  InChIKey <- vector(mode="character",length=n_cids)
  MolecularFormula <- vector(mode="character",length=n_cids)
  ExactMass <- vector(mode="character",length=n_cids)
  
  for (i in 1:n_cids) {
    # save input
    Input_CID[i] <- cids[i]
    # get parent CID
    Parent_CID[i] <- getPCIDs.CIDtype(cids[i],type = "parent",timeout=60)
    # get properties
    extra_info <- getPCproperty.MF(Parent_CID[i])
    name_info <- getPCdesc.title(Parent_CID[i])
    # fill in the gaps
    Name[i] <- name_info$Title
    IUPAC_Name[i] <- extra_info$IUPACName
    SMILES[i] <- extra_info$IsomericSMILES
    InChI[i] <- extra_info$InChI
    InChIKey[i] <- extra_info$InChIKey
    MolecularFormula[i] <- extra_info$MolecularFormula
    ExactMass[i] <- extra_info$MonoisotopicMass
  }
  MSinfo <- cbind(Input_CID, Parent_CID, Name, IUPAC_Name, SMILES, InChI, InChIKey,
                  MolecularFormula, ExactMass)
  write.csv(MSinfo, file_name ,row.names = F)
  return(file_name)
}



#' Format getMSInfo.cids files to input files for Shinyscreen
#' 
#' Format the getMSInfo.cids output to create input files for 
#' Shinyscreen (compound list, set list) plus a suspect file. 
#' 
#' @usage getMSInfo.files(file_name, set_id,min_mass=50,id_offset=0)
#' 
#' @param file_name File name from \code{getMSInfo.cids}
#' @param set_id The name of the set ID for Shinyscreen. Keep short and 
#' informative, e.g. \code{"atrazine_tps"}
#' @param min_mass The minimum mass to include in the suspect screening. 
#' Default \code{50}; increase to \code{100} if only MS/MS is run 
#' at \code{mz=100-1000}
#' @param id_offset This can be used to adjust the ID number assigned
#' automatically (default starts at 1)
#' @return Silently returns three files named systematically off \code{file_name} 
#' 
#' @details 
#' NOTE: For maximum convenience, run this function directly in future 
#' Shinyscreen dir. 
#' 
#' @author Emma Schymanski <emma.schymanski@@uni.lu>
#' 
#' @references 
#' PubChem: \url{http://pubchem.ncbi.nlm.nih.gov/} 
#' 
#' @seealso \code{\link{getMSInfo.cids}}
#' 
#' @examples
#' test_cids <- c(2256, 124886, 70292631, 136096020, 136114911, NA)
#' # the last 3 cause various problems and have been included deliberately
#' test_file <- getMSInfo.cids(test_cids,"test_MSinfo.csv")
#' getMSInfo.files("test_MSinfo.csv",set_id="atrazine_tp_subset")
#' 
#' @export
getMSInfo.files <- function(file_name, set_id,min_mass=50,id_offset=0) {
  #this takes the output from getMSInfo.cids and creates files for Shinyscreen
  MS_info <- read.csv(file_name,stringsAsFactors = F)
  #check which are NAs - these are only ions - and remove
  check_nas <- which(is.na(MS_info$Parent_CID))
  if (length(check_nas)>0) {
    MS_info <- MS_info[-which(is.na(MS_info$Parent_CID)),]
  }
  #remove duplicates
  MS_info <- unique(MS_info)
  # remove duplicate parent CIDs that had different source CIDs
  dup_CID_test <- which(duplicated(MS_info$Parent_CID))
  if (length(dup_CID_test)>0) {
    MS_info <- MS_info[-which(duplicated(MS_info$Parent_CID)),]
  }
  #remove those below mz=min_mass ... 
  mass_test <- which(as.numeric(MS_info$ExactMass)>min_mass)
  if (length(mass_test)>0) {
    MS_info <- MS_info[mass_test,]
  }
  #remove any SMILES with wildcards
  star_smiles <- grep("*",MS_info$SMILES,fixed=T)
  if (length(star_smiles)>0) {
    MS_info <- MS_info[-grep("*",MS_info$SMILES,fixed=T),]
  }
  # transform into compound list for ShinyScreen
  # minimum fields ID (4-digit), Name, SMILES, RT (can be empty)
  MS_info$ID <- sequence(length(MS_info$Parent_CID))+id_offset
  MS_info$RT <- ""
  MS_info$set <- set_id
  suspect_filename <- paste0(set_id, "_SuspectList.csv")
  cmpdList_filename <- paste0(set_id, "_CmpdList.csv")
  set_filename <- paste0(set_id, "_Set.csv")
  #output CSV with all info
  write.csv(MS_info,suspect_filename,row.names = F)
  # output compound list for ShinyScreen
  #ID, Name, SMILES, RT
  write.csv(MS_info[,c(10,3,5,11)],cmpdList_filename,row.names = F)
  #create set file
  # ID, set_id
  write.csv(MS_info[,c(10,12)], set_filename,row.names = F)
}


# Create Transformations table entries from HSDB Directory
createTransFile.hsdb <- function(hsdb_dir, hsdb_structure_file, file_name="",
                                 hsdb_desc="", dataset_DOI="",dataset_desc="",
                                 MSInfo=FALSE) {
  # set the default output file name: 
  if (nchar(file_name) < 4) {
    file_name <- "S68_HSDBTPS_TransformationTable.csv"
  }
  # note: if MSInfo=TRUE, can go off the MSInfo file not the structure file
  
  # set the default descriptions
  if (nchar(hsdb_desc) < 4) {
    hsdb_desc <- paste0("HSDB is a toxicology database that focuses on the ",
                        "toxicology of potentially hazardous chemicals. See ",
                        "https://pubchem.ncbi.nlm.nih.gov/source/11933")
  }
  
  if (nchar(dataset_desc) < 4) {
    dataset_desc <- "Transformation Products extracted from HSDB content in PubChem, validated by LCSB-ECI"
  }
  
  if (nchar(dataset_DOI) < 4) {
    dataset_DOI <- "10.5281/zenodo.3827487"
  }
  
  # get the file listing
  hsdb_files <- list.files(hsdb_dir,pattern="selected.csv",full.names = T)
  hsdb_all_tp_info <- ""
  
  for (i in 1:length(hsdb_files)) {
    
    hsdb_selected_file <- hsdb_files[i]
    hsdb_info <- read.csv(hsdb_selected_file, stringsAsFactors = F)
    tp_cids <- getHSDBCIDs.selected(hsdb_info)
    if (length(tp_cids)>0) {
      hsdb_tp_info <- createTPinfo.hsdb(tp_cids, hsdb_selected_file, hsdb_structure_file, 
                                        hsdb_desc, dataset_DOI, dataset_desc,MSInfo)
    } else {
      #    print(paste0("No information to add for ", basename(hsdb_selected_file)))
      # run this just with the file CID, so it's on record that this was done
      hsdb_tp_info <- createTPinfo.hsdb(hsdb_info$cids[1], hsdb_selected_file, 
                                        hsdb_structure_file, 
                                        hsdb_desc, dataset_DOI, dataset_desc,MSInfo)
    }
    
    if (nchar(hsdb_all_tp_info[1]) == 0) {
      hsdb_all_tp_info <- hsdb_tp_info
    } else {
      if (length(tp_cids)>0) {
        hsdb_all_tp_info <- merge(hsdb_all_tp_info,hsdb_tp_info,all.x=T,all.y=T)
      }
    }
    
  }
  write.csv(hsdb_all_tp_info,file_name,row.names = F)
  return(file_name)
}


# This is a small helper function to get only the CIDs from 
# selected records in the manaully-checked HSDB-extracted records
getHSDBCIDs.selected <- function(hsdb_info) {
  hsdb_tp_cids <- hsdb_info$selected_tp_cids[which(hsdb_info$use==TRUE)]
  hsdb_tp_cids <- paste(hsdb_tp_cids,collapse="|")
  hsdb_tp_cids <- suppressWarnings(as.numeric(unique(strsplit(hsdb_tp_cids,"|",fixed=T)[[1]])))
  return(hsdb_tp_cids)
}


# a helper function to create the files for the #Transformations table
# from HSDB extracted-data. Although this is was originally set up to 
# export a csv, it now exports information for a larger function. 
createTPinfo.hsdb <- function(hsdb_cids, hsdb_selected_file, hsdb_structure_file, 
                              hsdb_desc, dataset_DOI, dataset_desc,MSInfo=FALSE) {
  # read in the information
  hsdb_info <- read.csv(hsdb_selected_file,stringsAsFactors = F)
  MS_info <- read.csv(hsdb_structure_file,stringsAsFactors = F)
  #select only those that we'll use
  hsdb_info <- hsdb_info[which(hsdb_info$use==T),]
  
  n_cids <- length(hsdb_cids)
  # initiate vectors
  Input_CID <- vector(mode="character",length=n_cids)
  Parent_CID <- vector(mode="character",length=n_cids)
  Name <- vector(mode="character",length=n_cids)
  IUPAC_Name <- vector(mode="character",length=n_cids)
  Predecessor_CID <- vector(mode="character",length=n_cids)
  Predecessor_Name <- vector(mode="character",length=n_cids)
  Successor_CID <- vector(mode="character",length=n_cids)
  Successor_Name <- vector(mode="character",length=n_cids)
  IsPredecessor <- vector(mode="logical",length=n_cids)
  Transformation <- vector(mode="character",length=n_cids)
  Source_ID <- vector(mode="character",length=n_cids)
  Source_CID <- vector(mode="character",length=n_cids)
  Source <- vector(mode="character",length=n_cids)
  Source_Description <- vector(mode="character",length=n_cids)
  Dataset_DOI <- vector(mode="character",length=n_cids)
  Dataset_Description <- vector(mode="character",length=n_cids)
  Evidence_ID <- vector(mode="character",length=n_cids)
  Evidence_Description <- vector(mode="character",length=n_cids)
  
  # take care of the starting CID. If it's in the TP list, remove.
  # we will have to treat this as a special case
  start_cid <- hsdb_info$cids[1]
  i_start_cid <- which(hsdb_cids==start_cid)
  if (length(i_start_cid)>0) {
    hsdb_cids <- hsdb_cids[-i_start_cid]
  }
  # count the number of CIDs now starting CID is removed
  n_cids <- length(hsdb_cids)
  # start the counter, the parent goes at n=1
  n <- 1
  # set the current CID to the start CID
  curr_cid <- start_cid
  #find the index of the start CID in MSInfo
  if (MSInfo) {
    i_MSInfo <- which(MS_info$Input_CID==start_cid)
  } else {
    i_MSInfo <- which(MS_info$PubChemCID==start_cid)
  }
  # if the info doesn't exist, reretrieve, otherwise, get from MS_info
  if (length(i_MSInfo)==0) {
    cid_MSInfo <- getMSInfo.cid(curr_cid)
    Input_CID[n] <- cid_MSInfo$Input_CID
    Parent_CID[n] <- cid_MSInfo$Parent_CID
    Name[n] <- cid_MSInfo$Name
    IUPAC_Name[n] <- cid_MSInfo$IUPAC_Name
  } else {
    if (MSInfo) {
      Input_CID[n] <- MS_info$Input_CID[i_MSInfo]
      Parent_CID[n] <- MS_info$Parent_CID[i_MSInfo]
    } else {
      Input_CID[n] <- MS_info$PubChemCID[i_MSInfo]
      Parent_CID[n] <- MS_info$PubChemCID[i_MSInfo]
    }
    Name[n] <- MS_info$Name[i_MSInfo]
    IUPAC_Name[n] <- MS_info$IUPAC_Name[i_MSInfo]
  }
  Predecessor_CID[n] <- Parent_CID[n]
  Predecessor_Name[n] <- Name[n]
  Successor_CID[n] <- Parent_CID[n]
  Successor_Name[n] <- Name[n]
  IsPredecessor[n] <- NA
  Transformation[n] <- "none"
  Source_ID[n] <- hsdb_info$source_ID[1]
  Source_CID[n] <- hsdb_info$cids[1]
  Source[n] <- hsdb_info$source_name[1]
  Source_Description[n] <- hsdb_desc
  Dataset_DOI[n] <- dataset_DOI
  Dataset_Description[n] <- dataset_desc
  Evidence_ID[n] <- ""
  Evidence_Description[n] <- ""
  
  # go over the rest of the TPs
  for (i in 1:n_cids) {
    # for each CID, find out which reference(s) it is in
    curr_cid <- hsdb_cids[i]
    tp_entries <- grep(curr_cid,hsdb_info$selected_tp_cids,fixed=T)
    if (length(tp_entries)>0) {
      for (j in 1:length(tp_entries)) {
        #add one more to the counter
        n <- n+1
        # get the index of the TP in hsdb_info
        i_tp_entry <- tp_entries[j]
        # get the index of the TP in MS_info
        if (MSInfo) {
          i_MSInfo <- which(MS_info$Input_CID==curr_cid)
        } else {
          i_MSInfo <- which(MS_info$PubChemCID==curr_cid)
        }
        #i_MSInfo <- which(MS_info$Input_CID==curr_cid)
        # if the value is not found, reretrieve the info, otherwise, save
        if (length(i_MSInfo)==0) {
          cid_MSInfo <- getMSInfo.cid(curr_cid)
          Input_CID[n] <- cid_MSInfo$Input_CID
          Parent_CID[n] <- cid_MSInfo$Parent_CID
          Name[n] <- cid_MSInfo$Name
          IUPAC_Name[n] <- cid_MSInfo$IUPAC_Name
        } else {
          if (MSInfo) {
            Input_CID[n] <- MS_info$Input_CID[i_MSInfo]
            Parent_CID[n] <- MS_info$Parent_CID[i_MSInfo]
          } else {
            Input_CID[n] <- MS_info$PubChemCID[i_MSInfo]
            Parent_CID[n] <- MS_info$PubChemCID[i_MSInfo]
          }
          Name[n] <- MS_info$Name[i_MSInfo]
          IUPAC_Name[n] <- MS_info$IUPAC_Name[i_MSInfo]
        }
        if (hsdb_info$isPredecessor[i_tp_entry]) {
          # if the starting CID is predecessor, this entry is a TP
          Predecessor_CID[n] <- Parent_CID[1]
          Predecessor_Name[n] <- Name[1]
          Successor_CID[n] <- Parent_CID[n]
          Successor_Name[n] <- Name[n]
          IsPredecessor[n] <- FALSE
        } else {
          # otherwise this entry is the predecessor
          Predecessor_CID[n] <- Parent_CID[n]
          Predecessor_Name[n] <- Name[n]
          Successor_CID[n] <- Parent_CID[1]
          Successor_Name[n] <- Name[1]
          IsPredecessor[n] <- TRUE
        }
        Transformation[n] <- hsdb_info$transformation[i_tp_entry]
        Source_ID[n] <- hsdb_info$source_ID[1]
        Source_CID[n] <- hsdb_info$cids[1]
        Source[n] <- hsdb_info$source_name[1]
        Source_Description[n] <- hsdb_desc
        Dataset_DOI[n] <- dataset_DOI
        Dataset_Description[n] <- dataset_desc
        Evidence_ID[n] <- ""
        Evidence_Description[n] <- hsdb_info$ref_text[i_tp_entry]
      }
    }
  }
  #PubChem_CID <- Parent_CID
  #Source_CID <- Input_CID
  # create the format for saving
  tp_info <- cbind(#Name, IUPAC_Name, PubChem_CID, SMILES, InChI, InChIKey,
    #MolecularFormula, ExactMass, Source_CID, 
    Predecessor_CID, Predecessor_Name,
    Successor_CID, Successor_Name, #IsPredecessor, 
    Transformation, Source_ID, Source_CID,
    Source, Source_Description, Dataset_DOI, Dataset_Description, 
    Evidence_ID, Evidence_Description)
  #remove duplicate entries
  tp_info <- unique(tp_info)
  #remove NAs if there was no parent CID
  na_entries <- which(is.na(tp_info[,2]))
  if (length(na_entries)>0) {
    tp_info <- tp_info[-which(is.na(tp_info[,2])),]
  }
  # #write the file and return the file name. 
  # write.csv(tp_info, file_name ,row.names = F)
  # return(file_name)
  return(tp_info)
  
}


# # superceded function version, kept for the record. 
# # clean out once other version is thoroughly tested
# createTPinfo.hsdb <- function(hsdb_cids, hsdb_selected_file, MSInfo_file, hsdb_desc, 
#                               dataset_DOI, dataset_desc,file_name="") {
#   # if no filename, autocreate from hsdb_selected_file
#   if (nchar(file_name) < 4) {
#     file_name <- sub(".csv","_allInfo.csv",hsdb_selected_file)
#   }
#   
#   # read in the information
#   hsdb_info <- read.csv(hsdb_selected_file,stringsAsFactors = F)
#   MS_info <- read.csv(MSInfo_file,stringsAsFactors = F)
#   #select only those that we'll use
#   hsdb_info <- hsdb_info[which(hsdb_info$use==T),]
#   
#   n_cids <- length(hsdb_cids)
#   # initiate vectors
#   Input_CID <- vector(mode="character",length=n_cids)
#   Parent_CID <- vector(mode="character",length=n_cids)
#   Predecessor_CID <- vector(mode="character",length=n_cids)
#   Predecessor_Name <- vector(mode="character",length=n_cids)
#   Successor_CID <- vector(mode="character",length=n_cids)
#   Successor_Name <- vector(mode="character",length=n_cids)
#   IsPredecessor <- vector(mode="logical",length=n_cids)
#   Transformation <- vector(mode="character",length=n_cids)
#   Source_ID <- vector(mode="character",length=n_cids)
#   Source_CID <- vector(mode="character",length=n_cids)
#   Source <- vector(mode="character",length=n_cids)
#   Source_Description <- vector(mode="character",length=n_cids)
#   Dataset_DOI <- vector(mode="character",length=n_cids)
#   Dataset_Description <- vector(mode="character",length=n_cids)
#   Evidence_ID <- vector(mode="character",length=n_cids)
#   Evidence_Description <- vector(mode="character",length=n_cids)
#   
#   # take care of the starting CID. If it's in the TP list, remove.
#   # we will have to treat this as a special case
#   start_cid <- hsdb_info$cids[1]
#   i_start_cid <- which(hsdb_cids==start_cid)
#   if (length(i_start_cid)>0) {
#     hsdb_cids <- hsdb_cids[-i_start_cid]
#   }
#   # count the number of CIDs now starting CID is removed
#   n_cids <- length(hsdb_cids)
#   # start the counter, the parent goes at n=1
#   n <- 1
#   # set the current CID to the start CID
#   curr_cid <- start_cid
#   #find the index of the start CID in MSInfo
#   i_MSInfo <- which(MS_info$Input_CID==start_cid)
#   # if the info doesn't exist, reretrieve, otherwise, get from MS_info
#   if (length(i_MSInfo)==0) {
#     cid_MSInfo <- getMSInfo.cid(curr_cid)
#     Input_CID[n] <- cid_MSInfo$Input_CID
#     Parent_CID[n] <- cid_MSInfo$Parent_CID
#     Name[n] <- cid_MSInfo$Name
#     IUPAC_Name[n] <- cid_MSInfo$IUPAC_Name
#   } else {
#     Input_CID[n] <- MS_info$Input_CID[i_MSInfo]
#     Parent_CID[n] <- MS_info$Parent_CID[i_MSInfo]
#     Name[n] <- MS_info$Name[i_MSInfo]
#     IUPAC_Name[n] <- MS_info$IUPAC_Name[i_MSInfo]
#   }
#   Predecessor_CID[n] <- Parent_CID[n]
#   Predecessor_Name[n] <- Name[n]
#   Successor_CID[n] <- Parent_CID[n]
#   Successor_Name[n] <- Name[n]
#   IsPredecessor[n] <- NA
#   Transformation[n] <- "none"
#   Source_ID[n] <- hsdb_info$source_ID[1]
#   Source_CID[n] <- hsdb_info$cids[1]
#   Source[n] <- hsdb_info$source_name[1]
#   Source_Description[n] <- hsdb_desc
#   Dataset_DOI[n] <- dataset_DOI
#   Dataset_Description[n] <- dataset_desc
#   Evidence_ID[n] <- ""
#   Evidence_Description[n] <- ""
#   
#   # go over the rest of the TPs
#   for (i in 1:n_cids) {
#     # for each CID, find out which reference(s) it is in
#     curr_cid <- hsdb_cids[i]
#     tp_entries <- grep(curr_cid,hsdb_info$selected_tp_cids,fixed=T)
#     if (length(tp_entries)>0) {
#       for (j in 1:length(tp_entries)) {
#         #add one more to the counter
#         n <- n+1
#         # get the index of the TP in hsdb_info
#         i_tp_entry <- tp_entries[j]
#         # get the index of the TP in MS_info
#         i_MSInfo <- which(MS_info$Input_CID==curr_cid)
#         # if the value is not found, reretrieve the info, otherwise, save
#         if (length(i_MSInfo)==0) {
#           cid_MSInfo <- getMSInfo.cid(curr_cid)
#           Input_CID[n] <- cid_MSInfo$Input_CID
#           Parent_CID[n] <- cid_MSInfo$Parent_CID
#           Name[n] <- cid_MSInfo$Name
#           IUPAC_Name[n] <- cid_MSInfo$IUPAC_Name
#         } else {
#           Input_CID[n] <- MS_info$Input_CID[i_MSInfo]
#           Parent_CID[n] <- MS_info$Parent_CID[i_MSInfo]
#           Name[n] <- MS_info$Name[i_MSInfo]
#           IUPAC_Name[n] <- MS_info$IUPAC_Name[i_MSInfo]
#         }
#         if (hsdb_info$isPredecessor[i_tp_entry]) {
#           # if the starting CID is predecessor, this entry is a TP
#           Predecessor_CID[n] <- Parent_CID[1]
#           Predecessor_Name[n] <- Name[1]
#           Successor_CID[n] <- Parent_CID[n]
#           Successor_Name[n] <- Name[n]
#           IsPredecessor[n] <- FALSE
#         } else {
#           # otherwise this entry is the predecessor
#           Predecessor_CID[n] <- Parent_CID[n]
#           Predecessor_Name[n] <- Name[n]
#           Successor_CID[n] <- Parent_CID[1]
#           Successor_Name[n] <- Name[1]
#           IsPredecessor[n] <- TRUE
#         }
#         Transformation[n] <- hsdb_info$transformation[i_tp_entry]
#         Source_ID[n] <- hsdb_info$source_ID[1]
#         Source_CID[n] <- hsdb_info$cids[1]
#         Source[n] <- hsdb_info$source_name[1]
#         Source_Description[n] <- hsdb_desc
#         Dataset_DOI[n] <- dataset_DOI
#         Dataset_Description[n] <- dataset_desc
#         Evidence_ID[n] <- ""
#         Evidence_Description[n] <- hsdb_info$ref_text[i_tp_entry]
#       }
#     }
#   }
#   #PubChem_CID <- Parent_CID
#   #Source_CID <- Input_CID
#   # create the format for saving
#   tp_info <- cbind(#Name, IUPAC_Name, PubChem_CID, SMILES, InChI, InChIKey,
#     #MolecularFormula, ExactMass, Source_CID, 
#     Predecessor_CID, Predecessor_Name,
#     Successor_CID, Successor_Name, #IsPredecessor, 
#     Transformation, Source_ID, Source_CID,
#     Source, Source_Description, Dataset_DOI, Dataset_Description, 
#     Evidence_ID, Evidence_Description)
#   #remove duplicate entries
#   tp_info <- unique(tp_info)
#   #remove NAs if there was no parent CID
#   na_entries <- which(is.na(tp_info[,2]))
#   if (length(na_entries)>0) {
#     tp_info <- tp_info[-which(is.na(tp_info[,2])),]
#   }
#   # #write the file and return the file name. 
#   # write.csv(tp_info, file_name ,row.names = F)
#   # return(file_name)
#   return(tp_info)
# }




#NOTE: This function is now superceded by createTPinfo.hsdb
createFiles.trans <- function(hsdb_cids, hsdb_selected_file, MSInfo_file, hsdb_desc, 
                              dataset_DOI, dataset_desc,file_name="",
                              startCIDisPredecessor=TRUE) {
  # if no filename, autocreate from hsdb_selected_file
  if (nchar(file_name) < 4) {
    file_name <- sub(".csv","_allInfo.csv",hsdb_selected_file)
  }
  
  # read in the information
  hsdb_info <- read.csv(hsdb_selected_file,stringsAsFactors = F)
  MS_info <- read.csv(MSInfo_file,stringsAsFactors = F)
  #select only those that we'll use
  hsdb_info <- hsdb_info[which(hsdb_info$use==T),]
  
  n_cids <- length(hsdb_cids)
  # initiate vectors
  Input_CID <- vector(mode="character",length=n_cids)
  Parent_CID <- vector(mode="character",length=n_cids)
  Name <- vector(mode="character",length=n_cids)
  IUPAC_Name <- vector(mode="character",length=n_cids)
  SMILES <- vector(mode="character",length=n_cids)
  InChI <- vector(mode="character",length=n_cids)
  InChIKey <- vector(mode="character",length=n_cids)
  MolecularFormula <- vector(mode="character",length=n_cids)
  ExactMass <- vector(mode="character",length=n_cids)
  Predecessor_CID <- vector(mode="character",length=n_cids)
  Successor_CID <- vector(mode="character",length=n_cids)
  IsPredecessor <- vector(mode="logical",length=n_cids)
  Transformation <- vector(mode="character",length=n_cids)
  Source <- vector(mode="character",length=n_cids)
  Source_Description <- vector(mode="character",length=n_cids)
  Dataset_DOI <- vector(mode="character",length=n_cids)
  Dataset_Description <- vector(mode="character",length=n_cids)
  Evidence_DOI <- vector(mode="character",length=n_cids)
  Evidence_Description <- vector(mode="character",length=n_cids)
  
  # take care of the starting CID. If it's in the TP list, remove. 
  # we will have to treat this as a special case
  start_cid <- hsdb_info$cids[1]
  i_start_cid <- which(hsdb_cids==start_cid)
  if (length(i_start_cid)>0) {
    hsdb_cids <- hsdb_cids[-i_start_cid]
  }
  # count the number of CIDs now starting CID is removed
  n_cids <- length(hsdb_cids)
  # start the counter, the parent goes at n=1
  n <- 1
  # set the current CID to the start CID
  curr_cid <- start_cid
  #find the index of the start CID in MSInfo
  i_MSInfo <- which(MS_info$Input_CID==start_cid)
  # if the info doesn't exist, reretrieve, otherwise, get from MS_info
  if (length(i_MSInfo)==0) {
    cid_MSInfo <- getMSInfo.cid(curr_cid)
    Input_CID[n] <- cid_MSInfo$Input_CID
    Parent_CID[n] <- cid_MSInfo$Parent_CID
    Name[n] <- cid_MSInfo$Name
    IUPAC_Name[n] <- cid_MSInfo$IUPAC_Name
    SMILES[n] <- cid_MSInfo$SMILES
    InChI[n] <- cid_MSInfo$InChI
    InChIKey[n] <- cid_MSInfo$InChIKey
    MolecularFormula[n] <- cid_MSInfo$MolecularFormula
    ExactMass[n] <- cid_MSInfo$ExactMass
  } else {
    Input_CID[n] <- MS_info$Input_CID[i_MSInfo]
    Parent_CID[n] <- MS_info$Parent_CID[i_MSInfo]
    Name[n] <- MS_info$Name[i_MSInfo]
    IUPAC_Name[n] <- MS_info$IUPAC_Name[i_MSInfo]
    SMILES[n] <- MS_info$SMILES[i_MSInfo]
    InChI[n] <- MS_info$InChI[i_MSInfo]
    InChIKey[n] <- MS_info$InChIKey[i_MSInfo]
    MolecularFormula[n] <- MS_info$MolecularFormula[i_MSInfo]
    ExactMass[n] <- MS_info$ExactMass[i_MSInfo]
  }
  Predecessor_CID[n] <- Parent_CID[n]
  Successor_CID[n] <- Parent_CID[n]
  IsPredecessor[n] <- startCIDisPredecessor
  Transformation[n] <- "none"
  Source[n] <- hsdb_info$source_name[1]
  Source_Description[n] <- hsdb_desc
  Dataset_DOI[n] <- dataset_DOI
  Dataset_Description[n] <- dataset_desc
  Evidence_DOI[n] <- ""
  Evidence_Description[n] <- ""
  
  # go over the rest of the TPs
  for (i in 1:n_cids) {
    # for each CID, find out which reference(s) it is in
    curr_cid <- hsdb_cids[i]
    tp_entries <- grep(curr_cid,hsdb_info$selected_tp_cids,fixed=T)
    if (length(tp_entries)>0) {
      for (j in 1:length(tp_entries)) {
        #add one more to the counter
        n <- n+1
        # get the index of the TP in hsdb_info
        i_tp_entry <- tp_entries[j]
        # get the index of the TP in MS_info
        i_MSInfo <- which(MS_info$Input_CID==curr_cid)
        # if the value is not found, reretrieve the info, otherwise, save
        if (length(i_MSInfo)==0) {
          cid_MSInfo <- getMSInfo.cid(curr_cid)
          Input_CID[n] <- cid_MSInfo$Input_CID
          Parent_CID[n] <- cid_MSInfo$Parent_CID
          Name[n] <- cid_MSInfo$Name
          IUPAC_Name[n] <- cid_MSInfo$IUPAC_Name
          SMILES[n] <- cid_MSInfo$SMILES
          InChI[n] <- cid_MSInfo$InChI
          InChIKey[n] <- cid_MSInfo$InChIKey
          MolecularFormula[n] <- cid_MSInfo$MolecularFormula
          ExactMass[n] <- cid_MSInfo$ExactMass
        } else {
          Input_CID[n] <- MS_info$Input_CID[i_MSInfo]
          Parent_CID[n] <- MS_info$Parent_CID[i_MSInfo]
          Name[n] <- MS_info$Name[i_MSInfo]
          IUPAC_Name[n] <- MS_info$IUPAC_Name[i_MSInfo]
          SMILES[n] <- MS_info$SMILES[i_MSInfo]
          InChI[n] <- MS_info$InChI[i_MSInfo]
          InChIKey[n] <- MS_info$InChIKey[i_MSInfo]
          MolecularFormula[n] <- MS_info$MolecularFormula[i_MSInfo]
          ExactMass[n] <- MS_info$ExactMass[i_MSInfo]
        }
        if (hsdb_info$isPredecessor[i_tp_entry]) {
          # if the starting CID is predecessor, this entry is a TP
          Predecessor_CID[n] <- Parent_CID[1]
          Successor_CID[n] <- Parent_CID[n]
          IsPredecessor[n] <- FALSE
        } else {
          # otherwise this entry is the predecessor
          Predecessor_CID[n] <- Parent_CID[n]
          Successor_CID[n] <- Parent_CID[1]
          IsPredecessor[n] <- TRUE
        }
        Transformation[n] <- hsdb_info$transformation[i_tp_entry]
        Source[n] <- hsdb_info$source_name[1]
        Source_Description[n] <- hsdb_desc
        Dataset_DOI[n] <- dataset_DOI
        Dataset_Description[n] <- dataset_desc
        Evidence_DOI[n] <- ""
        Evidence_Description[n] <- hsdb_info$ref_text[i_tp_entry]
      }
    }
  }
  PubChem_CID <- Parent_CID
  Source_CID <- Input_CID
  # create the format for saving
  tp_info <- cbind(Name, IUPAC_Name, PubChem_CID, SMILES, InChI, InChIKey,
                   MolecularFormula, ExactMass, Source_CID, 
                   Predecessor_CID, Successor_CID, #IsPredecessor, 
                   Transformation, Source, Source_Description, 
                   Dataset_DOI, Dataset_Description, Evidence_DOI, Evidence_Description)
  #remove duplicate entries
  tp_info <- unique(tp_info)
  #remove NAs if there was no parent CID
  na_entries <- which(is.na(tp_info[,2]))
  if (length(na_entries)>0) {
    tp_info <- tp_info[-which(is.na(tp_info[,2])),]
  }
  #write the file and return the file name. 
  write.csv(tp_info, file_name ,row.names = F)
  return(file_name)
  
}



# Missing:
# a function to merge CIDs by source ID - CID mappings
# a function to retrieve source ID / CIDs for data sources with substance depositions
# TODO:
# rewrite getMSInfo.cids to depend on getMSInfo.cid
