# A script to generate a MassBank.EU export file
# for PubChem. Generates MS/MS summary and compound summary. 
# Anjana Elapavalore, Emma Schymanski (LCSB-ECI)
# Jeff Zhang and Evan Bolton (PubChem) for annotation inputs
# Plus Ben Shoemaker and Paul Thiessen (PubChem) for deposition help!


library(ReSOLUTION)

#### Global Set-up ####

# Set the run name for the final files 
incl_date = TRUE
# default "YYYYMMDD"
run_name <- gsub("-","",Sys.Date(),fixed=T)
test <- FALSE
#test <- TRUE
#test_name <- "LCSB"
# test_name should be the folder the test is run on

if (test && incl_date) {
  csv_name <- paste0("MASSBANK_DATA_SUMMARY_MSMS_", test_name, "_", run_name,".csv")
} else if (incl_date) {
  csv_name <- paste0("MASSBANK_DATA_SUMMARY_MSMS_", run_name,".csv")
} else if (test) {
  csv_name <- paste0("MASSBANK_DATA_SUMMARY_MSMS_", test_name,".csv")
} else {
  csv_name <- "MASSBANK_DATA_SUMMARY_MSMS.csv"
}


# this is the path to cloned MB.EU-data
base_dir <- "~/GitHub/MassBank-data/"
#base_dir <- "~/Pubchem/test/"

# this is the path that becomes our reporting dir
MBEU_locn <- "~/Pubchem/MB-PBRelease_2023/November/"
if (!dir.exists(MBEU_locn)) {
  dir.create(MBEU_locn)
}
# set this as working dir so we don't save over MassBank-data
setwd(MBEU_locn)

msms_csv_file <- paste0(MBEU_locn, csv_name)
cmpd_csv_file <- sub(".csv","_Substances.csv",msms_csv_file)
unique_cmpd_csv_file <- sub(".csv","_Unique_Substances.csv",msms_csv_file)


# ReSOLUTION has now been updated so this is no longer needed if you have
# ReSOLUTION v0.1.10 from 19 May 2021 or newer
# # save the "getMBRecordPeaks_fix.R" into your MBEU_locn then run
# source("getMBRecordPeaks_fix.R")

#### MassBank EU Processes ####

# set the field codes to extract
field_codes <- c("ACCESSION", "DEPRECATED", 
                 "AUTHORS:","COPYRIGHT:", "LICENSE","PUBLICATION",
                 "CH$NAME", "CH$SMILES", 
                 #"CH$EXACT_MASS", "CH$FORMULA", 
                 "CH$IUPAC", "CH$LINK: INCHIKEY", 
                 #"CH$LINK: CAS", "CH$LINK: PUBCHEM CID", "CH$LINK: CHEMSPIDER",
                 "CH$COMPOUND_CLASS",
                 "AC$INSTRUMENT:",
                 "AC$INSTRUMENT_TYPE:",
                 "AC$MASS_SPECTROMETRY: MS_TYPE",
                 "AC$MASS_SPECTROMETRY: ION_MODE",
                 "AC$MASS_SPECTROMETRY: IONIZATION",
                 "AC$MASS_SPECTROMETRY: COLLISION_ENERGY", 
                 "AC$MASS_SPECTROMETRY: FRAGMENTATION_MODE",
                 "AC$CHROMATOGRAPHY: COLUMN_NAME", 
                 "AC$CHROMATOGRAPHY: RETENTION_TIME",
                 "MS$FOCUSED_ION: PRECURSOR_M/Z", 
                 "MS$FOCUSED_ION: PRECURSOR_TYPE",
                 "PK$SPLASH:")

getMBRecordInfo(base_dir,msms_csv_file,
                field_codes = field_codes,
                colNames = "field_codes")

record_info <- read.csv(msms_csv_file, stringsAsFactors = F)

# Remove Deprecated records

Deprecated_records <- record_info[!is.na(record_info$DEPRECATED), ]
record_info <- record_info[!record_info$DEPRECATED %in% Deprecated_records$DEPRECATED,]
record_info <- record_info[,-2]
write.csv(record_info,msms_csv_file,row.names = F)

# create new column
record_info$TOP5_PEAKS <- ""

# tidy up column headers
MSMS_colnames <- colnames(record_info)
MSMS_colnames
# # adjust
MSMS_colnames <- c("ACCESSION","AUTHORS","COPYRIGHT","LICENSE","PUBLICATION","Name","SMILES",
                   "InChI","InChIKey","COMPOUND_CLASS","INSTRUMENT","INSTRUMENT_TYPE",                  
                   "MS_TYPE","MS_ION_MODE","MS_IONIZATION","MS_COLLISION_ENERGY",
                   "MS_FRAGMENTATION_MODE","CHROMATOGRAPHY_COLUMN_NAME",
                   "CHROMATOGRAPHY_RETENTION_TIME","PRECURSOR_MZ",
                   "PRECURSOR_TYPE_ADDUCT","SPLASH","TOP5_PEAKS")

colnames(record_info) <- MSMS_colnames


## Fill missing InChI Key

SMILES_df <- record_info[is.na(record_info$InChIKey) & record_info$SMILES != "N/A", c("SMILES")]

# Create a new data frame with only the "SMILES" column
result_df <- data.frame(SMILES = SMILES_df)


i <- 1
for (i in 1:length(result_df[,1])) {
  if (is.na(record_info$InChIKey[i]) || record_info$InChIKey[i] == "NA") {
    record_info$InChIKey[i] <- getSuspectInChIKey(record_info$SMILES[i], record_info$Name[i])
  }
}

write.csv(record_info,msms_csv_file, row.names = FALSE)

i <- 1
accession_id <- record_info$ACCESSION[i]
msms_files <- list.files(base_dir,full.names = T,pattern=".txt",recursive = T)
msms_file <- msms_files[grep(accession_id,msms_files,fixed = T)]
#msms_file <- paste0(base_dir,accession_id,".txt")
file.exists(msms_file)

# testing the top 5
msms_peaks <- getMBRecordPeaks(msms_file,fixColNames = T)

int_order <- order(msms_peaks$Int,decreasing = T)

msms_peaks_sort <- msms_peaks[int_order,]

msms_peak_line <- paste(msms_peaks_sort$mz[1:5],msms_peaks_sort$relInt[1:5],sep=";")

msms_peak_line <- paste(msms_peak_line,collapse = ":")


# now, tidy up the Name, Ionization and get the Top 5 peaks
for (i in 1:length(record_info$ACCESSION)) {
  accession_id <- record_info$ACCESSION[i]
  #fix Name
  name <- record_info$Name[i]
  name_edited <- strsplit(name,"|",fixed=T)[[1]][1]
  record_info$Name[i] <- name_edited
  #fix Ionization
  ionization <- record_info$MS_IONIZATION[i]
  if (!is.null(ionization)) {
    ionization_edited <- strsplit(ionization,"|",fixed=T)[[1]][1]
    record_info$MS_IONIZATION[i] <- ionization_edited
  }
  
  # get Top 5 peaks
  #msms_file <- msms_files[grep(accession_id,msms_files,fixed = T)]
  msms_file <- msms_files[str_detect(msms_files, paste0(accession_id, ".txt"))]
  if (file.exists(msms_file)) {
    msms_peaks <- getMBRecordPeaks(msms_file,fixColNames = T)
    int_order <- order(msms_peaks$Int,decreasing = T)
    msms_peaks_sort <- msms_peaks[int_order,]
    msms_peak_line <- paste(msms_peaks_sort$mz[1:5],msms_peaks_sort$relInt[1:5],sep=";")
    msms_peak_line <- paste(msms_peak_line,collapse = ":")
    # add to record_info
    record_info$TOP5_PEAKS[i] <- msms_peak_line
  }
}


write.csv(record_info,msms_csv_file,row.names = F)


#### now work on the Compound Summary ####

# this pre-selects only SMILES, InChI and InChIKey
#cmpd_info <- unique(record_info[,c(5:7)])

cmpd_info <- unique(record_info[,c(7:9)])

# set column headers:
cmpd_colnames <- colnames(cmpd_info)
cmpd_colnames

cmpd_colnames <- c("PUBCHEM_EXT_DATASOURCE_SMILES",
                   "PUBCHEM_EXT_DATASOURCE_INCHI",
                   "PUBCHEM_EXT_DATASOURCE_REGID")
# # set to PubChem colnames
# cmpd_colnames[1] <- "PUBCHEM_EXT_DATASOURCE_SMILES"
# cmpd_colnames[2] <- "PUBCHEM_EXT_DATASOURCE_INCHI"
# # InChIKey is mapped to RegID
# cmpd_colnames[3] <- "PUBCHEM_EXT_DATASOURCE_REGID"

colnames(cmpd_info) <- cmpd_colnames


# looking at catching the bad cases
star_smiles <- grep("*",cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES,fixed=T)
# 30 cases ... remove them ... 
if (length(star_smiles)>0) {
  cmpd_info <- cmpd_info[-star_smiles,]
}

na_smiles <- grep("N/A",cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES,fixed=T)
# one entry, all NAs
cmpd_info[na_smiles,]
#remove
if (length(na_smiles)>0) {
  cmpd_info <- cmpd_info[-na_smiles,]
}
# check:
grep("N/A",cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES,fixed=T)

# check N/A InChIs ... but none left
grep("N/A",cmpd_info$PUBCHEM_EXT_DATASOURCE_INCHI,fixed=T)

# now we have NA InChIKeys that may have valid SMILES
# 29 entries
# # wrong test... catches only NA within valid keys
# na_IK <- grep("NA",cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID,fixed=T)
# grep("NA",cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID,fixed=T, value=T)

# real test ... 
na_IK <- which(is.na(cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID))
# if any NAs exist:
if (length(na_IK)>0) {
  # i <- 1
  # na_IK_SMILES <- cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES[na_IK[i]]
  # getSuspectInChIKey(na_IK_SMILES)
  
  for (i in 1:length(na_IK)) {
    if (!InChIKey_test(cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID[na_IK[i]])) {
      na_IK_SMILES <- cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES[na_IK[i]]
      new_IK <- getSuspectInChIKey(na_IK_SMILES)
      if (InChIKey_test(new_IK)) {
        cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID[na_IK[i]] <- new_IK
      }
    }
  }
}

# check that none remain: 
#grep("NA",cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID,fixed=T, value=T)
which(is.na(cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID))

#write.csv(cmpd_info,cmpd_csv_file,row.names = F)

# now remove the problematic SMILES

# smiles_badlist <- read.csv("SMILES_badlist_20210519.csv",
#                            stringsAsFactors = F,header=F)
smiles_badlist <- read.csv("SMILES_badlist_20210910.csv",
                           stringsAsFactors = F,header=F)

# #testing to see if this really works
# smiles_badlist[1,] <- "CC(C)C1=CC=C(NC(=O)N(C)C)C=C1"

i <- 1
for (i in 1:length(smiles_badlist[,1])) {
  locn <- grep(smiles_badlist[i,],cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES,fixed=T)
  if (length(locn)>0) {
    cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES[locn] <- ""
  }
}

write.csv(cmpd_info,cmpd_csv_file,row.names = F)

# last step ... remove duplication ... (won't validate with duplicate IKs)
unique_IK <- unique(cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID)
unique_InChI <- vector(mode="character",length=length(unique_IK))
unique_SMILES <- vector(mode="character",length=length(unique_IK))

i <- 1
for (i in 1:length(unique_IK)) {
  IK_ind <- grep(unique_IK[i],cmpd_info$PUBCHEM_EXT_DATASOURCE_REGID)[1]
  unique_InChI[i] <- cmpd_info$PUBCHEM_EXT_DATASOURCE_INCHI[IK_ind]
  unique_SMILES[i] <- cmpd_info$PUBCHEM_EXT_DATASOURCE_SMILES[IK_ind]
}

new_cmpd_info <- cbind(unique_IK,unique_InChI,unique_SMILES)

cmpd_colnames <- colnames(new_cmpd_info)
cmpd_colnames
# set to PubChem colnames
cmpd_colnames <- c("PUBCHEM_EXT_DATASOURCE_REGID","PUBCHEM_EXT_DATASOURCE_INCHI",
                   "PUBCHEM_EXT_DATASOURCE_SMILES")
# cmpd_colnames[3] <- "PUBCHEM_EXT_DATASOURCE_SMILES"
# cmpd_colnames[2] <- "PUBCHEM_EXT_DATASOURCE_INCHI"
# # InChIKey is mapped to RegID
# cmpd_colnames[1] <- "PUBCHEM_EXT_DATASOURCE_REGID"

colnames(new_cmpd_info) <- cmpd_colnames


write.csv(new_cmpd_info,unique_cmpd_csv_file,row.names = F)


# file should now be ready for deposition...

# Copy to \Misc\from_mb_to_pb
