# PubChemLite Evaluation Supporter Functions
# Emma Schymanski, 18 May 2020
# Based off evaluation scripts written starting approx Oct-Nov 2019

# # This script relies on functions also found here:
# user_file_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/user_PCLite_eval.R?inline=false"
# download.file(user_file_url,paste0(base_dir,"user_PCLite_eval.R"))
# source(paste0(base_dir,"user_PCLite_eval.R"))


# Run the CASMI Evaluation Set and create summary file.
runPCLiteEval.CASMI <- function(cmpd_file,localDB,run_name,metfrag_dir,msms_dir,
                                pre2020=FALSE,base_dir="",verbose=FALSE, 
                                runMetFrag=TRUE) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  # read in compound info
  cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  # Detect polarity: pos/neg are mixed in this dataset ... 
  isPos <- grepl(" POSITIVE",cmpd_info$ION_MODE) 
  
  #set up the summary file 
  if (pre2020) {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,FPSum"
    LocalCSVScoreWeights <- ",1,1,1"
  } else {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,AnnoTypeCount"
    LocalCSVScoreWeights <- ",1,1,1"
  }
  cmpd_info <- user_mk_cmpd_info(cmpd_info)
  
  for (i in 1:length(cmpd_info$SourceFile)) {
    cmpdID <- cmpd_info$SourceFile[i]
    #number at the end may be different, but 5 or 0 says whether neg or pos
    # so trim to 5, but not 4 ... 
    cmpdID_char <- strtrim(sub("XX","",cmpdID),5)
    #RT <- cmpd_info$rt
    ExactMass <- cmpd_info$PRECURSOR_MZ[i] #note this is charged
    # find the MS/MS file and deal with issues if not found
    msms_file <- list.files(msms_dir,pattern=cmpdID_char, full.names = T)
    msms_file_exists <- file.exists(msms_file)
    if (!msms_file_exists) {
      cmpd_info$msms_peaks[i] <- "N/A"
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      cmpd_info$msms_avail[i] <- TRUE
      msms_peaks <- getMBRecordPeaks(msms_file)[,c(1,2)]
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
      # write the peaks into the file
      msms_peak_line <- ""
      for (j in seq_along(msms_peaks[,c(1)])) {
        msms_peak_line <- paste((paste(msms_peaks[j,c(1,2)],collapse=":")),msms_peak_line ,sep=";")
      }
      cmpd_info$msms_peaks[i] <- msms_peak_line
    }
    
    # now, run MetFrag and extract results for reporting into cmpd_info.
    # LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,FPSum"
    # LocalCSVScoreWeights <- ",1,1,1"
    
    results_filename <- paste0(run_name,"_",cmpdID_char,"_",as.character(i))
    if (isPos[i]) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=FALSE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=FALSE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i),": ",config_file))
      }
    }
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    #file.exists(results_file)
    
    MetFrag_res <- read_excel(results_file)
    # cmpd_info[i,] <- user_fill_cmpd_info(MetFrag_res,cmpd_info[i,],
    #                                     pre2020 = T, IKmatch = T)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- user_fill_cmpd_info(MetFrag_res,cmpd_info[i,],
                                           pre2020 = pre2020, IKmatch = T,
                                           IK = cmpd_info$INCHIKEY[i])
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
    
  }
  
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}


## Making initial compound info file for the EA spectra, starting
## from the MS/MS directory of EA files ... 
mk_cmpd_info.EA <- function(EA_msms_dir,file_list="EA_filenames.csv") {
  # first, write file list
  write.csv(list.files(msms_dir),file_list,row.names=F)
  cmpd_info <- read.csv(file_list,stringsAsFactors = F)
  colnames(cmpd_info) <- c("Files")
  #file.exists(cmpd_file)
  
  #set up cmpd_info, 01 are neg, 02 are pos. 
  # each file has a header with info ... 
  # Sample: EAX027501
  # Charge: 1
  # NeutralPrecursorMass: 219.9699
  # Mode: 3
  ## RetentionTime: 7.0
  ## InchiKey: IWEDIXLBFLAXBO-UHFFFAOYSA-N
  ## ChemSpider: 2922
  ## PubChem: 3030
  ## Formula: C8H6Cl2O3
  
  # set up sample info ... 
  cmpd_info$Sample <- ""
  cmpd_info$Charge <- ""
  cmpd_info$NeutralPrecMass <- ""
  cmpd_info$Mode <- ""
  cmpd_info$RetentionTime <- ""
  cmpd_info$InChIKey <- ""
  cmpd_info$CSID <- ""
  cmpd_info$PCID <- ""
  cmpd_info$Formula <- ""
  return(cmpd_info)
  
}


## Evaluation function for the EA spectra, based off cmpd_info
## created using mk_cmpd_info.EA (above) and then user_mk_cmpd_info (user_PCLite_eval.R)
runPCLiteEval.EA <- function(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                             pre2020=FALSE,base_dir="",verbose=FALSE, 
                             runMetFrag=TRUE) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  
  # initialize variables
  isPos <- NA
  n_header <- 9
  CSID <- ""
  PCID <- ""
  
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  #set up the summary file 
  if (pre2020) {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,FPSum"
    LocalCSVScoreWeights <- ",1,1,1"
  } else {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,AnnoTypeCount"
    LocalCSVScoreWeights <- ",1,1,1"
  }
  
  for (i in 1:length(cmpd_info$Files)) {
    n_header <- 9
    cmpdID <- cmpd_info$Files[i]
    msms_file <- paste0(msms_dir,cmpdID)
    #1 or 2 at end says whether neg or pos, so trim to 6 not 4. 
    cmpdID_char <- strtrim(sub("EAX","",cmpdID),6)
    # find the MS/MS file and deal with issues if not found
    msms_file_exists <- file.exists(msms_file)
    if (!msms_file_exists) {
      cmpd_info$msms_peaks[i] <- "N/A"
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      #read in setting params
      msms_file_contents <- readLines(msms_file)
      comment_patterns <- c("# Sample: ","# Charge: ","# NeutralPrecursorMass: ",
                            "# Mode: ","## RetentionTime: ","## InchiKey: ",
                            "## ChemSpider: ","## PubChem: ","## Formula: ")
      cmpd_info$Sample[i] <- sub(comment_patterns[1],"",
                                 msms_file_contents[grep(comment_patterns[1],msms_file_contents)])
      cmpd_info$Charge[i] <- sub(comment_patterns[2],"",
                                 msms_file_contents[grep(comment_patterns[2],msms_file_contents)])
      cmpd_info$NeutralPrecMass[i] <- sub(comment_patterns[3],"",
                                          msms_file_contents[grep(comment_patterns[3],msms_file_contents)])
      ExactMass <- cmpd_info$NeutralPrecMass[i]
      cmpd_info$Mode[i] <- sub(comment_patterns[4],"",
                               msms_file_contents[grep(comment_patterns[4],msms_file_contents)])
      cmpd_info$RetentionTime[i] <- sub(comment_patterns[5],"",
                                        msms_file_contents[grep(comment_patterns[5],msms_file_contents)])
      cmpd_info$InChIKey[i] <- sub(comment_patterns[6],"",
                                   msms_file_contents[grep(comment_patterns[6],msms_file_contents)])
      CSID <- sub(comment_patterns[7],"",
                  msms_file_contents[grep(comment_patterns[7],msms_file_contents)])
      PCID <- sub(comment_patterns[8],"",
                  msms_file_contents[grep(comment_patterns[8],msms_file_contents)])
      if (length(CSID)>0) {
        cmpd_info$CSID[i] <- CSID
      } else {
        n_header <- n_header-1
      }
      if (length(PCID)>0) {
        cmpd_info$PCID[i] <- PCID
      } else {
        n_header <- n_header-1
      }
      cmpd_info$Formula[i] <- sub(comment_patterns[9],"",
                                  msms_file_contents[grep(comment_patterns[9],msms_file_contents)])
      
      #read in the peaks
      cmpd_info$msms_avail[i] <- TRUE
      msms_peaks <- read.table(msms_file,skip=n_header)
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
      msms_peak_line <- ""
      for (j in seq_along(msms_peaks[,c(1)])) {
        msms_peak_line <- paste((paste(msms_peaks[j,c(1,2)],collapse=":")),msms_peak_line ,sep=";")
      }
      cmpd_info$msms_peaks[i] <- msms_peak_line
    }
    
    #do pos/neg detection
    if (cmpd_info$Mode[i] == "3") {
      isPos <- FALSE
    } else {
      isPos <- TRUE
    }
    
    results_filename <- paste0(run_name,"_",cmpdID_char,"_",as.character(i))
    if (isPos) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i),": ",config_file))
      }
    }
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- user_fill_cmpd_info(MetFrag_res,cmpd_info[i,],
                                           pre2020 = pre2020, IKmatch = T,
                                           IK=as.character(cmpd_info$InChIKey[i]))
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
  }
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}  


## Making initial compound info file for the EQEx, EQExPlus, UF spectra, 
## starting from the MS/MS directory of respective files ... 
mk_cmpd_info.EQtoUF <- function(msms_dir,file_list="") {
  if (nchar(file_list) < 2) {
    file_list <- paste0(basename(msms_dir),"_filenames.csv")
  }
  # first, write file list
  write.csv(list.files(msms_dir),file_list,row.names=F)
  cmpd_info <- read.csv(file_list,stringsAsFactors = F)
  colnames(cmpd_info) <- c("Files")
  #file.exists(cmpd_file)
  
  # set up sample info ... 
  cmpd_info$Sample <- ""
  cmpd_info$isPos <- ""
  cmpd_info$NeutralPrecMass <- ""
  cmpd_info$PrecIonMode <- ""
  cmpd_info$SampleName <- ""
  cmpd_info$RetentionTime <- ""
  cmpd_info$InChIKey <- ""
  cmpd_info$InChI <- ""
  cmpd_info$CSID <- ""
  cmpd_info$PCID <- ""
  cmpd_info$Formula <- ""
  
  return(cmpd_info)
  
}

# Run PubChemLite evaluation for EQEx, EQExPlus and UF datasets. 
runPCLiteEval.EQtoUF <- function(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                                 EQ=TRUE,pre2020=FALSE,base_dir="", 
                                 verbose=FALSE,runMetFrag=TRUE) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  
  # initialize variables
  isPos <- NA
  n_header <- 11
  CSID <- ""
  PCID <- ""
  
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  #set up the summary file 
  if (pre2020) {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,FPSum"
    LocalCSVScoreWeights <- ",1,1,1"
  } else {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,AnnoTypeCount"
    LocalCSVScoreWeights <- ",1,1,1"
  }
  
  for (i in 1:length(cmpd_info$Files)) {
    n_header <- 11
    cmpdID <- cmpd_info$Files[i]
    msms_file <- paste0(msms_dir,cmpdID)
    #1 or 2 at end says whether neg or pos, so trim to 6 not 4. 
    if (EQ) {
      cmpdID_char <- strtrim(sub("EQX","",cmpdID),6)
    } else {
      cmpdID_char <- strtrim(sub("UFX","",cmpdID),6)
    }
    msms_file_exists <- file.exists(msms_file)
    if (!msms_file_exists) {
      cmpd_info$msms_peaks[i] <- "N/A"
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      #read in setting params
      msms_file_contents <- readLines(msms_file)
      comment_patterns <- c("# Sample: ","# IsPositiveIonMode: ",
                            "# NeutralPrecursorMass: ","# PrecursorIonMode: ",
                            "# SampleName: ","## InchiKey: ","## RetentionTime: ",
                            "## ChemSpider: ","## PubChem: ","## InChI: ",
                            "## NeutralPrecursorMolecularFormula: ")
      cmpd_info$Sample[i] <- sub(comment_patterns[1],"",
                                 msms_file_contents[grep(comment_patterns[1],msms_file_contents)])
      cmpd_info$isPos[i] <- sub(comment_patterns[2],"",
                                msms_file_contents[grep(comment_patterns[2],msms_file_contents)])
      cmpd_info$NeutralPrecMass[i] <- sub(comment_patterns[3],"",
                                          msms_file_contents[grep(comment_patterns[3],msms_file_contents)])
      ExactMass <- cmpd_info$NeutralPrecMass[i]
      cmpd_info$PrecIonMode[i] <- sub(comment_patterns[4],"",
                                      msms_file_contents[grep(comment_patterns[4],msms_file_contents)])
      cmpd_info$SampleName[i] <- sub(comment_patterns[5],"",
                                     msms_file_contents[grep(comment_patterns[5],msms_file_contents)])
      cmpd_info$InChIKey[i] <- sub(comment_patterns[6],"",
                                   msms_file_contents[grep(comment_patterns[6],msms_file_contents)])
      cmpd_info$RetentionTime[i] <- sub(comment_patterns[7],"",
                                        msms_file_contents[grep(comment_patterns[7],msms_file_contents)])
      CSID <- sub(comment_patterns[8],"",
                  msms_file_contents[grep(comment_patterns[8],msms_file_contents)])
      PCID <- sub(comment_patterns[9],"",
                  msms_file_contents[grep(comment_patterns[9],msms_file_contents)])
      if (length(CSID)>0) {
        cmpd_info$CSID[i] <- CSID
      } else {
        n_header <- n_header-1
      }
      if (length(PCID)>0) {
        cmpd_info$PCID[i] <- PCID
      } else {
        n_header <- n_header-1
      }
      cmpd_info$InChI[i] <- sub(comment_patterns[10],"",
                                msms_file_contents[grep(comment_patterns[10],msms_file_contents)])
      cmpd_info$Formula[i] <- sub(comment_patterns[11],"",
                                  msms_file_contents[grep(comment_patterns[11],msms_file_contents)])
      
      #read in the peaks
      cmpd_info$msms_avail[i] <- TRUE
      #msms_peaks <- getMBRecordPeaks(msms_file)[,c(1,2)]
      msms_peaks <- read.table(msms_file,skip=n_header)
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
      # write out peaks into cmpd_info
      msms_peak_line <- ""
      for (j in seq_along(msms_peaks[,c(1)])) {
        msms_peak_line <- paste((paste(msms_peaks[j,c(1,2)],collapse=":")),msms_peak_line ,sep=";")
      }
      cmpd_info$msms_peaks[i] <- msms_peak_line
    }
    
    #do pos/neg detection
    isPos <- as.logical(cmpd_info$isPos[i])
    
    results_filename <- paste0(run_name,"_",cmpdID_char,"_",as.character(i))
    if (isPos) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    # run MetFrag
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i),": ",config_file))
      }
    }
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- user_fill_cmpd_info(MetFrag_res,cmpd_info[i,],
                                           pre2020 = pre2020, IKmatch = T,
                                           IK=as.character(cmpd_info$InChIKey[i]))
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
  }
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}


# Run the CASMI Evaluation Set and create summary file for CompTox.
runCompToxEval.CASMI <- function(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                                 base_dir="",verbose=FALSE,runMetFrag=TRUE) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  # # read in compound info
  # cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  # Detect polarity: pos/neg are mixed in this dataset ... 
  isPos <- grepl(" POSITIVE",cmpd_info$ION_MODE) 
  
  #set up the summary file 
  LocalCSVScoreTerms <- "NUMBER_OF_PUBMED_ARTICLES,PUBCHEM_DATA_SOURCES,DATA_SOURCES"
  LocalCSVScoreWeights <- ",1,1,1"
  cmpd_info <- mk_cmpd_info.CompTox(cmpd_info)
  
  
  for (i in 1:length(cmpd_info$SourceFile)) {
    cmpdID <- cmpd_info$SourceFile[i]
    #number at the end may be different, but 5 or 0 says whether neg or pos
    # so trim to 5, but not 4 ... 
    cmpdID_char <- strtrim(sub("XX","",cmpdID),5)
    #RT <- cmpd_info$rt
    ExactMass <- cmpd_info$PRECURSOR_MZ[i] #note this is charged
    # find the MS/MS file and deal with issues if not found
    msms_file <- list.files(msms_dir,pattern=cmpdID_char, full.names = T)
    msms_file_exists <- file.exists(msms_file)
    if (!msms_file_exists) {
      cmpd_info$msms_peaks[i] <- "N/A"
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      cmpd_info$msms_avail[i] <- TRUE
      msms_peaks <- getMBRecordPeaks(msms_file)[,c(1,2)]
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
      # write out peaks into cmpd_info
      msms_peak_line <- ""
      for (j in seq_along(msms_peaks[,c(1)])) {
        msms_peak_line <- paste((paste(msms_peaks[j,c(1,2)],collapse=":")),msms_peak_line ,sep=";")
      }
      cmpd_info$msms_peaks[i] <- msms_peak_line
    }
    
    # now, run MetFrag and extract results for reporting into cmpd_info.
    results_filename <- paste0(run_name,"_",cmpdID_char,"_",as.character(i))
    if (isPos[i]) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=FALSE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=FALSE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    
    # run MetFrag
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i),": ",config_file))
      }
    }
    
    #runMetFrag(config_file, metfrag_path, CL_name = "MetFrag2.4.5-CL.jar")
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    #file.exists(results_file)
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- fill_cmpd_info.CompTox(MetFrag_res,cmpd_info[i,], IKmatch = T,
                                              IK=as.character(cmpd_info$INCHIKEY[i]))
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
    
  }
  
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}


# Run the EA Evaluation Set and create summary file for CompTox.
runCompToxEval.EA <- function(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                              base_dir="",verbose=FALSE,runMetFrag=TRUE) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  #set up the summary file 
  LocalCSVScoreTerms <- "NUMBER_OF_PUBMED_ARTICLES,PUBCHEM_DATA_SOURCES,DATA_SOURCES"
  LocalCSVScoreWeights <- ",1,1,1"
  cmpd_info <- mk_cmpd_info.CompTox(cmpd_info)
  
  # go through the files
  for (i in 1:length(cmpd_info$Files)) {
    n_header <- 9
    cmpdID <- cmpd_info$Files[i]
    msms_file <- paste0(msms_dir,cmpdID)
    #1 or 2 at end says whether neg or pos, so trim to 6 not 4. 
    cmpdID_char <- strtrim(sub("EAX","",cmpdID),6)
    #RT <- cmpd_info$rt
    #ExactMass <- cmpd_info$PRECURSOR_MZ[i] #note this is charged
    # find the MS/MS file and deal with issues if not found
    #msms_file <- list.files(msms_dir,pattern=cmpdID_char, full.names = T)
    msms_file_exists <- file.exists(msms_file)
    if (!msms_file_exists) {
      cmpd_info$msms_peaks[i] <- "N/A"
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      #read in setting params
      msms_file_contents <- readLines(msms_file)
      comment_patterns <- c("# Sample: ","# Charge: ","# NeutralPrecursorMass: ",
                            "# Mode: ","## RetentionTime: ","## InchiKey: ",
                            "## ChemSpider: ","## PubChem: ","## Formula: ")
      cmpd_info$Sample[i] <- sub(comment_patterns[1],"",
                                 msms_file_contents[grep(comment_patterns[1],msms_file_contents)])
      cmpd_info$Charge[i] <- sub(comment_patterns[2],"",
                                 msms_file_contents[grep(comment_patterns[2],msms_file_contents)])
      cmpd_info$NeutralPrecMass[i] <- sub(comment_patterns[3],"",
                                          msms_file_contents[grep(comment_patterns[3],msms_file_contents)])
      ExactMass <- cmpd_info$NeutralPrecMass[i]
      cmpd_info$Mode[i] <- sub(comment_patterns[4],"",
                               msms_file_contents[grep(comment_patterns[4],msms_file_contents)])
      cmpd_info$RetentionTime[i] <- sub(comment_patterns[5],"",
                                        msms_file_contents[grep(comment_patterns[5],msms_file_contents)])
      cmpd_info$InChIKey[i] <- sub(comment_patterns[6],"",
                                   msms_file_contents[grep(comment_patterns[6],msms_file_contents)])
      CSID <- sub(comment_patterns[7],"",
                  msms_file_contents[grep(comment_patterns[7],msms_file_contents)])
      PCID <- sub(comment_patterns[8],"",
                  msms_file_contents[grep(comment_patterns[8],msms_file_contents)])
      if (length(CSID)>0) {
        cmpd_info$CSID[i] <- CSID
      } else {
        n_header <- n_header-1
      }
      if (length(PCID)>0) {
        cmpd_info$PCID[i] <- PCID
      } else {
        n_header <- n_header-1
      }
      cmpd_info$Formula[i] <- sub(comment_patterns[9],"",
                                  msms_file_contents[grep(comment_patterns[9],msms_file_contents)])
      
      #read in the peaks
      cmpd_info$msms_avail[i] <- TRUE
      #msms_peaks <- getMBRecordPeaks(msms_file)[,c(1,2)]
      msms_peaks <- read.table(msms_file,skip=n_header)
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
      # write out peaks into cmpd_info
      msms_peak_line <- ""
      for (j in seq_along(msms_peaks[,c(1)])) {
        msms_peak_line <- paste((paste(msms_peaks[j,c(1,2)],collapse=":")),msms_peak_line ,sep=";")
      }
      cmpd_info$msms_peaks[i] <- msms_peak_line
    }
    
    # now, run MetFrag and extract results for reporting into cmpd_info.
    #do pos/neg detection
    if (cmpd_info$Mode[i] == "3") {
      isPos <- FALSE
    } else {
      isPos <- TRUE
    }
    
    results_filename <- paste0(run_name,"_",cmpdID_char,"_",as.character(i))
    if (isPos) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i),": ",config_file))
      }
    }
    
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- fill_cmpd_info.CompTox(MetFrag_res,cmpd_info[i,], IKmatch = T,
                                              IK=as.character(cmpd_info$InChIKey[i]))
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
    
    
  }
  
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}


# Run CompTox evaluation for EQEx, EQExPlus and UF datasets. 
runCompToxEval.EQtoUF <- function(cmpd_info,localDB,run_name,metfrag_dir,msms_dir,
                                  EQ=TRUE,base_dir="",verbose=FALSE,runMetFrag=TRUE) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  
  # initialize variables
  isPos <- NA
  n_header <- 11
  CSID <- ""
  PCID <- ""
  
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  #set up the summary file 
  LocalCSVScoreTerms <- "NUMBER_OF_PUBMED_ARTICLES,PUBCHEM_DATA_SOURCES,DATA_SOURCES"
  LocalCSVScoreWeights <- ",1,1,1"
  cmpd_info <- mk_cmpd_info.CompTox(cmpd_info)
  
  for (i in 1:length(cmpd_info$Files)) {
    n_header <- 11
    cmpdID <- cmpd_info$Files[i]
    msms_file <- paste0(msms_dir,cmpdID)
    #1 or 2 at end says whether neg or pos, so trim to 6 not 4. 
    if (EQ) {
      cmpdID_char <- strtrim(sub("EQX","",cmpdID),6)
    } else {
      cmpdID_char <- strtrim(sub("UFX","",cmpdID),6)
    }
    msms_file_exists <- file.exists(msms_file)
    if (!msms_file_exists) {
      cmpd_info$msms_peaks[i] <- "N/A"
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      #read in setting params
      msms_file_contents <- readLines(msms_file)
      comment_patterns <- c("# Sample: ","# IsPositiveIonMode: ",
                            "# NeutralPrecursorMass: ","# PrecursorIonMode: ",
                            "# SampleName: ","## InchiKey: ","## RetentionTime: ",
                            "## ChemSpider: ","## PubChem: ","## InChI: ",
                            "## NeutralPrecursorMolecularFormula: ")
      cmpd_info$Sample[i] <- sub(comment_patterns[1],"",
                                 msms_file_contents[grep(comment_patterns[1],msms_file_contents)])
      cmpd_info$isPos[i] <- sub(comment_patterns[2],"",
                                msms_file_contents[grep(comment_patterns[2],msms_file_contents)])
      cmpd_info$NeutralPrecMass[i] <- sub(comment_patterns[3],"",
                                          msms_file_contents[grep(comment_patterns[3],msms_file_contents)])
      ExactMass <- cmpd_info$NeutralPrecMass[i]
      cmpd_info$PrecIonMode[i] <- sub(comment_patterns[4],"",
                                      msms_file_contents[grep(comment_patterns[4],msms_file_contents)])
      cmpd_info$SampleName[i] <- sub(comment_patterns[5],"",
                                     msms_file_contents[grep(comment_patterns[5],msms_file_contents)])
      cmpd_info$InChIKey[i] <- sub(comment_patterns[6],"",
                                   msms_file_contents[grep(comment_patterns[6],msms_file_contents)])
      cmpd_info$RetentionTime[i] <- sub(comment_patterns[7],"",
                                        msms_file_contents[grep(comment_patterns[7],msms_file_contents)])
      CSID <- sub(comment_patterns[8],"",
                  msms_file_contents[grep(comment_patterns[8],msms_file_contents)])
      PCID <- sub(comment_patterns[9],"",
                  msms_file_contents[grep(comment_patterns[9],msms_file_contents)])
      if (length(CSID)>0) {
        cmpd_info$CSID[i] <- CSID
      } else {
        n_header <- n_header-1
      }
      if (length(PCID)>0) {
        cmpd_info$PCID[i] <- PCID
      } else {
        n_header <- n_header-1
      }
      cmpd_info$InChI[i] <- sub(comment_patterns[10],"",
                                msms_file_contents[grep(comment_patterns[10],msms_file_contents)])
      cmpd_info$Formula[i] <- sub(comment_patterns[11],"",
                                  msms_file_contents[grep(comment_patterns[11],msms_file_contents)])
      
      #read in the peaks
      cmpd_info$msms_avail[i] <- TRUE
      #msms_peaks <- getMBRecordPeaks(msms_file)[,c(1,2)]
      msms_peaks <- read.table(msms_file,skip=n_header)
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
      # write out peaks into cmpd_info
      msms_peak_line <- ""
      for (j in seq_along(msms_peaks[,c(1)])) {
        msms_peak_line <- paste((paste(msms_peaks[j,c(1,2)],collapse=":")),msms_peak_line ,sep=";")
      }
      cmpd_info$msms_peaks[i] <- msms_peak_line
    }
    
    #do pos/neg detection
    isPos <- as.logical(cmpd_info$isPos[i])
    
    results_filename <- paste0(run_name,"_",cmpdID_char,"_",as.character(i))
    if (isPos) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    # run MetFrag
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i),": ",config_file))
      }
    }
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- fill_cmpd_info.CompTox(MetFrag_res,cmpd_info[i,], IKmatch = T,
                                              IK=as.character(cmpd_info$InChIKey[i]))
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
  }
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}


#Run PubChemLite evaluation on the benchmarking set
# where MS/MS is present in the file
runPCLiteEval.BM <- function(cmpd_info,localDB,run_name,metfrag_dir,useMSMS=TRUE,
                             pre2020=FALSE,base_dir="",verbose=FALSE, 
                             runMetFrag=TRUE, useExactMass=TRUE,i_corr = 0,
                             extra_terms=c("none"),UDS_terms="",UDS_weights="",
                             CCSbase_adducts=c("none"),XlogP=FALSE) {
  # define CCS terms
  CCS_adducts <- c("[M+H]+","[M+K]+","[M+NH4]+","[M+Na-2H]-","[M+Na]+","[M-H]-",
                   "[M]+","[M]-") 
  
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  #set up the summary file 
  if (pre2020) {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,FPSum"
    LocalCSVScoreWeights <- ",1,1,1"
  } else if (nchar(UDS_terms)>1 && nchar(UDS_weights)>1) {
    LocalCSVScoreTerms <- UDS_terms
    LocalCSVScoreWeights <- UDS_weights
  } else {
    LocalCSVScoreTerms <- "PubMed_Count,Patent_Count,AnnoTypeCount"
    LocalCSVScoreWeights <- ",1,1,1"
  }
  
  for (i in 1:length(cmpd_info$PCIDs)) {
    # check if the MS/MS info is present and should be used
    if (useMSMS) {
      msms_file_exists <- cmpd_info$msms_avail[i]
    } else {
      msms_file_exists <- FALSE
    }
    # come up with the cases 
    if (!msms_file_exists) {
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      #read in MS/MS
      msms_file_contents <- cmpd_info$msms_peaks[i]
      #read in the peaks
      msms_peaks <- strsplit(msms_file_contents,";")[[1]]
      msms_peaks <- sub(":"," ", msms_peaks)
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    }
    
    #do pos/neg detection & find ExactMass
    isPos <- cmpd_info$isPos[i]
    ExactMass <- cmpd_info$ExactMass[i]
    MolForm <- cmpd_info$MolecularFormula[i]
    
    results_filename <- paste0(run_name,"_",as.character(i+i_corr))
    if (isPos && useExactMass) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else if (isPos && !useExactMass) { 
      config_file <- MetFragConfig.formula(mol_form = MolForm, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else if (!isPos && !useExactMass) { 
      config_file <- MetFragConfig.formula(mol_form = MolForm, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                           results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                           peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                           DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                           IsPosMode = FALSE,filter_by_InChIKey = T,
                                           UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
      
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i+i_corr),": ",config_file))
      }
    }
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- user_fill_cmpd_info(MetFrag_res,cmpd_info[i,],
                                           pre2020 = pre2020, IKmatch = T,
                                           IK=as.character(cmpd_info$InChIKey[i]),
                                           extra_terms=extra_terms,XlogP=XlogP)
      #add CCS adducts if present
      test_terms <- grep(TRUE, CCS_adducts %in% CCSbase_adducts)
      if(length(test_terms)>=1) {
        cmpd_info[i,] <- fill_cmpd_info.CCSbase(MetFrag_res,cmpd_info[i,],
                                                adducts=CCSbase_adducts)
      }
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
  }
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}  


# CompTox function
# Run CompTox evaluation for benchmarking dataset. 
runCompToxEval.BM <- function(cmpd_info,localDB,run_name,metfrag_dir,useMSMS=TRUE,
                              base_dir="",verbose=FALSE,runMetFrag=TRUE) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  #set up the summary file 
  LocalCSVScoreTerms <- "NUMBER_OF_PUBMED_ARTICLES,PUBCHEM_DATA_SOURCES,DATA_SOURCES"
  LocalCSVScoreWeights <- ",1,1,1"
  #cmpd_info <- mk_cmpd_info.CompTox(cmpd_info)
  
  for (i in 1:length(cmpd_info$PCIDs)) {
    # check if the MS/MS info is present and should be used
    if (useMSMS) {
      msms_file_exists <- cmpd_info$msms_avail[i]
    } else {
      msms_file_exists <- FALSE
    }
    # come up with the cases 
    if (!msms_file_exists) {
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      #read in MS/MS
      msms_file_contents <- cmpd_info$msms_peaks[i]
      #read in the peaks
      msms_peaks <- strsplit(msms_file_contents,";")[[1]]
      msms_peaks <- sub(":"," ", msms_peaks)
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    }
    
    #do pos/neg detection & find ExactMass
    isPos <- cmpd_info$isPos[i]
    ExactMass <- cmpd_info$ExactMass[i]
    
    results_filename <- paste0(run_name,"_",as.character(i))
    if (isPos) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "LocalCSV", localDB_path=localDB,useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T,
                                   UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    # run MetFrag
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i),": ",config_file))
      }
    }
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- fill_cmpd_info.CompTox(MetFrag_res,cmpd_info[i,], IKmatch = T,
                                              IK=as.character(cmpd_info$InChIKey[i]))
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
  }
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}




# Full PubChem function
# Run Full PubChem evaluation for benchmarking dataset. 
runFullPCEval.BM <- function(cmpd_info,run_name,metfrag_dir,useMSMS=TRUE,
                             base_dir="",verbose=FALSE,runMetFrag=TRUE,
                             i_corr = 0) {
  # set up base_dir to equal current dir if not given
  if (nchar(base_dir) < 2) {
    base_dir <- paste0(getwd(),"/")
  }
  # create dir for results
  results_run_dir <- paste0(base_dir,run_name)
  dir.create(results_run_dir)
  
  # # #set up the summary file - these terms do not get read in
  # LocalCSVScoreTerms <- "PubChemNumberPubMedReferences,PubChemNumberPatents"
  # LocalCSVScoreWeights <- ",1,1"

  for (i in 1:length(cmpd_info$PCIDs)) {
    # check if the MS/MS info is present and should be used
    if (useMSMS) {
      msms_file_exists <- cmpd_info$msms_avail[i]
    } else {
      msms_file_exists <- FALSE
    }
    # come up with the cases 
    if (!msms_file_exists) {
      msms_file <- paste0(base_dir,"/dummy_msms.txt")
      msms_peaks <- read.table(msms_file)
      # write peaks to MetFrag temp file
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    } else {
      #read in MS/MS
      msms_file_contents <- cmpd_info$msms_peaks[i]
      #read in the peaks
      msms_peaks <- strsplit(msms_file_contents,";")[[1]]
      msms_peaks <- sub(":"," ", msms_peaks)
      MetFrag_msms <- paste0(base_dir,"/MetFrag_temp_msms.txt")
      write.table(msms_peaks,MetFrag_msms,quote=F,row.names=F, col.names = F)
    }
    
    #do pos/neg detection & find ExactMass
    isPos <- cmpd_info$isPos[i]
    ExactMass <- cmpd_info$ExactMass[i]
    
    results_filename <- paste0(run_name,"_",as.character(i+i_corr))
    if (isPos) {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M+H]+", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "ExtendedPubChem", useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = TRUE,filter_by_InChIKey = T)#,add_refs = F,
                                   #UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    } else {
      config_file <- MetFragConfig(mass = ExactMass, adduct_type = "[M-H]-", neutralPrecursorMass=TRUE, 
                                   results_filename = results_filename, ppm=5, mzabs=0.001, frag_ppm=5, 
                                   peaklist_path = MetFrag_msms, base_dir = results_run_dir, 
                                   DB = "ExtendedPubChem", useMonaIndiv = T,useMoNAMetFusion = F, 
                                   IsPosMode = FALSE,filter_by_InChIKey = T)#,add_refs = F,
                                   #UDS_Category = LocalCSVScoreTerms, UDS_Weights = LocalCSVScoreWeights)
    }
    
    # run MetFrag
    if (runMetFrag) {
      runMetFrag(config_file, metfrag_dir, CL_name = "MetFrag2.4.5-CL.jar")
      if (verbose) {
        print(paste0("MetFrag complete for entry ",as.character(i+i_corr),": ",config_file))
      }
    }
    
    results_file <- paste0(results_run_dir,"/results/",results_filename,".xls")
    
    MetFrag_res <- read_excel(results_file)
    
    if (length(MetFrag_res)>0) {
      cmpd_info[i,] <- fill_cmpd_info.FullPC(MetFrag_res,cmpd_info[i,], IKmatch = T,
                                              IK=as.character(cmpd_info$InChIKey[i]))
    } else {
      cmpd_info$num_poss_IDs[i] <- 0
    }
  }
  #create summary csv
  summary_csv_name <- paste0("MetFragResultSummaryPubChemLite_",run_name,".csv")
  write.csv(cmpd_info,summary_csv_name,row.names = F)
  return(summary_csv_name)
  
}


#### plotting CCS values for candidates #####
plotCCS <- function(CCS_val_MpHp,CCS_val_MmHm,main_title) {
  # set the limits to data limits
  yrange <- c(min(CCS_val_MpHp,CCS_val_MmHm,na.rm = TRUE), 
              max(CCS_val_MpHp,CCS_val_MmHm,na.rm = TRUE))
  ylim <- c((yrange[1]-10),(yrange[2]+10))
  mean_CCS_val_MpHp <- mean(CCS_val_MpHp)
  #main <- "Predicted CCSbase for M+H+ and M-H- ions of Candidates"
  # plot
  plot(CCS_val_MpHp,ylim=ylim,main=main_title,xlab="Candidate Number",
       ylab="Predicted CCS (A2)",col="black")
  lines(x=c(1,length(CCS_val_MpHp)), y=c(mean_CCS_val_MpHp,mean_CCS_val_MpHp),
        lty="dashed",lwd=0.5)
  lines(x=c(1,length(CCS_val_MpHp)), y=0.97*c(mean_CCS_val_MpHp,mean_CCS_val_MpHp),
        lty="dotdash",lwd=0.5)
  lines(x=c(1,length(CCS_val_MpHp)), y=1.03*c(mean_CCS_val_MpHp,mean_CCS_val_MpHp),
        lty="dotdash",lwd=0.5)
  points(CCS_val_MmHm,col="red")
  legend("bottom", 
         legend=c("CCS [M+H]+","CCS [M-H]-","Mean CCS [M+H]+","3% from Mean CCS"),
         col=c("black","red","black","black"), 
         lty=c("blank","blank","dashed","dotdash"),
         pch=c(1,1,NA,NA),bty="n")
}


#### plotting XlogP values for candidates #####
plotXlogP <- function(XlogP_val,main_title) {
  # set the limits to data limits
  yrange <- c(min(XlogP_val,na.rm = TRUE), 
              max(XlogP_val,na.rm = TRUE))
  ylim <- c((yrange[1]-2),(yrange[2]+2))
  mean_XlogP <- mean(XlogP_val)
  #main_title <- "Predicted XlogP values of Candidates"
  # plot
  plot(XlogP_val,ylim=ylim,main=main_title,xlab="Candidate Number",
       ylab="XlogP",col="black")
  lines(x=c(1,length(XlogP_val)), y=c(mean_XlogP,mean_XlogP),
        lty="dashed",lwd=0.5)
  lines(x=c(1,length(XlogP_val)), y=c(mean_XlogP,mean_XlogP)+1,
        lty="dotdash",lwd=0.5)
  lines(x=c(1,length(XlogP_val)), y=c(mean_XlogP,mean_XlogP)-1,
        lty="dotdash",lwd=0.5)
  #points(CCS_val_MmHm,col=series_colour[2])
  legend("bottom", 
         legend=c("XlogP","Mean XlogP","Mean XlogP+/-1"),
         col=c("black","black","black"), 
         lty=c("blank","dashed","dotdash"),
         pch=c(1,NA,NA),bty="n")
}


#### plotting CCS and XlogP values for candidates #####
plotCCSnXlogP <- function(CCS_val,XlogP_val,main_title,topN = NA,
                          exp_CCS = NA, exp_XlogP = NA) {
  # set the limits to CCS data limits
  yrange <- c(min(CCS_val,na.rm = TRUE), max(CCS_val,na.rm = TRUE))
  ylim <- c((yrange[1]-10),(yrange[2]+40))
  mean_CCS_val <- mean(CCS_val)

    # set limits etc for XlogP
  yrangeXlogP <- c(min(XlogP_val,na.rm = TRUE), 
              max(XlogP_val,na.rm = TRUE))
  ylimXlogP <- c((yrangeXlogP[1]-8),(yrangeXlogP[2]+2))
  mean_XlogP <- mean(XlogP_val,na.rm = TRUE)

#  xlim=c(1,max(length(CCS_val_MmHm)+10,1.1*length(CCS_val_MmHm)))
  
  # select only top X
  if (!is.na(topN)) {
    if (topN<length(CCS_val)) {
      CCS_val <- CCS_val[1:topN]
      XlogP_val <- XlogP_val[1:topN]
    }
  }
  
  xlim=c(1,length(CCS_val))
  
  # plot CCS bits
  par(mar=c(5.1,4.1,4.1,3.1))
  plot(CCS_val,ylim=ylim,main=main_title,xlab="Candidate Number",
       ylab="Predicted CCS (A2)",col="black",xlim=xlim)
  if (is.na(exp_CCS)) {
    lines(x=c(1,length(CCS_val)), y=c(mean_CCS_val,mean_CCS_val),
          lty="dashed",lwd=0.5)
    lines(x=c(1,length(CCS_val)), y=0.97*c(mean_CCS_val,mean_CCS_val),
          lty="dotdash",lwd=0.5)
    lines(x=c(1,length(CCS_val)), y=1.03*c(mean_CCS_val,mean_CCS_val),
          lty="dotdash",lwd=0.5)
  } else {
    lines(x=c(1,length(CCS_val)), y=c(exp_CCS,exp_CCS),
          lty="dashed",lwd=0.5,col="red")
    lines(x=c(1,length(CCS_val)), y=0.97*c(exp_CCS,exp_CCS),
          lty="dotdash",lwd=0.5,col="red")
    lines(x=c(1,length(CCS_val)), y=1.03*c(exp_CCS,exp_CCS),
          lty="dotdash",lwd=0.5,col="red")
  }

  # add XlogP bits
  plot.window(xlim=xlim, ylim=ylimXlogP)
  points(XlogP_val,ylim=ylimXlogP,col="blue",pch=4)
  if (is.na(exp_XlogP)) {
    lines(x=c(1,length(XlogP_val)), y=c(mean_XlogP,mean_XlogP),
          lty="dashed",lwd=0.5,col="blue",ylim=ylimXlogP)
    lines(x=c(1,length(XlogP_val)), y=c(mean_XlogP,mean_XlogP)+1,
          lty="dotdash",lwd=0.5,col="blue",ylim=ylimXlogP)
    lines(x=c(1,length(XlogP_val)), y=c(mean_XlogP,mean_XlogP)-1,
          lty="dotdash",lwd=0.5,col="blue",ylim=ylimXlogP)
  } else {
    lines(x=c(1,length(XlogP_val)), y=c(exp_XlogP,exp_XlogP),
          lty="dashed",lwd=0.5,col="blue",ylim=ylimXlogP)
    lines(x=c(1,length(XlogP_val)), y=c(exp_XlogP,exp_XlogP)+1,
          lty="dotdash",lwd=0.5,col="blue",ylim=ylimXlogP)
    lines(x=c(1,length(XlogP_val)), y=c(exp_XlogP,exp_XlogP)-1,
          lty="dotdash",lwd=0.5,col="blue",ylim=ylimXlogP)
  }
  axis(4, col.axis="blue")
  mtext("XlogP",side=4,line=2,col="blue")
  
  if (is.na(exp_CCS) && is.na(exp_XlogP)) {
    legend_labels <- c("CCS","Mean CCS","3% from Mean CCS",
                       "XlogP","Mean XlogP","Mean XlogP+/-1")
    legend_colours <- c("black","black","black","blue","blue","blue")
  } else if (is.na(exp_CCS) && !is.na(exp_XlogP)) {
    legend_labels <- c("CCS","Mean CCS","3% from Mean CCS",
                       "XlogP","Exp. XlogP","Exp. XlogP+/-1")
    legend_colours <- c("black","black","black","blue","blue","blue")
  } else if (!is.na(exp_CCS) && is.na(exp_XlogP)) {
    legend_labels <- c("CCS","Exp. CCS","3% from Exp. CCS",
                       "XlogP","Mean XlogP","Mean XlogP+/-1")
    legend_colours <- c("black","red","red","blue","blue","blue")
  } else {
    legend_labels <- c("CCS","Exp. CCS","3% from Exp. CCS",
                       "XlogP","Exp. XlogP","Exp. XlogP+/-1")
    legend_colours <- c("black","red","red","blue","blue","blue")
  }
  
  legend("bottom", 
         legend=legend_labels,
         col=legend_colours, 
         lty=c("blank","dashed","dotdash","blank","dashed","dotdash"),
         pch=c(1,NA,NA,1,NA,NA),bty="n",cex=0.6,ncol=3)
}

