# Creating graphics in R for PubChemLite paper
# E. Schymanski, 7 March 2020 - 15 March 2020
# Modified 7 July 2020 with updated evaluations.
# Tweaked Oct 2020 for manuscript submission
# Updated early Nov with PubChemLite exposomics results 
# based off ideas from Datendesign mit R (book by Thomas Rahlf)

# set wd etc
# curr_dir <- "C:/DATA/PubChem/PubChemLite/Evaluation/"
# setwd(curr_dir)
curr_dir <- getwd()
# create a subdirectory within the current directory
fig_dir <- paste0(curr_dir,"/figures/")
if (!dir.exists(fig_dir)) {
  dir.create(fig_dir)
}
setwd(fig_dir)

#### Download data files for figures #####

fig_s1_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/figures/PCLite_Benchmarking.csv"
download.file(fig_s1_url,"PCLite_Benchmarking.csv")

fig_s2_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/figures/PCLitevCompTox_Benchmarking.csv"
download.file(fig_s2_url,"PCLitevCompTox_Benchmarking.csv")

fig_3_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/figures/PCLite_MergedBenchmarkEvalPlot.csv"
download.file(fig_3_url,"PCLite_MergedBenchmarkEvalPlot.csv")

fig_6_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/figures/PCLite_ScoreAnnoAgroEval.csv"
download.file(fig_6_url,"PCLite_ScoreAnnoAgroEval.csv")

fig_s3_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/figures/PCLite_MergedBenchmark_AnnotationEval.csv"
download.file(fig_s3_url,"PCLite_MergedBenchmark_AnnotationEval.csv")



#### Figure S1: Bar Chart for  PubChemLite Eval ####
# setting up labels
label_1 <- "MetFragRL Evaluation Set PubChem 2016\nMS/MS only (n=473)"
label_2 <- "CASMI2016 Category 2 ChemSpider 2016\nMS/MS only (n=208)"
label_3 <- "CASMI2016 Category 3 ChemSpider 2016\nMS/MS+RT+References (n=208)"
label_4 <- "CASMI2016 PubChemLite tier0\nMS/MS,References,Patents,FPSum (n=208)"
label_5 <- "CASMI2016 PubChemLite tier1\nMS/MS,References,Patents,FPSum (n=208)"
label_6 <- "MetFragRL Combined PubChem 2016\nMS/MS,RT,References,Patents (n=1298)"
label_7 <- "MetFragRL Combined PubChemLite tier0\nMS/MS,References,Patents,FPSum (n=1298)"
label_8 <- "MetFragRL Combined PubChemLite tier1\nMS/MS,References,Patents,FPSum (n=1298)"
label_names <- c(label_8, label_7, label_6, label_5, label_4, label_3, 
                 label_2,label_1)

#load data from table
BD_data <- read.csv("PCLite_Benchmarking.csv",row.names = c(1))

# now the figure
#Initiate PDF file, set parameters
pdf_file <- "FigS1_PCLiteBenchmarking.pdf"
cairo_pdf(pdf_file, bg="grey98", width=13, height=9)
par(omi=c(0.0, 0.75, 1.25, 0.75), mai=c(1.6, 3.75, 0.5, 0), lheight=1.15, 
    family ="Lato Leight", las=1)

##choose the colours!
ramp <- colorRamp(c("blue", "grey"))
plot_colours <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
plot_colours[1] <- "#00008B"

#base graphic
x <- barplot(as.matrix(BD_data), names.arg = label_names, cex.names=1.2, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours, axes=F)
# Building the legend ... 
px <- c(2,22,42,62,82)
py <- rep(10,5)
tx <- c(14,34,56,74,94)
ty <- rep(10,5)
points(px,py,pch=15,cex=4,col=plot_colours,xpd=T)
text(tx, ty, rownames(BD_data),adj=1, xpd=T, family="Lato Light", font=3, cex=1.2)
# x-axis labels
mtext(c(0,20,40,60,80,100), at=c(0,20,40,60,80,100),1,line=0,cex=1.2)
#Additional text elements
#Title of the whole graphic
mtext("Benchmarking of PubChemLite Against Previously Published Datasets",
      3,line=2.2,adj=0,cex=1.8,outer=T,family="Lato Black")
# bottom right note
mtext("All values in %",1,line=2,adj=1,cex=1.05,font=3)
#source comment right at the bottom
mtext("CASMI2016: Schymanski et al 2017; MetFragRL: Ruttkies et al 2016", 
      1,line=4.5,adj=1,cex=1.05,font=3)
# the number of data points statement
mtext("Note: Rank=2 and missing values unavailable for PubChem2016 data (see text)",
      1, line=2,adj=0, cex=1.05,family="Lato",font=3)

#close graphics device
dev.off()



#### Fig S2: Bar Chart for PubChemLite v CompTox####
# setting up labels
label_1 <- "All data (n=1506) PubChemLite tier0\nMS/MS,References,Patents,FPSum"
label_2 <- "All data (n=1506) PubChemLite tier1\nMS/MS,References,Patents,FPSum"
label_3 <- "All data (n=1506) CompTox March 2019\nMS/MS,PCSources,Data,References"
label_4 <- "No missing (n=1357) PubChemLite tier0\nMS/MS,References,Patents,FPSum"
label_5 <- "No missing (n=1357) PubChemLite tier1\nMS/MS,References,Patents,FPSum"
label_6 <- "No missing (n=1357) CompTox March 2019\nMS/MS,PCSources,Data,References"
label_names <- c(label_6, label_5, label_4, label_3, label_2, label_1)

#load data from table
BD_data <- read.csv("PCLitevCompTox_Benchmarking.csv",row.names = c(1))

# now the figure
#Initiate PDF file, set parameters
pdf_file <- "FigS2_PCLitevCompTox.pdf"
cairo_pdf(pdf_file, bg="grey98", width=13, height=8.5)
par(omi=c(0.0, 0.75, 1.25, 0.75), mai=c(1.6, 3.75, 0.5, 0), lheight=1.15, 
    family ="Lato Leight", las=1)

##choose the colours!
##flipping the data and plot colours
ramp <- colorRamp(c("blue", "grey"))
plot_colours <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
plot_colours[1] <- "#00008B"

#base graphic
x <- barplot(as.matrix(BD_data), names.arg = label_names, cex.names=1.2, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours, axes=F)
# Building the legend ... 
px <- c(2,22,42,62,82)
py <- rep(7.5,5)
tx <- c(14,34,56,74,94)
ty <- rep(7.5,5)
points(px,py,pch=15,cex=4,col=plot_colours,xpd=T)
text(tx, ty, rownames(BD_data),adj=1, xpd=T, family="Lato Light", font=3)
# x-axis labels
mtext(c(0,20,40,60,80,100), at=c(0,20,40,60,80,100),1,line=0,cex=1.2)
#Additional text elements
#Title of the whole graphic
mtext("PubChemLite versus CompTox: All Datasets Combined with MS/MS",
      3,line=2.2,adj=0,cex=1.8,outer=T,family="Lato Black")
# bottom right note
mtext("All values in %",1,line=2,adj=1,cex=1.05,font=3)
#source comment right at the bottom
mtext("Combined datasets from Schymanski et al 2017 and Ruttkies et al 2016", 
      1,line=4.5,adj=1,cex=1.05,font=3)
# the number of data points statement
mtext("Note: Missing values for CompTox may be overestimated due to tautomer forms",
      1, line=2,adj=0, cex=1.05,family="Lato",font=3)

#close graphics device
dev.off()

#### Fig 3: PCLite Benchmarking Set ####
# setting up labels
label_1 <- "CompTox 7 Mar 2019 (n=977)\nLiterature, # Sources, # PC Sources"
label_2 <- "PubChemLite tier0 12 Jun 2020 (n=977)\nMS/MS, Literature, Patents, AnnoTypeCount"
label_3 <- "PubChemLite tier0 22 May 2020 (n=977)\nMS/MS, Literature, Patents, AnnoTypeCount"
label_4 <- "PubChemLite tier0 14 Jan 2020 (n=977)\nMS/MS, Literature, Patents, AnnoTypeCount"
label_5 <- "PubChemLite tier0 18 Nov 2019 (n=977)\nMS/MS, Literature, Patents, FPSum"
label_6 <- "PubChemLite tier0 14 Jan 2020 (n=977)\nLiterature, Patents, AnnoTypeCount"
label_7 <- "CompTox 7 Mar 2019 (n=977)\nMS/MS, Literature, # Sources, # PC Sources"
label_8 <- "PubChemLite expo 31 Oct 2020 (n=977)\nMS/MS, Literature, Patents, AnnoTypeCount"
label_names <- c(label_8, label_2, label_3, label_4, label_5, label_7, label_6, label_1)

#load data from table
BD_data <- read.csv("PCLite_MergedBenchmarkEvalPlot.csv",row.names = c(1),
                    check.names = F)

# now the figure
#Initiate PDF file, set parameters
pdf_file <- "Fig3_PCLite_MergedEval.pdf"
cairo_pdf(pdf_file, bg="grey98", width=13, height=9)
par(omi=c(0.0, 0.75, 1.25, 0.75), mai=c(1.6, 3.75, 0.5, 0), lheight=1.15, 
    family ="Lato Leight", las=1)

##choose the colours!
##flipping the data and plot colours
ramp <- colorRamp(c("blue", "grey"))
plot_colours <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
plot_colours[1] <- "#00008B"

ramp <- colorRamp(c("darkgreen", "grey"))
plot_colours2 <- rgb( ramp(seq(0, 1, length = 5)), max = 255)


#base graphic
x <- barplot(t(as.matrix(BD_data)), names.arg = label_names, cex.names=1.2, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours2, axes=F)
y <- barplot(t(as.matrix(BD_data[c(1,2,3,4,5,6),])), names.arg = c("","","","","",""),
             cex.names=1.1, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours, axes=F, add=T)
# Building the legend ... 
px <- c(2,22,42,62,82)
py <- rep(10,5)
tx <- c(14,34,56.5,74,94)
ty <- rep(10,5)
points(px,py,pch=15,cex=4,col=plot_colours,xpd=T)
text(tx, ty, colnames(BD_data),adj=1, xpd=T, family="Lato Light", font=3, cex=1.2)
# x-axis labels
mtext(c(0,20,40,60,80,100), at=c(0,20,40,60,80,100),1,line=0,cex=1.2)
#Additional text elements
#Title of the whole graphic
mtext("PubChemLite (various dates) vs CompTox with MS/MS and Annotation Terms",
      3,line=2.2,adj=0,cex=1.8,outer=T,family="Lato Black")
# bottom right note
mtext("All values in %",1,line=2,adj=1,cex=1.05,font=3)
# #source comment right at the bottom
# mtext("Green: deduplicated compounds from (7,15). Blue: Agrochemicals", 
#       1,line=4.5,adj=1,cex=0.95,font=3)
# the number of data points statement
mtext("Blue: Includes MS/MS and annotation scores. Green: Annotation scores only.",
      1, line=2,adj=0, cex=1.05,family="Lato",font=3)

#close graphics device
dev.off()





#### Fig 6: Bar chart for Agrochemicals Evaluation ####
# Table 5 (needs to move to SI). 
# TODO: MS/MS data with Jan 2020 tier1 not Jun 2020 tier0. 
# setting up labels
label_2 <- "PubChemLite tier0 12 Jun 2020 Agro (n=318)\nMS/MS, Literature, Patents, AgroChemScore"
label_3 <- "PubChemLite tier1 14 Jan 2020 (n=1336)\nLiterature, Patents, AgroChemScore"
label_4 <- "PubChemLite tier1 14 Jan 2020 (n=1336)\nLiterature, Patents"
label_5 <- "Full PubChem Query 22 Jan 2020 (n=1336)\nLiterature, Patents"
label_names <- c(label_2, label_3, label_4, label_5)

#load data from table
BD_data <- read.csv("PCLite_ScoreAnnoAgroEval.csv",row.names = c(1),check.names = F)
BD_data <- BD_data[c(1:4),]

# now the figure
#Initiate PDF file, set parameters
pdf_file <- "Fig6_PCLite_AgroChemEval.pdf"
cairo_pdf(pdf_file, bg="grey98", width=13, height=7)
par(omi=c(0.0, 0.75, 1.25, 0.75), mai=c(1.6, 3.75, 0.5, 0), lheight=1.15, 
    family ="Lato Leight", las=1)

##choose the colours!
# blue
ramp <- colorRamp(c("blue", "grey"))
plot_colours <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
plot_colours[1] <- "#00008B"

# green
ramp <- colorRamp(c("darkgreen", "grey"))
plot_colours2 <- rgb( ramp(seq(0, 1, length = 5)), max = 255)


#base graphic
x <- barplot(t(as.matrix(BD_data)), names.arg = label_names, cex.names=1.2, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours2, axes=F)
# now plot the rest green
y <- barplot(t(as.matrix(BD_data[c(1),])), names.arg = c(""),
             cex.names=1.2, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours, axes=F, add=T)

# Building the legend ... 
px <- c(2,22,42,62,82)
py <- rep(5.2,5)
tx <- c(14,34,56.5,74,94)
ty <- rep(5.2,5)
points(px,py,pch=15,cex=4,col=plot_colours2,xpd=T)
text(tx, ty, colnames(BD_data),adj=1, xpd=T, family="Lato Light", font=3, cex=1.2)
# x-axis labels
mtext(c(0,20,40,60,80,100), at=c(0,20,40,60,80,100),1,line=0,cex=1.2)
#Additional text elements
#Title of the whole graphic
mtext("PubChemLite and PubChem with Agrochemicals, no MS/MS",
      3,line=2.2,adj=0,cex=1.8,outer=T,family="Lato Black")
# bottom right note
mtext("All values in %",1,line=2,adj=1,cex=1.05,font=3)
# #source comment right at the bottom
mtext("Green: Annotation only. Blue: MS/MS & annotation; subset of 977 benchmark set. ",
      1, line=2,adj=0, cex=1.05,family="Lato",font=3)

#close graphics device
dev.off()



#### Fig S3: Bar chart for Annotation Scores (with MS/MS) ####
# Tables not yet part of article
# setting up labels
label_1 <- "PubChemLite tier0 12 Jun 2020 Agro (n=318)\nMS/MS, Literature, Patents, AgroChemScore"
label_2 <- "PubChemLite tier0 12 Jun 2020 Drugs (n=466)\nMS/MS, Literature, Patents, DrugMedicScore"
label_3 <- "PubChemLite tier0 12 Jun 2020 Drugs (n=466)\nMS/MS, Literature, Patents, PharmacoInfoScore"
label_4 <- "PubChemLite tier0 12 Jun 2020 ToxInfo (n=708)\nMS/MS, Literature, Patents, ToxInfoScore"
#label_5 <- "Full PubChem Query 22 Jan 2020 (n=1336)\nLiterature, Patents"
label_names <- c(label_1, label_2, label_3, label_4)

#load data from table
BD_data <- read.csv("PCLite_MergedBenchmark_AnnotationEval.csv",row.names = c(1),check.names = F)

# now the figure
#Initiate PDF file, set parameters
pdf_file <- "FigS3_PCLiteBenchmark_AnnoEval.pdf"
cairo_pdf(pdf_file, bg="grey98", width=13, height=8)
par(omi=c(0.0, 0.75, 1.25, 0.75), mai=c(1.6, 3.75, 0.5, 0), lheight=1.15, 
    family ="Lato Leight", las=1)

##choose the colours!
##flipping the data and plot colours
ramp <- colorRamp(c("blue", "grey"))
plot_colours <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
plot_colours[1] <- "#00008B"

ramp <- colorRamp(c("darkgreen", "grey"))
plot_colours2 <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
#plot_colours[1] <- "#00008B"


#base graphic
x <- barplot(t(as.matrix(BD_data)), names.arg = label_names, cex.names=1.2, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours, axes=F)
# y <- barplot(t(as.matrix(BD_data[c(1,2,3),])), names.arg = c("","",""),
#              cex.names=1.1, horiz=T,
#              border=NA, xlim=c(0,100), col=plot_colours2, axes=F, add=T)
# Building the legend ... 
px <- c(2,22,42,62,82)
py <- rep(5.1,5)
tx <- c(14,34,56.5,74,94)
ty <- rep(5.1,5)
points(px,py,pch=15,cex=4,col=plot_colours,xpd=T)
text(tx, ty, colnames(BD_data),adj=1, xpd=T, family="Lato Light", font=3, cex=1.2)
# x-axis labels
mtext(c(0,20,40,60,80,100), at=c(0,20,40,60,80,100),1,line=0,cex=1.2)
#Additional text elements
#Title of the whole graphic
mtext("PubChemLite Benchmark with Various Annotation Categories & Scores, with MS/MS",
      3,line=2.2,adj=0,cex=1.8,outer=T,family="Lato Black")
# bottom right note
mtext("All values in %",1,line=2,adj=1,cex=1.05,font=3)
# #source comment right at the bottom
# mtext("Green: deduplicated compounds from (7,15). Blue: Agrochemicals", 
#       1,line=4.5,adj=1,cex=0.95,font=3)
# the number of data points statement
mtext("The benchmarking dataset was subsetted using indicated annotation content",
      1, line=2,adj=0, cex=1.05,family="Lato",font=3)

#close graphics device
dev.off()




#### Previous Fig 4: Bar chart for Annotation Scores alone (no MS/MS) ####
# Tables 4&5 combined. Now superceded with other plots
# setting up labels
label_1 <- "PubChemLite tier0 14 Jan 2020 (n=977)\nAnnoTypeCount, Literature, Patents"
label_2 <- "PubChemLite tier1 14 Jan 2020 (n=977)\nAnnoTypeCount, Literature, Patents"
label_3 <- "PubChemLite tier1 14 Jan 2020 (n=1336)\nAgrochemical_Score, Literature, Patents"
label_4 <- "PubChemLite tier1 14 Jan 2020 (n=1336)\nLiterature, Patents"
label_5 <- "Full PubChem Query 22 Jan 2020 (n=1336)\nLiterature, Patents"
label_names <- c(label_3, label_4, label_5, label_2, label_1)

#load data from table
BD_data <- read.csv("PCLite_ScoreAnnoAgroEval.csv",row.names = c(1),check.names = F)
BD_data <- BD_data[c(-1),]

# now the figure
#Initiate PDF file, set parameters
pdf_file <- "FigNN_PCLite_AnnoEval_superceded.pdf"
cairo_pdf(pdf_file, bg="grey98", width=13, height=8)
par(omi=c(0.0, 0.75, 1.25, 0.75), mai=c(1.6, 3.75, 0.5, 0), lheight=1.15, 
    family ="Lato Leight", las=1)

##choose the colours!
##flipping the data and plot colours
ramp <- colorRamp(c("blue", "grey"))
plot_colours <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
plot_colours[1] <- "#00008B"

ramp <- colorRamp(c("darkgreen", "grey"))
plot_colours2 <- rgb( ramp(seq(0, 1, length = 5)), max = 255)
#update first one to dark blue, as otherwise the last two colours are too similar
#plot_colours[1] <- "#00008B"


#base graphic
x <- barplot(t(as.matrix(BD_data)), names.arg = label_names, cex.names=1.2, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours, axes=F)
y <- barplot(t(as.matrix(BD_data[c(1,2,3),])), names.arg = c("","",""),
             cex.names=1.1, horiz=T,
             border=NA, xlim=c(0,100), col=plot_colours2, axes=F, add=T)
# Building the legend ... 
px <- c(2,22,42,62,82)
py <- rep(6.5,5)
tx <- c(14,34,56.5,74,94)
ty <- rep(6.5,5)
points(px,py,pch=15,cex=4,col=plot_colours,xpd=T)
text(tx, ty, colnames(BD_data),adj=1, xpd=T, family="Lato Light", font=3, cex=1.2)
# x-axis labels
mtext(c(0,20,40,60,80,100), at=c(0,20,40,60,80,100),1,line=0,cex=1.2)
#Additional text elements
#Title of the whole graphic
mtext("PubChemLite and PubChem with Various Annotation Categories, no MS/MS",
      3,line=2.2,adj=0,cex=1.8,outer=T,family="Lato Black")
# bottom right note
mtext("All values in %",1,line=2,adj=1,cex=1.05,font=3)
# #source comment right at the bottom
# mtext("Green: deduplicated compounds from (7,15). Blue: Agrochemicals", 
#       1,line=4.5,adj=1,cex=0.95,font=3)
# the number of data points statement
mtext("Blue: deduplicated compounds from (7,15). Green: Agrochemicals (14 Jan. 2020)",
      1, line=2,adj=0, cex=1.05,family="Lato",font=3)

#close graphics device
dev.off()

