# Evaluation Script for PubChemLite
# Emma Schymanski, 18 Nov 2019
# Revised 11 May 2020 and many dates since
# Now being modified to form Zenodo checks for auto-commit

###### Set up packages & folders ######

# Set up packages
# if (!requireNamespace("BiocManager", quietly = TRUE))
#   install.packages("BiocManager")
# BiocManager::install()

# ##If you don't have the latest RChemMass, reinstall:
# library(devtools)
# install_github("MassBank/RMassBank",dependencies=F)
# install_github("schymane/RChemMass", dependencies = F)
# install_github("schymane/ReSOLUTION", dependencies = F)
library(devtools)
library(RMassBank)
library(ReSOLUTION)
library(RChemMass)

# set up folders
# by default this will work off and create subfolders 
# in the current directory
base_dir <- getwd()
# Please adjust path as necessary for your chosen system
setwd(base_dir)

# This script requires that all necessary CSV files are saved here:
db_dir <- paste0(base_dir, "/MetFrag_CSVs/")
# To download necessary files, see "PCLite_eval_prepMetFrag.R"
# Please adjust path as necessary for your chosen system

# This script requires that the MetFrag jar is saved here:
metfrag_dir <- paste0(base_dir, "/MetFrag/")
# To download necessary files, see "PCLite_eval_prepMetFrag.R"
# Please adjust path as necessary for your chosen system

# get user_PCLite_eval.R file
user_file_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/user_PCLite_eval.R?inline=false"
download.file(user_file_url,paste0(base_dir,"user_PCLite_eval.R"))
source(paste0(base_dir,"user_PCLite_eval.R"))
# get PCLite_eval_support.R file
user_file_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/PCLite_eval_support.R?inline=false"
download.file(user_file_url,paste0(base_dir,"PCLite_eval_support.R"))
source(paste0(base_dir,"PCLite_eval_support.R"))
# get extractAnnotations.R file
extr_anno_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/annotations/tps/extractAnnotations.R?inline=false"
download.file(extr_anno_url,paste0(base_dir,"extractAnnotations.R"))
source(paste0(base_dir,"extractAnnotations.R"))

# get the PCLite Benchmarking file needed
zenodo_url <- "https://zenodo.org/record/4146957/files/"
file_url <- paste0(zenodo_url, "PCLite_Benchmark_wMSMS_info.csv")
download.file(file_url, paste0(base_dir,"PCLite_Benchmark_wMSMS_info.csv"))


##### MetFrag Preparation ######

# # To prepare files for MetFrag, please see "PCLite_eval_prepMetFrag.R"
# file_url <- "https://git-r3lab.uni.lu/eci/pubchem/-/raw/master/pubchemlite/R/PCLite_eval_prepMetFrag.R"
# download.file(file_url, paste0(base_dir,"PCLite_eval_prepMetFrag.R"))
# # please open script and run sections as needed
# # this includes MS/MS and mapping files needed for sections below. 


##### testing new PubChemLite Exposomics #####

## 31 Oct 2020, exposomics version
localDB <- paste0(db_dir,"PubChemLite_31Oct2020_exposomics.csv")
file.exists(localDB)
cmpd_file <- paste0(base_dir, "PCLite_Benchmark_wMSMS_info.csv")
file.exists(cmpd_file)
# set up results
run_name <- "exposomics_31Oct2020_MergedBenchmark"
pre2020 <- FALSE
useMSMS <- TRUE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# run MetFrag evaluation
### if MetFrag has not already run, choose this: 
### note this takes about an hour or more
eval_file <- runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                              base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE)
### If MetFrag has run, do this: 
### note this takes a few minutes (and does nothing in the interim, until it ends)
eval_file <- runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,pre2020,
                              base_dir=base_dir,verbose=TRUE,runMetFrag=FALSE)
eval_file #this just prints the file name

cmpd_info <- read.csv(eval_file,stringsAsFactors = F)
# now you can check some metrics ... e.g. that columns are full ... 
IKFB_r1 <- length(which(cmpd_info$IKeyFBlock_rank==1))
n_entries <- length(cmpd_info$IKeyFBlock_rank)
IKFB_r1/n_entries 
#this is currently ~81 % and should stay relatively consistent. 
# Flag things if this deviates by +/- 2 %? 5 %?

IKFB_fail <- length(which(is.na(cmpd_info$IKeyFBlock_rank)))
IKFB_fail
IKFB_fail/n_entries
# number failing (not present in PubChemLite) is currently 15. 
# this number shouldn't increase, (+/- 1 may be OK), it could/should decrease. 

# these are actually the main two checks. Do we need more???
# in answer to that question ... yes perhaps and here's the update from 11 Jan
IKFB_r1 <- length(which(cmpd_info$IKeyFBlock_rank==1))
n_entries <- length(cmpd_info$IKeyFBlock_rank)
IKFB_r1/n_entries #~~0.812
#this is currently ~81 % and should stay relatively consistent. 
# Flag things if this deviates by +/- 2 %? 5 %?

IKFB_rle2 <- length(which(cmpd_info$IKeyFBlock_rank<=2))
IKFB_rle2/n_entries #~~0.931

IKFB_rle5 <- length(which(cmpd_info$IKeyFBlock_rank<=5))
IKFB_rle5/n_entries #~~0.975 

IKFB_numeric <- length(which(cmpd_info$IKeyFBlock_rank>0))
IKFB_numeric #~~962

IKFB_fail <- length(which(is.na(cmpd_info$IKeyFBlock_rank)))
IKFB_fail #~~15

n_entries # should always equal 977!


# Below is just a brief functionality check .. which should also run
# this checks that the "bad element" removal is working


# test using molecular formula

run_name <- "exposomics_31Oct2020_MergedBenchmark_MolForm"
pre2020 <- FALSE
useMSMS <- TRUE
useExactMass <- FALSE

cmpd_info <- read.csv(cmpd_file,stringsAsFactors = F)
cmpd_info <- user_mk_cmpd_info(cmpd_info,skipMSMS = T)

# # to just test, run this, otherwise comment out the subsetting: 
# for mol form, this includes previous problematic entries ... 
cmpd_info <- cmpd_info[c(1:5,835:837),]
# run MetFrag evaluation
eval_file2 <- runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,
                               pre2020,base_dir=base_dir,verbose=TRUE,runMetFrag=TRUE,
                               useExactMass=useExactMass)
# If MetFrag has already run, use this
eval_file2 <- runPCLiteEval.BM(cmpd_info,localDB,run_name,metfrag_dir,useMSMS,
                               pre2020,base_dir=base_dir,verbose=TRUE,runMetFrag=FALSE,
                               useExactMass=useExactMass)
eval_file2


cmpd_info <- read.csv(eval_file2,stringsAsFactors = F)
# now you can check some metrics ... 
IKFB_r1 <- length(which(cmpd_info$IKeyFBlock_rank==1))
# should equal 8


IKFB_fail <- length(which(is.na(cmpd_info$IKeyFBlock_rank)))
IKFB_fail
# should equal zero. 
