## This file contains functions and class definitions which define the
## process of configuring the evaluation run based on the input
## parameters. Currently, the input parameters can be provided using
## environment variables. See the envconfig function for more details.


## Define configuration structure
setClass(Class = "envconfig",
         representation = list(
             ## The repository.
             dir_src = "character",
             ## Path to metfrag jar.
             fn_metfrag_jar = "character",
             ## Working directory.
             dir_wrk = "character",
             ## MetFrag DB path.
             fn_metfrag_db = "character",
             ## The Zenodo repository of the benchmark file.
             url_zenodo_bench = "character",
             ## Local destination file name relative to dir_wrk.
             fn_zenodo_bench = "character",
             ## Which scripts to source.
             fn_source = "character",
             ## MetFrag run name.
             run_name = "character",
             ## Futures plan.
             future_plan = "character",
             ## Number of workers.
             future_nworkers = "integer",
             ## Java binary
             fn_java_bin = "character",
             ## Chronological evaluation results tracking happens in a
             ## CSV with fn_track_pref prefix.
             fn_track_pref = "character",
             ## The tracking file resides in dir_track.
             dir_track = "character",
             ## Data/time of the CSV creation.
             time = "POSIXct"), 
         prototype =  list(dir_src = NA_character_,
                           fn_metfrag_jar = NA_character_,
                           dir_wrk = NA_character_,
                           dir_track = NA_character_,
                           fn_metfrag_db = NA_character_,
                           url_zenodo_bench = NA_character_,
                           fn_zenodo_bench = NA_character_,
                           fn_source = c(
                               "pubchemlite/R/eval4pub/lib.R",
                               "pubchemlite/R/eval4pub/mfprops.R",
                               "pubchemlite/R/eval4pub/mfconfig.R",
                               "pubchemlite/R/eval4pub/result.R",
                               "pubchemlite/R/user_PCLite_eval.R",
                               "pubchemlite/R/PCLite_eval_support.R",
                               "annotations/tps/extractAnnotations.R"),
                           run_name = NA_character_,
                           future_plan = NA_character_,
                           fn_track_pref = "pcl_quality_tracker",
                           future_nworkers = NA_integer_,
                           fn_java_bin = "java"),
         sealed = F)




## The main function. Creates an envconfig object based on the
## environment variables.
envconfig <- function() {
    dir_src = Sys.getenv("PCL_DIRSRC")
    if (!dir.exists(dir_src)) {
        stop("The PCL_DIRSRC ( ",dir_src," ) path does not exist. Aborting.")
        dir.create(path=dir_src,recursive = T)
        
    }
    fn_metfrag_jar = Sys.getenv("PCL_FN_METFRAG_JAR")
    if (!file.exists(fn_metfrag_jar)) stop("Path to metfrag binary PCL_FN_METFRAG_JAR (",
                                           fn_metfrag_jar,
                                           ") ",
                                           "does not exist.")
    dir_wrk = Sys.getenv("PCL_DIRWRK")
    if (!dir.exists(dir_wrk)) {
        warning("The PCL_DIRWRK path does not exist. Creating: ", dir_wrk)
        dir.create(path=dir_wrk,recursive = T)
        
    }

    fn_metfrag_db = Sys.getenv("PCL_FN_METFRAG_DB")
    if (!file.exists(fn_metfrag_db)) stop("Path to metfrag DB file PCL_FN_METFRAG_DB (",
                                          fn_metfrag_db,
                                          ") ",
                                          "does not exist.")

    url_zenodo_bench = Sys.getenv("PCL_URL_ZENODO_BENCH")
    if (nchar(url_zenodo_bench)==0) stop("Url to Zenodo benchmark PCL_URL_ZENODO_BENCH undefined.")

    fn_zenodo_bench = Sys.getenv("PCL_FN_ZENODO_BENCH")
    if (nchar(fn_zenodo_bench)==0) stop("Destination filename for Zenodo benchmark PCL_FN_ZENODO_BENCH undefined.")
    fn_zenodo_bench <- file.path(dir_wrk,fn_zenodo_bench)

    run_name = Sys.getenv("PCL_RUN_NAME")
    if (nchar(run_name)==0) stop("Url to Zenodo benchmark PCL_RUN_NAME undefined.")
    
    future_plan <- Sys.getenv("PCL_FUTURE_PLAN")
    if (nchar(future_plan)==0) {
        warning("Undefined PCL_FUTURE_PLAN. Setting sequential")
        future_plan <- "sequential"
        future_nworkers <- NA_integer_
    }

    if (future_plan!="sequential") {
        future_nworkers <- Sys.getenv("PCL_FUTURE_NWORKERS")
        if (nchar(future_nworkers)==0) {
            stop("Undefined PCL_FUTURE_NWORKERS. Abort.")
            
        }
        future_nworkers <- as.integer(future_nworkers)
    } else future_nworkers <- NA_integer_

    fn_java_bin <- Sys.getenv("PCL_FN_JAVA_BIN")
    if (nchar(fn_java_bin)==0) {
        warning("Undefined PCL_FN_JAVA_BIN. Default is `java'")
        fn_java_bin <- "java"
    }

    dir_track <- Sys.getenv("PCL_DIR_TRACK")
    if (nchar(dir_track)==0) {
        stop("Undefined PCL_DIR_TRACK, the directory containing the quality tracker files.")
        
    }

    t <- Sys.getenv("PCL_TIME")
    if (nchar(t)==0) t <- Sys.time()
    new("envconfig",
        dir_src = dir_src,
        fn_metfrag_jar = fn_metfrag_jar,
        dir_wrk = dir_wrk,
        fn_metfrag_db = fn_metfrag_db,
        url_zenodo_bench = url_zenodo_bench,
        fn_zenodo_bench = fn_zenodo_bench,
        run_name = run_name,
        future_plan = future_plan,
        future_nworkers = future_nworkers,
        fn_java_bin = fn_java_bin,
        dir_track = dir_track,
        time = as.POSIXct(t))

    
}

basedir <- function(x) {
    normalizePath(x@dir_wrk)
}


download_zenodo_bench <- function(x) {
    url_loc <- x@url_zenodo_bench
    fn_loc <- x@fn_zenodo_bench
    download.file(url_loc,fn_loc)
}

## This function takes an envconfig object and sources what needs to
## be sourced.
source_r_files <- function (x) {
    r_source <- file.path(x@dir_src,x@fn_source)

    for (fn in r_source) {
        message("Sourcing: ", fn)
        source(fn)
    }
    
}

## Load the compound info table, then process it using
## user_mk_cmpd_info.
get_cmpd_info <- function(x) {
    
    cmpd_info <- read.csv(x@fn_zenodo_bench,stringsAsFactors = F)
    user_mk_cmpd_info(cmpd_info,skipMSMS = T)
}

## Set up future parallel backend. 
plan <- function(x) {
    if (x@future_plan != "sequential") {
        logmsg("Planning future ",x@future_plan," for ", x@future_nworkers, " workers.")
        future::plan(x@future_plan,
                     workers = x@future_nworkers)
    } else {
        logmsg("Planning sequential future.")
        future::plan("sequential")
    }
}


## Few helpers.
dir_topres <- function(x) file.path(x@dir_wrk,x@run_name)
dir_mfconf <- function(x) file.path(dir_topres(x),"x")
dir_mflog <- function(x) file.path(dir_topres(x),"log")
dir_mfres <- function(x) file.path(dir_topres(x),"results")
dir_mfspec <- function(x) file.path(dir_topres(x),"spec")

get_fn_track <- function(x,tag) {
    file.path(x@dir_track,paste0(x@fn_track_pref,"_",tag,".csv"))
}

## Create all the required paths.
make_paths <- function(x) {
    dir_res <-  dir_topres(x)
    dir.create(dir_res, recursive = T, showWarnings = F)
    dir.create(dir_mfconf(x), showWarnings = F)
    dir.create(dir_mflog(x), showWarnings = F)
    dir.create(dir_mfres(x), showWarnings = F)
    dir.create(dir_mfspec(x), showWarnings = F)
    logmsg("MetFrag results, configs and logs are in: ", dir_res)

}

summary_fn <- function(x) file.path(x@dir_wrk,
                                         paste0("MetFragResultSummaryPubChemLite_",
                                                x@run_name,".csv"))

## Prepare for evaluation by logging the envconfig object, downloading
## what is needed and planning the future.
eval_prep <- function(x) {
    logme(x)
    
    ## Get the PCLite Benchmarking file needed.
    download_zenodo_bench(x)

    plan(x)
    list(envconfig=x,
         cmpd_info=get_cmpd_info(x))
}
